/**
 * This class contain the algorithm implementation for point/line topological predicates
 * \author Reasey Praing
 * \version 0.1
 * \version 0.2
 * \date	2006-11-08
 * \note for mslob
 */
#ifndef ALGORITHM_PL_H
#define ALGORITHM_PL_H

#include <iostream>
#include "point2D.h"
#include "line2D.h"
#include "poi2D.h"
#include "hseg2D.h"
#include "statusHseg2D.h"
#include "EventSchedule.h"
#include "SweepLine.h"
#include "MatchPred.h"
#include "dvector.h"

namespace stu
{
using namespace stu;

class Algorithm_pl
{
private:
	bool *vf, *vg;
	int m, n;
public:
	bool debug;
	Algorithm_pl() : m(3), n(1) 
	{
		debug = false;
	}
	Algorithm_pl(bool dg) : debug(dg)
	{
		m = 3;
		n = 1;
	}
	
	int topPred(point2D& p, line2D& l, int verify)  //verify = -1 for determination; >0 for verification; return id for determination and true answer; return -1 for false answer
	{
		//cout << "<start>";
		debug = false;
		whichObj object;
		whatStatus status;
		initializeFlags();
		//cout << "<start1>";
		dvector<poi2D> dyn1;
		dvector<hseg2D> dyn2;
		EventSchedule<point2D, line2D, poi2D, hseg2D> es(p, l,dyn1,dyn2);
		//cout << "<start2>";
		//***********Begin Algorithm************
		
		//Create sweepline object
		SweepLine<point2D, line2D, poi2D, hseg2D> S;
		//cout << "<start3>";
		poi2D last_dp;
		statusHseg2D sh;
		es.select_first(object, status);
		//cout << "<start4>" << (status) <<  allTrue() << p.getElements().size() << l.getElements().size() <<"\n";
		while(status != end_of_second && status != end_of_both && !allTrue())	//>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>MODIFIED 4-20-06
		{
		//	cout << "<start5>";
			if(object == first)
			{
				if(debug) cout << "<first>: "<<*(es.i1)<<endl;
				if(S.poi_in_seg( *(es.i1), 'F'))
				{	
				//	cout << " f1 ";
					vf[1] = true; //if(debug) cout<<"1 ";
				}
				else
				{
				//	cout << " f2 ";
					vf[0] = true; //if(debug) cout<<"2 ";
				}
			}
			else if(object == second)
			{
				if(debug) cout << "<second>: "<<*(es.i2)<<endl;
				sh = statusHseg2D(*(es.i2), 'G');
				if(sh.hseg->isLeft())
					S.add_left(sh, es);
				else
					S.del_right(sh, es);
				if(last_dp.isUndefined()||(!last_dp.isUndefined() && sh.hseg->getDominating() != last_dp))
				{
				//	cout << " s1 ";
					last_dp = sh.hseg->getDominating();
					if(!es.look_aheadG(es.i2))
					{	vg[0] = true; //if(debug) cout<<"3 ";
					}
				}
				last_dp = sh.hseg->getDominating();
			}
			else //object = both
			{
				if(debug) cout << "<both>: "<<*(es.i1) << " == " <<*(es.i2) <<endl;
				sh = statusHseg2D(*(es.i2), 'G');
				if(sh.hseg->isLeft())
					S.add_left(sh, es);
				else
					S.del_right(sh, es);
				last_dp = sh.hseg->getDominating();
				if(es.look_aheadG(es.i2))
				{
				//	cout << " b1 ";
					vf[1] = true; //if(debug) cout<<"4 ";
				}
				else
				{
				//	cout << "here\n";
					vf[2] = true; //if(debug) cout<<"5 ";
				}
			}
			es.select_next(object, status);
		}
//		if(status == end_of_first)						//>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>MODIFIED 4-20-06
//			vg[0] = true;
		if(status == end_of_second)
		{	vf[0] = true; //if(debug) cout<<"6 ";
		}
		//***********End Algorithm************
		if(debug) cout<<endl;
		//Debug info
		if(debug)
		{
			printFlags();
		}

		//Match Predicate
		//std::cout<<" ******1******* \n";
		MatchPred mp(vf, vg, m, n, _point2D, _line2D);
		//std::cout<<" ******1.5******* \n";
		int z = mp.matchpl(verify);
		//std::cout<<" ******"<<z<<"******* \n";
		return z;
	}
	void initializeFlags()
	{
		vf = new bool[m];
		vg = new bool[n];
		for(int i = 0; i < m; i++)
			vf[i] = false;
		for(int i = 0; i < n; i++)
			vg[i] = false;
	}
	void printFlags()
	{
		cout << "Flags: ";
		for(int i = 0; i < m; i++)
			cout << "vf[" << i << "] = " << vf[i] << ",   ";
		for(int i = 0; i < n; i++)
			cout << "vg[" << i << "] = " << vg[i] << ",   ";
		cout << endl;
	}
	bool allTrue()
	{
		bool res = true;
		for(int i = 0; i < m; i++)
			res = res && vf[i];
		for(int i = 0; i < n; i++)
			res = res && vg[i];
		return res;
	}
};
}
#endif
