/**
 * This class contain the algorithm implementation for point/region topological predicates
 * \author Reasey Praing
 * \version 0.1
 * \version 0.2
 * \date	2006-11-09
 * \note for mslob
 */
#ifndef ALGORITHM_PR_H
#define ALGORITHM_PR_H

#include <iostream>
#include "point2D.h"
#include "region2D.h"
#include "poi2D.h"
#include "hseg2D.h"
#include "statusHseg2D.h"
#include "EventSchedule.h"
#include "SweepLine.h"
#include "MatchPred.h"
#include "dvector.h"



namespace stu
{
using namespace stu;

class Algorithm_pr
{
private:
	bool *vf, *vg;
	int m, n;
public:
	bool debug;
	Algorithm_pr() : m(3), n(0) 
	{
		debug = false;
	}
	Algorithm_pr(bool dg) : debug(dg)
	{
		m = 3;
		n = 0;
	}
	
	int topPred(point2D& p, region2D& r, int verify)  //verify = -1 for determination; >0 for verification; return id for determination and true answer; return -1 for false answer
	{
		debug = false;
		whichObj object;
		whatStatus status;
		initializeFlags();
		dvector<poi2D> dyn1;
		dvector<hseg2D> dyn2;
		EventSchedule<point2D, region2D, poi2D, hseg2D> es(p, r,dyn1,dyn2);
		
		//***********Begin Algorithm************
		
		//Create sweepline object
		//std::cout<<p.getElements().size()<<r.getElements().size()<<endl;
		SweepLine<point2D, region2D, poi2D, hseg2D> S;
		es.select_first(object, status);
		while(status == end_of_none && !allTrue())
		{
			if(object == first)
				if(S.poi_on_seg( *(es.i1), 'F'))
				{	vf[1] = true; if(debug) cout<<"1 ";}
				else if(S.poi_in_region(*(es.i1)))
				{	vf[0] = true; if(debug) cout<<"2 ";}
				else
				{	vf[2] = true; if(debug) cout<<"3 ";}
			else
			{
				//cout<<"isUndefined: " << es.i2->getDominating().isUndefined() << es.i2->getSubmissive().isUndefined() << endl;
				statusHseg2D sh(*(es.i2), 'G');
				if(es.i2->isLeft())
					S.add_left(sh, es);
				else
					S.del_right(sh, es);
				if(object == both)
				{	vf[1] = true; if(debug) cout<<"4 ";}
			}
			es.select_next(object, status);
		}
		if(status == end_of_second)
		{	vf[2] = true; if(debug) cout<<"5 ";}
		//***********End Algorithm************
		//Debug info
		if(debug) cout<<endl;
		if(debug)
		{
			printFlags();
		}

		//Match Predicate
		MatchPred mp(vf, vg, m, n, _point2D, _region2D);
		return mp.matchpr(verify);
	}
	void initializeFlags()
	{
		vf = new bool[m];
		vg = new bool[n];
		for(int i = 0; i < m; i++)
			vf[i] = false;
		for(int i = 0; i < n; i++)
			vg[i] = false;
	}
	void printFlags()
	{
		cout << "Flags: ";
		for(int i = 0; i < m; i++)
			cout << "vf[" << i << "] = " << vf[i] << ",   ";
		for(int i = 0; i < n; i++)
			cout << "vg[" << i << "] = " << vg[i] << ",   ";
		cout << endl;
	}
	bool allTrue()
	{
		bool res = true;
		for(int i = 0; i < m; i++)
			res = res && vf[i];
		for(int i = 0; i < n; i++)
			res = res && vg[i];
		return res;
	}
};
}
#endif
