/**
 * This class contain the algorithm implementation for region/region topological predicates
 * \author Reasey Praing
 * \version 0.1
 * \version 0.2
 * \date	2006-11-09
 * \note for mslob
 */
#ifndef ALGORITHM_RR_H
#define ALGORITHM_RR_H

#include <iostream>
#include "region2D.h"
#include "poi2D.h"
#include "hseg2D.h"
#include "statusHseg2D.h"
#include "EventSchedule.h"
#include "SweepLine.h"
#include "MatchPred.h"
#include "dvector.h"
#include "vectorBinarySearchWrapper.h"

namespace stu
{
using namespace stu;

class Algorithm_rr
{
private:
	bool *vf, *vg;
	int m, n;
public:
	bool debug;
	Algorithm_rr() : m(7), n(8) 
	{
		debug = false;
	}
	Algorithm_rr(bool dg) : debug(dg)
	{
		m = 7;
		n = 8;
	}
	
	void operate(region2D& r1, region2D& r2, int operation, dvector<hseg2D>& result){
		topPred(r1,r2,-1,operation,&result);
	}
	 
	bool checkOperate(int operation, int ms, int ns, char owner, hseg2D& t){
		if (operation==UNION)
		{
			if (((ms==1 || ms==2) && ns==0) || ((ns==1 || ns==2) && ms==0)){
				if ((ms==1 || ms==2) && ns==0)
					t.setInsideAbove(false);
				else
					t.setInsideAbove(true);
				return true;
			}else
				return false;
		}
		else if (operation==INTERSECTION)
		{
			if (((ms==1 || ms==0) && ns==2) || ((ns==1 || ns==0) && ms==2)){
				if ((ms==1 || ms==0) && ns==2)
					t.setInsideAbove(true);
				else
					t.setInsideAbove(false);
				return true;
			}else
				return false;		
		}
		else if (operation==DIFFERENCE)
		{
			if (owner=='F')
			{
				if ((ms==0 && ns==1) || (ns==0 && ms==1))
				{
					if (ms==0 && ns==1)
						t.setInsideAbove(true);
					else
						t.setInsideAbove(false);
					
					return true;
				}else
					return false;
			}else if (owner=='G')
			{
				if ((ms==1 && ns==2) || (ns==1 && ms==2))
				{
					if (ms==1 && ns==2)
						t.setInsideAbove(false);
					else
						t.setInsideAbove(true);
					
					return true;
				}else
					return false;			
			}else //owner is B or T
			{
				if (ms==1 && ns==1)
				{
					t.setInsideAbove((owner=='T'));
					return true;
				}else
					return false;
			}
		}else
			throw runtime_error("Invalid Operation Specfied for region2Ds.");
	}
	
	int topPred(region2D& r1, region2D& r2, int verify, int operation=-1, dvector<hseg2D>* result=NULL)  //verify = -1 for determination; >0 for verification; return id for determination and true answer; return -1 for false answer, dvector and operation only used for spatial operations
	{
		debug = false;
		whichObj object;
		whatStatus status;
		initializeFlags();
		dvector<hseg2D> dyn1;
		dvector<hseg2D> dyn2;
		EventSchedule<region2D, region2D, hseg2D, hseg2D> es(r1, r2,dyn1,dyn2);
		
		//***********Begin Algorithm************
		
		//Create sweepline object
		SweepLine<region2D, region2D, hseg2D, hseg2D> S;
		poi2D last_dp_in_F, last_dp_in_G;
		statusHseg2D sh;
//		cout<<"------------------- Algorithm_rr Debug -----------------\n";
		es.select_first(object, status);
		while((status == end_of_none && result==NULL) || (status != end_of_both && result!=NULL) && !allTrue())
		{

			if(object == first)
			{
//				cout<<"First: {"<<es.i1->getDominating()<<", "<<es.i1->getSubmissive()<<") , "<<es.i1->isLeft()<<", "<<es.i1->isInsideAbove()<<"}"<<endl;
				sh = statusHseg2D(*(es.i1), 'F');
				last_dp_in_F = (es.i1)->getDominating();
			}
			else if(object == second)
			{
//				cout<<"Second: {"<<es.i2->getDominating()<<", "<<es.i2->getSubmissive()<<") , "<<es.i2->isLeft()<<", "<<es.i2->isInsideAbove()<<"}"<<endl;
				sh = statusHseg2D(*(es.i2), 'G');
				last_dp_in_G = (es.i2)->getDominating();
			}
			else
			{
//				cout<<"Both: {"<<es.i1->getDominating()<<", "<<es.i1->getSubmissive()<<") , "<<es.i1->isLeft()<<", "<<es.i1->isInsideAbove()<<"}"<<endl;
//				cout<<" {"<<es.i2->getDominating()<<", "<<es.i2->getSubmissive()<<") , "<<es.i2->isLeft()<<", "<<es.i2->isInsideAbove()<<"}"<<endl;
				
				/*modified 11/21/06 to add T value to statusHseg2D owner for use in difference operation to identify where F is. If 
				owner==B, F is on the bottom, else (owner==T), F is on top.*/
				char bt='B';
				
				int mb = 0, nb = 0;			//>>>>>>>>>>>>>>>> Added  6/6/06 to handle right hseg also <<<<<<<<<<<<<<<<<<<<<
				if(es.i1->isInsideAbove())
				{
					nb++;
					if (es.i1->isLeft()) bt='T';
				}
				else{
					mb++;
					if (!es.i1->isLeft()) bt='T';
				}
				
				if(es.i2->isInsideAbove())
				{
					nb++;
				}
				else
					mb++;
				
				sh = statusHseg2D(*(es.i1), bt);
				last_dp_in_F = (es.i1)->getDominating();
				last_dp_in_G = (es.i2)->getDominating();
				
				setFlag(true,mb,nb); if(debug) cout<<" 1";
				setFlag(false,mb,nb);
				
// 				if (result!=NULL)
// 				{
// 					hseg2D copyseg(*(sh.hseg));
// 					if (checkOperate(operation, mb, nb, sh.owner, copyseg))
// 						result->push_back(copyseg);
// 				}
			}
			if((!last_dp_in_F.isUndefined() && !last_dp_in_G.isUndefined() && last_dp_in_F == last_dp_in_G))  //>>>>>>>>>>>>>>>> Splitted 5/12/06 <<<<<<<<<<<<<<<<<<<<<
			{	vg[7] = true; if(debug) cout<<" 2";}
			else if((object == first || object == both) && es.look_aheadG(es.i1))
			{	vg[7] = true; if(debug) cout<<" 3";}
			else if((object == second || object == both) && es.look_aheadF(es.i2))
			{	vg[7] = true; if(debug) cout<<" 4";}
			if(!sh.hseg->isLeft())
			{

				statusHseg2D left;
				//cerr << "The segment to del: " << sh << endl;
				bool found=S.del_rightReturn(sh, es, left);
				//cerr << "The segment left: " << left << endl;

				//will add to resultset of operation in halfsegment order
				
				if (found){
					if (result!=NULL)
					{
						uint insertIdx;
						vectorBinarySearchWrapper< dvector<hseg2D>, hseg2D > finder( result );
						hseg2D copyseg(*(left.hseg));
						hseg2D brocopy(copyseg.getSubmissive(), copyseg.getDominating(), !copyseg.isInsideAbove());
						if (checkOperate(operation, left.m, left.n, left.owner, copyseg))
						{	
							if (finder.find(copyseg,insertIdx))
								cerr << "Should NOT have found the hseg (Algorithm_rr.h)" << endl;

							result->insert(insertIdx,copyseg);

							if (finder.find(brocopy,insertIdx))
								cerr << "Should NOT have found the brother of the hseg (Algorithm_rr.h)" << endl;

							result->insert(insertIdx,brocopy);

						}
					}
					//Setting flags
					int ms = left.m;
					int ns = left.n;
					if(object == first)
					{	setFlag(true,ms,ns); if(debug) cout<<"["<<ms<<","<<ns<<"] 5";}
					else if(object == second)
					{	setFlag(false,ms,ns); if(debug) cout<<"["<<ms<<","<<ns<<"] 6";}
					else
					{
						setFlag(true,ms,ns); if(debug) cout<<" 7";
						setFlag(false,ms,ns); if(debug) cout<<" 8";
					}
					
				}
//				cout<<"DEL RIGHT\n";
			}
			else
			{
//				cout<<"ADD LEFT\n";
				//cerr << "The segment to ADD: " << sh << endl;
				bool skip = false;
				S.add_left_skip(sh, es, skip);
				if(skip)
				{
					es.select_next(object, status);
					continue;
				}
					
				int mp = 0, np = 0, ms = 0, ns = 0;
				
				//Binary search for sh in status structure and set overlap number for its predecessor
				bool found = false;
				uint index;
				vectorBinarySearchWrapper< dvector<statusHseg2D>, statusHseg2D > binaryFinder( &(S.status) );
				found = binaryFinder.find(sh, index);
				if(S.status.size()>0 && sh == S.status[0])
					np = 0;
				else	if(found)
				{
					if(S.status[index].owner == sh.owner)
					{
						mp = S.status[index-1].n;
						np = S.status[index-1].n;
					}
					if(S.status[index].owner != sh.owner)
					{
						mp = S.status[index].n;
						np = S.status[index].n;
					}
				}
//				else
//					cerr<<"The inserted status halfsegment cannot be located."<<endl;
/*				
				dvector<statusHseg2D>::Iterator i = S.status.getIterator();
				i.next();
				if(!i.end() && *(sh.hseg) == *(i.get()->hseg))
					np = 0;
				else
				{
					for(i.toEnd(); !i.begin();)
					{
						i.prev();
						if(*(i.get()->hseg) == *(sh.hseg))
						{
							break;
						}
					}
					if(i.get()->owner == sh.owner && !i.begin())
					{
						i.prev();
						mp = i.get()->n;
						np = i.get()->n;
					}
					else if(i.get()->owner != sh.owner)
					{
						mp = i.get()->n;
						np = i.get()->n;
					}
				}
*/
				ms = np;
				ns = np;
//				cout<<"ms,ns:"<<ms<<" "<<ns<<endl;
				if(object == first || object == both)
					if(es.i1->isInsideAbove())					//	>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>MODIFIED 5/12/06 use i1 instead of sh<<<<<<<<<<<<<<<<<<<<<
						ns++;
					else
						ns--;
				if(object == second || object == both)
					if(es.i2->isInsideAbove())					//	>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>MODIFIED 5/12/06 use i2 instead of sh<<<<<<<<<<<<<<<<<<<<<
						ns++;
					else
						ns--;

				//Update overlapnumber
				if(found)
				{
					statusHseg2D temp(*(S.status[index].hseg), S.status[index].owner);
					temp.m = ms;
					temp.n = ns;
					S.status.erase(index);
					S.status.insert(index,temp);
				}
				else
					cerr<<"Cannot find the inserted status halfsegment to be updated."<<endl;
				
/*
				dvector<statusHseg2D>::Iterator i = S.status.getIterator();
				for(i.toEnd(); !i.begin();)
				{
					i.prev();
					if(*(i.get()->hseg) == *(sh.hseg) && i.get()->owner == sh.owner)
						break;
				}
				statusHseg2D temp(*(i.get()->hseg),i.get()->owner);
				temp.m = ms;
				temp.n = ns;
				S.status.erase(i);
				S.status.insert(i,temp);
*/
				//				cout<<"ms: "<<ms<<" ns: "<<ns<< endl;
// 				if (result!=NULL)
// 				{
// 					hseg2D copyseg(*(temp.hseg));
// 					if (checkOperate(operation, ms, ns, temp.owner, copyseg))
// 						result->push_back(copyseg);
// 				}

/*
				//Setting flags
				if(object == first)
				{	setFlag(true,ms,ns); if(debug) cout<<"["<<ms<<","<<ns<<"] 5";}
				else if(object == second)
				{	setFlag(false,ms,ns); if(debug) cout<<"["<<ms<<","<<ns<<"] 6";}
				else
				{
					setFlag(true,ms,ns); if(debug) cout<<" 7";
					setFlag(false,ms,ns); if(debug) cout<<" 8";
				}
*/
			}
/*			//Debug printing
			if(!last_dp_in_F.isUndefined())
				cout<<"last_dp_in_F: "<<last_dp_in_F<<endl;
			if(!last_dp_in_G.isUndefined())
				cout<<"last_dp_in_G: "<<last_dp_in_G<<endl;
			printFlags();
*/
			es.select_next(object, status);
		}
		if(status == end_of_first)
		{
			vg[0] = true; if(debug) cout<<" 9";
			vg[2] = true; if(debug) cout<<" 10";
		}
		else if(status == end_of_second)
		{
			vf[0] = true; if(debug) cout<<" 11";
			vf[2] = true; if(debug) cout<<" 12";
		}

		//***********End Algorithm************
		//Test
//		vf[0] = true; vf[2] = true; vf[4] = true;vf[6] = true; 
//		vg[0] = true; vg[2] = true; vg[4] = true;vg[6] = true; vg[7]=true;
		//Debug info
		if(debug) cout<<endl;
		if(debug)
		{
			printFlags();
		}

		//Match Predicate
		MatchPred mp(vf, vg, m, n, _region2D, _region2D);
		return mp.matchrr(verify);
	}	
	void initializeFlags()
	{
		vf = new bool[m];
		vg = new bool[n];
		for(int i = 0; i < m; i++)
			vf[i] = false;
		for(int i = 0; i < n; i++)
			vg[i] = false;
	}
	void printFlags()
	{
		cout << "Flags: ";
		for(int i = 0; i < m; i++)
			cout << "vf[" << i << "] = " << vf[i] << ",   ";
		for(int i = 0; i < n; i++)
			cout << "vg[" << i << "] = " << vg[i] << ",   ";
		cout << endl;
	}
	bool allTrue()
	{
		bool res = true;
		for(int i = 0; i < m; i++)
			res = res && vf[i];
		for(int i = 0; i < n; i++)
			res = res && vg[i];
		return res;
	}
	void setFlag(bool isF, int m, int n)
	{
		// {F0/1, F0/2, F1/0, F1/1, F1/2, F2/0, F2/1}
		//{G0/1, G0/2, G1/0, G1/1, G1/2, G2/0, G2/1, point_in_common}
		if(isF)
			vf[(m*3 + n - 1)] = true;    //Flatten Formula
		else
			vg[(m*3 + n - 1)] = true;
	}
};
}
#endif
