/** This class is for parallel object traversal
 *  \author Reasey Praing, Mark McKenney
 *  \version 0.1
 *  \date    2005-10-05
 *  \version 0.2
 *  \date    2005-10-14 
 *  \version 0.4 fixing for mSlob
 *  \date    2006-10-18 -> 24
 */

#ifndef EVENTSCHEDULE_H
#define EVENTSCHEDULE_H

#include "poi2D.h"
#include "hseg2D.h"
#include "point2D.h"

//#include "line2D.h"
//#include "region2D.h"

#include "dvector.h"
#include "enumTypes.h"

namespace stu
{
using namespace stu;

template<class T1, class T2, class T1i, class T2i> //T1 and T2 are the respective object types. T1i and T2i are the sub-object types
class EventSchedule
{
public:
	//T1 *F; T2 *G;
// 	typename std::vector<T1>::const_iterator i1, ssi1, ssi1b, ssi1e;
// 	typename std::vector<T2>::const_iterator i2, ssi2, ssi2b, ssi2e;
// 	typename std::vector<T1>::iterator dsi1;
// 	typename std::vector<T2>::iterator dsi2;
	
	bool active1, active2;	//active1 = true -> ss1, otherwise ds1; similarly for active2
	T1i *i1;
	T2i *i2;
	typename T1::Iterator ssi1;
	typename T2::Iterator ssi2;
	typename stu::dvector<T1i>::Iterator dsi1;
	typename stu::dvector<T2i>::Iterator dsi2;
	
	//Dynamic Structures
//	typename std::vector<T1> ds1, staticVec1;
//	typename std::vector<T2> ds2, staticVec2;
	
	T1 *ss1;
	T2 *ss2;
	typename stu::dvector<T1i> *ds1;
	typename stu::dvector<T2i> *ds2;	
	
	EventSchedule();
	EventSchedule( T1 &obj1,  T2 &obj2, stu::dvector<T1i> &dyn1, stu::dvector<T2i> &dyn2);
	//~EventSchedule();
	void select_first(whichObj& object, whatStatus& status);
	void select_next(whichObj& object, whatStatus& status);
	void select(whichObj& object, whatStatus& status);
	void advance1();
	void advance2();
	bool look_aheadF(T1i *h, T1i& next); //returns true if found element and put in next, used in face validation
	bool look_aheadF(T1i *h);
	bool look_aheadG(T2i *h);
	void insertF(const T1i h);
	void insertG(T2i h);
};
			
	template<class T1, class T2, class T1i, class T2i> EventSchedule<T1, T2, T1i, T2i>::EventSchedule()
	{}
	//template<class T1, class T2> EventSchedule<T1, T2>::~EventSchedule()
	//{}
	template<class T1, class T2, class T1i, class T2i> EventSchedule<T1, T2, T1i, T2i>::EventSchedule(T1 &obj1, T2 &obj2, dvector<T1i> &dyn1, dvector<T2i> &dyn2)
	{
		ss1 = &obj1;
		ss2 = &obj2;
		ds1 = &dyn1;
		ds2 = &dyn2;
		active1 = true;
		active2 = true;
	//	ssi1b = obj1.getIterator();
	//	ssi1e = staticVec1.end();
	//	ssi2b = staticVec2.begin();
	//	ssi2e = staticVec2.end();

	}
	template<class T1, class T2, class T1i, class T2i> void EventSchedule<T1, T2, T1i, T2i>::select_first(whichObj& object, whatStatus& status)
	{
		ssi1 = ss1->getIterator();
		ssi1.next();
		ssi2 = ss2->getIterator();
		ssi2.next();
		dsi1 = ds1->getIterator();
		dsi1.next();
		dsi2 = ds2->getIterator();
		dsi2.next();
		if(!ssi1.end())
		{
			i1 = ssi1.get();
			active1 = true;
		}
		if(!ssi2.end())
		{
			i2 = ssi2.get();
			active2 = true;
		}
		select(object, status);
	}
	template<class T1, class T2, class T1i, class T2i> void EventSchedule<T1, T2, T1i, T2i>::select_next(whichObj& object, whatStatus& status)
	{
		if(status == end_of_first)
			advance2();
		else if(status == end_of_second)
			advance1();
		else if(status == end_of_none)
		{
			if(object == first)
				advance1();
			else if(object == second)
				advance2();
			else
			{
				advance1();
				advance2();
			}
		}
		select(object, status);
	}
	template<class T1, class T2, class T1i, class T2i> void EventSchedule<T1, T2, T1i, T2i>::select(whichObj& object, whatStatus& status)
	{
		if(ssi1.end() && dsi1.end() && ssi2.end() && dsi2.end())
		{
			status = end_of_both;
			object = both;
		}
		else if(ssi1.end() && dsi1.end() && !(ssi2.end() && dsi2.end()))
		{
			status = end_of_first;
			object = second;
		}
		else if(!(ssi1.end() && dsi1.end()) && ssi2.end() && dsi2.end())
		{
			status = end_of_second;
			object = first;
		}
		else
		{
			status = end_of_none;

			if(*i2 > *i1) //Need comparison (<, ==) b/w poi2D and hseg2D
				object = first;
			else if(*i2 == *i1)
				object = both;
			else
				object = second;
		}
	}
	template<class T1, class T2, class T1i, class T2i> void EventSchedule<T1, T2, T1i, T2i>::advance1()
	{
		if(!dsi2.end() && !ssi2.end() && *(dsi2.get()) < *(ssi2.get()))
		{
			i2 = dsi2.get();
			active2 = false;
		}
		if(!dsi1.end() && !active1)
		{
			ds1->erase( dsi1 ); //erase begin
			dsi1 = ds1->getIterator();
			dsi1.next();
		}
		else if(active1 && !ssi1.end())
			ssi1.next();
		if(!dsi1.end() && !ssi1.end() && *(dsi1.get()) < *(ssi1.get()))
		{
			i1 = dsi1.get();
			active1 = false;
		}
		else if(!ssi1.end())
		{
			i1 = ssi1.get();
			active1 = true;
		}
	}
	template<class T1, class T2, class T1i, class T2i> void EventSchedule<T1, T2, T1i, T2i>::advance2()
	{
		if(!dsi1.end() && !ssi1.end() && *(dsi1.get()) < *(ssi1.get()))
		{
			i1 = dsi1.get();
			active1 = false;
		}
		if(!dsi2.end() && !active2)
		{
			ds2->erase( dsi2 ); //erase begin
			dsi2 = ds2->getIterator();
			dsi2.next();
		}
		else if(active2 && !ssi2.end())
			ssi2.next();
		if(!dsi2.end() && !ssi2.end() && *(dsi2.get()) < *(ssi2.get()))
		{
			i2 = dsi2.get();
			active2 = false;
		}
		else if(!ssi2.end())
		{
			i2 = ssi2.get();
			active2 = true;
		}
	}

	template<class T1, class T2, class T1i, class T2i> bool EventSchedule<T1, T2, T1i, T2i>::look_aheadF(T1i *h, T1i& next )
	{
		bool isEnd = false;
		T1i *t_i;
		typename stu::dvector<T1i>::Iterator t_dsi;
		typename T1::Iterator t_ssi;
		t_i = i1; t_dsi = dsi1; t_ssi = ssi1;
		if(!active1)
			t_dsi.next();
		else if(active1 && !ssi1.end())
			t_ssi.next();
		if(!t_dsi.end() && !t_ssi.end() && *(t_dsi.get()) < *(t_ssi.get()))
			t_i = t_dsi.get();
		else
		{
			if(!t_ssi.end())
				t_i = t_ssi.get();
			else
				isEnd = true;
		}
		 
		if(!(ssi1.end() && active1))
		{
			if(!isEnd){
				next=*t_i;
				return true;
			}
			else
				return false;
		}
		else
			return false;
	}

	
	template<class T1, class T2, class T1i, class T2i> bool EventSchedule<T1, T2, T1i, T2i>::look_aheadF(T1i *h)
	{
		bool isEnd = false;
		T1i *t_i;
		typename stu::dvector<T1i>::Iterator t_dsi;
		typename T1::Iterator t_ssi;
		t_i = i1; t_dsi = dsi1; t_ssi = ssi1;
		if(!active1)
			t_dsi.next();
		else if(active1 && !ssi1.end())
			t_ssi.next();
		if(!t_dsi.end() && !t_ssi.end() && *(t_dsi.get()) < *(t_ssi.get()))
			t_i = t_dsi.get();
		else
		{
			if(!t_ssi.end())
				t_i = t_ssi.get();
			else
				isEnd = true;
		}
		if(*h == *i1)
		{
			if(!isEnd && h->testIntersection(*t_i) != SEGMENT) //Taking care of dangling right hseg
			{
				//cout<<"++++++++ "<<(itr->getSegment().seg2DIntersection(t_i->getSegment()).isUndefined())<<endl;
				//cout<<"itr: {"<<itr->getSegment().getP()<<", "<<itr->getSegment().getQ()<<") , "<<itr->isLeft()<<"}"<<endl;
				//cout<<"t_i: {"<<t_i->getSegment().getP()<<", "<<t_i->getSegment().getQ()<<") , "<<t_i->isLeft()<<"}"<<endl;
				return (h->getDominating() == t_i->getDominating());
			}
			else
				return false;
		}
		else if(!(ssi1.end() && active1))
		{
			bool result = (h->getDominating() == i1->getDominating());
			if(!isEnd)
				result = result || (h->getDominating() == t_i->getDominating());
			return result;
		}
		else
			return false;
	}
	
	template<class T1, class T2, class T1i, class T2i> bool EventSchedule<T1, T2, T1i, T2i>::look_aheadG(T2i *h)
	{
		bool isEnd = false;
		T2i *t_i;
		typename stu::dvector<T2i>::Iterator t_dsi;
		typename T2::Iterator t_ssi;
		 t_i = i2; t_dsi = dsi2; t_ssi = ssi2;
		 if(!active2)
			 t_dsi.next();
		 else if(active2 && !ssi2.end())
			 t_ssi.next();
		 if(!t_dsi.end() && !t_ssi.end() && *(t_dsi.get()) < *(t_ssi.get()))
			 t_i = t_dsi.get();
		 else
		 {
			 if(!t_ssi.end())
				 t_i = t_ssi.get();
			 else
				 isEnd = true;
		 }
		 if(*h == *i2)
		{
			if(!isEnd && h->testIntersection(*t_i) != SEGMENT) //Taking care of dangling right hseg
				return (h->getDominating() == t_i->getDominating());
			else
				return false;
		}
		else if(!(ssi2.end() && active2))
		{
			bool result = (h->getDominating() == i2->getDominating());
			if(!isEnd)
				result = result ||(h->getDominating() == t_i->getDominating()) ;
			return result;
		}
		else
			return false;
	}
	
	template<class T1, class T2, class T1i, class T2i> void EventSchedule<T1, T2, T1i, T2i>::insertF(T1i h)
	{
		//hseg2D h1 = *(hseg2D*)(void*)(&h); //maybe not needed
//		cout<<"***** Insert to F: " << "{"<<h1.getDominating()<<", "<<h1.getSubmissive()<<") , "<<h1.isLeft()<<", "<<h1.isInsideAbove()<<"}"<<endl;

		bool found = false, i1Reset = false;
		//typename stu::dvector<T1i>::Iterator tmp = ds1->getIterator();
		if(!active1)
			i1Reset = true;

		//Insert h into dynamic structure of F (using binary search)
		uint index;
		vectorBinarySearchWrapper< dvector<T1i>, T1i > binaryFinder( ds1 );
		binaryFinder.find(h, index);
		ds1->insert(index, h);
		
/*		if(ds1->isEmpty())
		{
			ds1->push_back(h);
			//std::cout<<"h11  "<<(*dsi1)<<"  "<<h<<std::endl;
		}
		else
		{
			//tmp = ds1.begin();
			while(tmp.next())
			{
				if( h < *(tmp.get()) )
				{
					ds1->insert(tmp, h);
//					cout<<"Inserting "<<h<<" into F dyn"<<endl;
					//std::cout<<"h343  "<<*(tmp-1)<<"  "<<h<<std::endl;
					//std::cout<<"h342  "<<(*dsi1)<<"  "<<h<<std::endl;
					
					found = true;
					break;
				}
			}
			if( !found && tmp.end() )
			{
				ds1->push_back(h);
//				cout<<"Inserting "<<h<<" into F dyn"<<endl;
			}
		}
*/
		dsi1 = ds1->getIterator(); dsi1.next();
		if(i1Reset && !dsi1.end())
			i1 = dsi1.get();
		//Debug
/*		typename std::vector<T1>::iterator itr;
		cout<<"***** DS1 Content ******\n";
		for(itr = ds1.begin(); itr != ds1.end(); itr++)
		{
			hseg2D h2 = *(hseg2D*)(void*)&(*itr);
			cout<<"{"<<h2.getDominating()<<", "<<h2.getSubmissive()<<") , "<<h2.isLeft()<<", "<<h2.isInsideAbove()<<"}"<<endl;
		}
		cout<<"*************************\n";
*/
	}
	template<class T1, class T2, class T1i, class T2i> void EventSchedule<T1, T2, T1i, T2i>::insertG(T2i h)	//Binary search will be done later
	{
		//hseg2D h1 = *(hseg2D*)(void*)(&h); //maybe not needed
//		cout<<"***** Insert to F: " << "{"<<h1.getDominating()<<", "<<h1.getSubmissive()<<") , "<<h1.isLeft()<<", "<<h1.isInsideAbove()<<"}"<<endl;

		bool found = false, i2Reset = false;
//		typename stu::dvector<T2i>::Iterator tmp = ds2->getIterator();
		if(!active2)
			i2Reset = true;
		
		//Insert h into dynamic structure of F (using binary search)
		uint index;
		vectorBinarySearchWrapper< dvector<T2i>, T2i > binaryFinder( ds2 );
		binaryFinder.find(h, index);
		ds2->insert(index, h);
		
/*		if(ds2->isEmpty())
		{
			ds2->push_back(h);
			//std::cout<<"h11  "<<(*dsi1)<<"  "<<h<<std::endl;
		}
		else
		{
			//tmp = ds1.begin();
			while( tmp.next() )
			{
				if( h < *(tmp.get()) )
				{
					ds2->insert(tmp, h);
//					cout<<"Inserting "<<h<<" into G dyn"<<endl;
					//std::cout<<"h343  "<<*(tmp-1)<<"  "<<h<<std::endl;
					//std::cout<<"h342  "<<(*dsi1)<<"  "<<h<<std::endl;
					
					found = true;
					break;
				}
			}
			if( !found && tmp.end() )
			{
				ds2->push_back(h);
//				cout<<"Inserting "<<h<<" into G dyn"<<endl;
			}
		}
*/
		dsi2 = ds2->getIterator(); dsi2.next();
		if(i2Reset && !dsi2.end())
			i2 = dsi2.get();
		//Debug
/*		typename std::vector<T1>::iterator itr;
		cout<<"***** DS1 Content ******\n";
		for(itr = ds1.begin(); itr != ds1.end(); itr++)
		{
		hseg2D h2 = *(hseg2D*)(void*)&(*itr);
		cout<<"{"<<h2.getDominating()<<", "<<h2.getSubmissive()<<") , "<<h2.isLeft()<<", "<<h2.isInsideAbove()<<"}"<<endl;
	}
		cout<<"*************************\n";
*/
	}
}
#endif
