#ifndef MATCHPRED_H
#define MATCHPRED_H
#include <iostream>
#include "enumTypes.h"
#include <time.h>

namespace stu
{
using namespace stu;

class Node
{
	public:
		int id;
		bool isLeaf;
		int l;
		int r;
	Node(int _id, bool _isLeaf, int _l, int _r) : id(_id), isLeaf(_isLeaf), l(_l), r(_r)
	{ }
};

class MatchPred
{
private:
	bool *vf;
	bool *vg;
	int m;
	int n;
	sdtType ftype;
	sdtType gtype;
	static const int option = 2;
	/*option: 
	1: Non-interlaced 9IMC determination / Non-interlaced 9IMC verification
	2: Non-interlaced 9IMC + MCDT determination / Non-interlaced 9IMC + MT verification
	3: Interlaced 9IMC determination (early determination) / Interlaced 9IMC verification (early invalidation)
	*/

	bool evaluateNode(bool ov[], bool ch[], int ovlength)
	{
		for(int i = 0; i < ovlength; i++)
			if(ov[i] == true && ch[i] == true)
				return true;
		return false;
	}
public:
	MatchPred(bool *_vf, bool *_vg, int _m, int _n, sdtType _ftype, sdtType _gtype) : vf(_vf), vg(_vg), m(_m), n(_n), ftype(_ftype), gtype(_gtype) { }
	
	int matchpp(int num)
	{
		//Point matrices
		bool matrix[5][9] = {{false,false,true,false,false,false,true,false,true},
			{true,false,false,false,false,false,false,false,true},
			{true,false,false,false,false,false,true,false,true},
			{true,false,true,false,false,false,false,false,true},
			{true,false,true,false,false,false,true,false,true}};
		//Characterize 9-intersection elements for point/point
		bool ppch[8][4] = { {false, true, false, true},
			{false, false, false, false},
			{true, false, false, false},
			{false, false, false, false},
			{false, false, false, false},
			{false, false, false, false},
			{false, false, true, false},
			{false, false, false, false},
		} ;
		//Building the minimum cost decision tree for point/point
		Node pptr[9] = {Node(2, false, 1, 2), Node(6, false, 3, 4), Node(0, false, 5, 6), Node(1, true, 0, 0), Node(2, true, 0, 0),
			Node(0, true, 0, 0), Node(6, false, 7, 8), Node(3, true,0,0), Node(4, true,0,0)
		};
		//Thin matrices
		bool tmatrix[5][9] = {{1,0,0,0,0,0,0,0,0},{0,0,1,0,0,0,1,0,0},{0,0,1,0,0,0,1,0,0},{0,0,1,0,0,0,1,0,0},{1,0,1,0,0,0,1,0,0}};
		
		//Flatten the boolean vectors
		bool ov[m + n];
		int i = 0;
		for(int j = 0; j < m; j++)
		{
			ov[i] = vf[j];
			i++;
		}
		for(int k = 0; k < n; k++)
		{
			ov[i] = vg[k];
			i++;
		}
		i = 0;
		if(option == 1 && num == -1)
		{
			//Basic Determination
			int j = 0;
			bool found = false;
//			clock_t start2 = clock();
//			for(int l = 0; l < 1000000; l++)
//			{
				found = false;
				for(j = 0; j < 5; j++)
				{
					for(int k = 0; k < 8; k++)
					{
						if(evaluateNode(ov, ppch[k], n+m) != matrix[j][k])
							break;
						else if(k == 7)
							found = true;
					}
					if(found)
						break;
				}
//			}
//			double t2 = ((double)clock() - start2) / CLOCKS_PER_SEC;
//			cout << t2 << endl;
			return j + 1;
		}
		else if(option == 1 && num > 0)
		{
			//Basic Verification
//			clock_t start2 = clock();
			bool found = false;
//			for(int l = 0; l < 10000000; l++)
//			{
				found = false;
				for(int k = 0; k < 8; k++)
				{
					if(evaluateNode(ov, ppch[k], n+m) != matrix[num-1][k])
						break;
					else if(k == 7)
						found = true;
				}
//			}
//			double t2 = ((double)clock() - start2) / CLOCKS_PER_SEC;
//			cout << t2 << endl;
			if(found)
				return num;
			else
				return 0;
		}
		else if(option == 2 && num == -1)
		{
			//MCDT
//			clock_t start1 = clock();
//			int tmp = i;
//			for(int l = 0; l < 1000000; l++)
//			{
//				i = tmp;
				while(!pptr[i].isLeaf)
				{
					if(!evaluateNode(ov, ppch[pptr[i].id], n+m))
						i = pptr[i].l;
					else
						i = pptr[i].r;
				}
//			}
//			double t1 = ((double)clock() - start1) / CLOCKS_PER_SEC;
//			cout << t1 << endl;
			return pptr[i].id + 1;
		}
		else if (option == 2 && num > 0)
		{
			//Matrix Thinning
//			clock_t start2 = clock();
			bool found = true;
//			for(int l = 0; l < 10000000; l++)
//			{
				found = true;
				for(int k = 0; k < 8; k++)
				{
					if(tmatrix[num-1][k])
						if(evaluateNode(ov, ppch[k], n+m) != matrix[num-1][k])
						{
							found = false;
							break;
						}
				}
//			}
//			double t2 = ((double)clock() - start2) / CLOCKS_PER_SEC;
//			cout << t2 << endl;
			if(found)
				return num;
			else
				return 0;
		}
		else
		{
			cerr<<"Invalid option and/or verification id\n";
			return -1;
		}
	}
	int matchpl(int num)
	{
		//Point/Line matrices
		bool matrix[14][9] = {{false,false,true,false,false,false,true,false,true},{false,false,true,false,false,false,true,true,true},{false,true,false,false,false,false,true,false,true},
			{false,true,false,false,false,false,true,true,true},{false,true,true,false,false,false,true,false,true},{false,true,true,false,false,false,true,true,true},
			{true,false,false,false,false,false,true,false,true},{true,false,false,false,false,false,true,true,true},{true,false,true,false,false,false,true,false,true},
			{true,false,true,false,false,false,true,true,true},{true,true,false,false,false,false,true,false,true},{true,true,false,false,false,false,true,true,true},
			{true,true,true,false,false,false,true,false,true},{true,true,true,false,false,false,true,true,true}};
		//Characterize 9-intersection elements for point/line
		bool plch[8][4] = { {false, true, false, false},
			{false, false, true, false},
			{true, false, false, false},
			{false, false, false, false},
			{false, false, false, false},
			{false, false, false, false},
			{true, true, true, true},
			{false, false, false, true},
		} ;
		//Building the minimum cost decision tree for point/line
		Node pltr[27] = {Node(0, false, 1, 2), Node(1, false, 3, 4), Node(1, false, 5, 6), Node(7, false, 7, 8), Node(2, false, 9, 10),
			Node(2, false, 11, 12), Node(2, false, 13, 14), Node(0, true, 0, 0), Node(1, true, 0, 0), Node(7, false, 15, 16),
			Node(7, false, 17, 18), Node(7, false, 19, 20), Node(7, false, 21, 22), Node(7, false, 23, 24), Node(7, false, 25, 26),
			Node(2, true, 0, 0), Node(3, true, 0, 0), Node(4, true, 0, 0), Node(5, true, 0, 0), Node(6, true, 0, 0),
			Node(7, true, 0, 0), Node(8, true, 0, 0), Node(9, true, 0, 0), Node(10, true, 0, 0), Node(11, true, 0, 0),
			Node(12, true, 0, 0), Node(13, true, 0, 0)
		};
		//Thin matrices
		bool tmatrix[14][9] = {{1,1,0,0,0,0,0,1,0},{1,1,0,0,0,0,0,1,0},{1,0,1,0,0,0,0,1,0},{1,0,1,0,0,0,0,1,0},{1,1,1,0,0,0,0,1,0},
			{1,1,1,0,0,0,0,1,0},{0,1,1,0,0,0,0,1,0},{0,1,1,0,0,0,0,1,0},{1,1,1,0,0,0,0,1,0},{1,1,1,0,0,0,0,1,0},
			{1,1,1,0,0,0,0,1,0},{1,1,1,0,0,0,0,1,0},{1,1,1,0,0,0,0,1,0},{1,1,1,0,0,0,0,1,0}};
		//Flatten the boolean vectors
		bool ov[m + n];
		int i = 0;
		for(int j = 0; j < m; j++)
		{
			ov[i] = vf[j];
			i++;
		}
		for(int k = 0; k < n; k++)
		{
			ov[i] = vg[k];
			i++;
		}
		i = 0;
		if(option == 1 && num == -1)
		{
			//Basic Determination
			int j = 0;
			bool found = false;
//			clock_t start2 = clock();
//			for(int l = 0; l < 1000000; l++)
//			{
				found = false;
				for(j = 0; j < 14; j++)
				{
					for(int k = 0; k < 8; k++)
					{
						if(evaluateNode(ov, plch[k], n+m) != matrix[j][k])
							break;
						else if(k == 7)
							found = true;
					}
					if(found)
						break;
				}
//			}
//			double t2 = ((double)clock() - start2) / CLOCKS_PER_SEC;
//			cout << t2 << endl;
			return j + 1;
		}
		else if(option == 1 && num > 0)
		{
			//Basic Verification
//			clock_t start2 = clock();
			bool found = false;
//			for(int l = 0; l < 10000000; l++)
//			{
				found = false;
				for(int k = 0; k < 8; k++)
				{
					if(evaluateNode(ov, plch[k], n+m) != matrix[num-1][k])
						break;
					else if(k == 7)
						found = true;
				}
//			}
//			double t2 = ((double)clock() - start2) / CLOCKS_PER_SEC;
//			cout << t2 << endl;
			if(found)
				return num;
			else
				return 0;
		}
		else if(option == 2 && num == -1)
		{
			//MCDT
//			clock_t start1 = clock();
//			int tmp = i;
//			for(int l = 0; l < 1000000; l++)
//			{
//				i = tmp;
				while(!pltr[i].isLeaf)
				{
					if(!evaluateNode(ov, plch[pltr[i].id], n+m))
						i = pltr[i].l;
					else
						i = pltr[i].r;
				}
//			}
//			double t1 = ((double)clock() - start1) / CLOCKS_PER_SEC;
//			cout << t1 << endl;
			return pltr[i].id + 1;
		}
		else if (option == 2 && num > 0)
		{
			//Matrix Thinning
//			clock_t start2 = clock();
			bool found = true;
//			for(int l = 0; l < 10000000; l++)
//			{
				found = true;
				for(int k = 0; k < 8; k++)
				{
					if(tmatrix[num-1][k])
						if(evaluateNode(ov, plch[k], n+m) != matrix[num-1][k])
					{
						found = false;
						break;
					}
				}
//			}
//			double t2 = ((double)clock() - start2) / CLOCKS_PER_SEC;
//			cout << t2 << endl;
			if(found)
				return num;
			else
				return 0;
		}
	}
	
	int matchpr(int num)
	{
		//Point/Region matrices
		bool matrix[7][9] = { {0,0,1,0,0,0,1,1,1},{0,1,0,0,0,0,1,1,1},{0,1,1,0,0,0,1,1,1},{1,0,0,0,0,0,1,1,1},
			{1,0,1,0,0,0,1,1,1},{1,1,0,0,0,0,1,1,1},{1,1,1,0,0,0,1,1,1} };
		//Characterize 9-intersection elements for point/region
		bool prch[8][4] = { {true, false, false, false},
			{false, true, false, false},
			{false, false, true, false},
			{false, false, false, false},
			{false, false, false, false},
			{false, false, false, false},
			{true, true, true, true},
			{true, true, true, true},
		} ;
		//Building the minimum cost decision tree for point/region
		Node prtr[13] = { Node(0, false, 1, 2), Node(1, false, 3, 4), Node(1, false, 5, 6), Node(0, true, 0, 0), Node(2, false, 7, 8),
			Node(2, false, 9, 10), Node(2, false, 11, 12), Node(1, true, 0, 0), Node(2, true, 0, 0), Node(3, true, 0, 0),
			Node(4, true, 0, 0), Node(5, true, 0, 0), Node(6, true, 0, 0)
		};
		//Thin matrices
		bool tmatrix[7][9] = {{1,1,0,0,0,0,0,0,0},{1,0,1,0,0,0,0,0,0},{1,1,1,0,0,0,0,0,0},{0,1,1,0,0,0,0,0,0},{1,1,1,0,0,0,0,0,0},
			{1,1,1,0,0,0,0,0,0},{1,1,1,0,0,0,0,0,0}};
		//Flatten the boolean vectors
		bool ov[m + n];
		int i = 0;
		for(int j = 0; j < m; j++)
		{
			ov[i] = vf[j];
			i++;
		}
		for(int k = 0; k < n; k++)
		{
			ov[i] = vg[k];
			i++;
		}
		i = 0;
		if(option == 1 && num == -1)
		{
			//Basic Determination
			int j = 0;
			bool found = false;
//			clock_t start2 = clock();
//			for(int l = 0; l < 1000000; l++)
//			{
				found = false;
				for(j = 0; j < 7; j++)
				{
					for(int k = 0; k < 8; k++)
					{
						if(evaluateNode(ov, prch[k], n+m) != matrix[j][k])
							break;
						else if(k == 7)
							found = true;
					}
					if(found)
						break;
				}
//			}
//			double t2 = ((double)clock() - start2) / CLOCKS_PER_SEC;
//			cout << t2 << endl;
			return j + 1;
		}
		else if(option == 1 && num > 0)
		{
			//Basic Verification
//			clock_t start2 = clock();
			bool found = false;
//			for(int l = 0; l < 10000000; l++)
//			{
				found = false;
				for(int k = 0; k < 8; k++)
				{
					if(evaluateNode(ov, prch[k], n+m) != matrix[num-1][k])
						break;
					else if(k == 7)
						found = true;
				}
//			}
//			double t2 = ((double)clock() - start2) / CLOCKS_PER_SEC;
//			cout << t2 << endl;
			if(found)
				return num;
			else
				return 0;
		}
		else if(option == 2 && num == -1)
		{
			//MCDT
//			clock_t start1 = clock();
//			int tmp = i;
//			for(int l = 0; l < 1000000; l++)
//			{
//				i = tmp;
				while(!prtr[i].isLeaf)
				{
					if(!evaluateNode(ov, prch[prtr[i].id], n+m))
						i = prtr[i].l;
					else
						i = prtr[i].r;
				}
//			}
//			double t1 = ((double)clock() - start1) / CLOCKS_PER_SEC;
//			cout << t1 << endl;
			return prtr[i].id + 1;
		}
		else if (option == 2 && num > 0)
		{
			//Matrix Thinning
//			clock_t start2 = clock();
			bool found = true;
//			for(int l = 0; l < 10000000; l++)
//			{
				found = true;
				for(int k = 0; k < 8; k++)
				{
					if(tmatrix[num-1][k])
						if(evaluateNode(ov, prch[k], n+m) != matrix[num-1][k])
					{
						found = false;
						break;
					}
				}
//			}
//			double t2 = ((double)clock() - start2) / CLOCKS_PER_SEC;
//			cout << t2 << endl;
			if(found)
				return num;
			else
				return 0;
		}
	}
	int matchll(int num)
	{
		//Line matrices
		bool matrix[82][9] = { {0,0,1,0,0,0,1,0,1},{0,0,1,0,0,0,1,1,1},{0,0,1,0,0,1,1,0,1},{0,0,1,0,0,1,1,1,1},
			{0,0,1,0,1,0,1,0,1},{0,0,1,0,1,0,1,1,1},{0,0,1,0,1,1,1,0,1},{0,0,1,0,1,1,1,1,1},
			{0,0,1,1,0,0,1,0,1},{0,0,1,1,0,0,1,1,1},{0,0,1,1,0,1,1,0,1},{0,0,1,1,0,1,1,1,1},
			{0,0,1,1,1,0,1,0,1},{0,0,1,1,1,0,1,1,1},{0,0,1,1,1,1,1,0,1},{0,0,1,1,1,1,1,1,1},
			{0,1,1,0,0,0,1,0,1},{0,1,1,0,0,0,1,1,1},{0,1,1,0,0,1,1,0,1},{0,1,1,0,0,1,1,1,1},
			{0,1,1,0,1,0,1,0,1},{0,1,1,0,1,0,1,1,1},{0,1,1,0,1,1,1,0,1},{0,1,1,0,1,1,1,1,1},
			{0,1,1,1,0,0,1,0,1},{0,1,1,1,0,0,1,1,1},{0,1,1,1,0,1,1,0,1},{0,1,1,1,0,1,1,1,1},
			{0,1,1,1,1,0,1,0,1},{0,1,1,1,1,0,1,1,1},{0,1,1,1,1,1,1,0,1},{0,1,1,1,1,1,1,1,1},
			{1,0,0,0,0,0,0,0,1},{1,0,0,0,0,0,1,0,1},{1,0,0,0,0,0,1,1,1},{1,0,0,0,1,0,0,0,1},
			{1,0,0,0,1,0,1,0,1},{1,0,0,0,1,0,1,1,1},{1,0,0,1,0,0,1,0,1},{1,0,0,1,0,0,1,1,1},
			{1,0,0,1,1,0,1,0,1},{1,0,0,1,1,0,1,1,1},{1,0,1,0,0,0,0,0,1},{1,0,1,0,0,0,1,0,1},
			{1,0,1,0,0,0,1,1,1},{1,0,1,0,0,1,0,0,1},{1,0,1,0,0,1,1,0,1},{1,0,1,0,0,1,1,1,1},
			{1,0,1,0,1,0,0,0,1},{1,0,1,0,1,0,1,0,1},{1,0,1,0,1,0,1,1,1},{1,0,1,0,1,1,0,0,1},
			{1,0,1,0,1,1,1,0,1},{1,0,1,0,1,1,1,1,1},{1,0,1,1,0,0,1,0,1},{1,0,1,1,0,0,1,1,1},
			{1,0,1,1,0,1,1,0,1},{1,0,1,1,0,1,1,1,1},{1,0,1,1,1,0,1,0,1},{1,0,1,1,1,0,1,1,1},
			{1,0,1,1,1,1,1,0,1},{1,0,1,1,1,1,1,1,1},{1,1,1,0,0,0,0,0,1},{1,1,1,0,0,0,1,0,1},
			{1,1,1,0,0,0,1,1,1},{1,1,1,0,0,1,0,0,1},{1,1,1,0,0,1,1,0,1},{1,1,1,0,0,1,1,1,1},
			{1,1,1,0,1,0,0,0,1},{1,1,1,0,1,0,1,0,1},{1,1,1,0,1,0,1,1,1},{1,1,1,0,1,1,0,0,1},
			{1,1,1,0,1,1,1,0,1},{1,1,1,0,1,1,1,1,1},{1,1,1,1,0,0,1,0,1},{1,1,1,1,0,0,1,1,1},
			{1,1,1,1,0,1,1,0,1},{1,1,1,1,0,1,1,1,1},{1,1,1,1,1,0,1,0,1},{1,1,1,1,1,0,1,1,1},
			{1,1,1,1,1,1,1,0,1},{1,1,1,1,1,1,1,1,1} };
		//Characterize 9-intersection elements for line/line
		bool llch[8][9] = { {true, true, false, false, false, false, false, false, false},
			{false, false, false, false, false, false, false, true, false},
			{false, false, true, false, false, false, false, false, false},
			{false, false, false, true, false, false, false, false, false},
			{false, false, false, false, true, false, false, false, false},
			{false, false, false, false, false, true, false, false, false},
			{false, false, false, false, false, false, true, false, false},
			{false, false, false, false, false, false, false, false, true}
		} ;
		//Building the minimum cost decision tree for line/line
		Node lltr[163] = {Node(1, false, 1, 2), Node(3, false, 3, 4), Node(0, false, 5, 6), Node(4, false, 7, 8), Node(0, false, 9, 10),
			Node(3, false, 11, 12), Node(3, false, 13, 14), Node(5, false, 15, 16), Node(5, false, 17, 18), Node(4, false, 19, 20),
			Node(2, false, 21, 22), Node(4, false, 23, 24), Node(4, false, 25, 26), Node(4, false, 27, 28), Node(4, false, 29, 30),
			Node(2, false, 31, 32), Node(0, false, 33, 34), Node(2, false, 35, 36), Node(0, false, 37, 38), Node(5, false, 39, 40),
			Node(5, false, 41, 42), Node(4, false, 43, 44), Node(4, false, 45, 46), Node(5, false, 47, 48), Node(5, false, 49, 50),
			Node(5, false, 51, 52), Node(5, false, 53, 54), Node(5, false, 55, 56), Node(5, false, 57, 58), Node(5, false, 59, 60),
			Node(5, false, 61, 62), Node(6, false, 63, 64), Node(0, false, 65, 66), Node(7, false, 67, 68), Node(6, false, 69, 70),
			Node(6, false, 71, 72), Node(0, false, 73, 74), Node(7, false, 75, 76), Node(6, false, 77, 78), Node(7, false, 79, 80),
			Node(7, false, 81, 82), Node(7, false, 83, 84), Node(7, false, 85, 86), Node(7, false, 87, 88), Node(7, false, 89, 90),
			Node(5, false, 91, 92), Node(5, false, 93, 94), Node(7, false, 95, 96), Node(7, false, 97, 98), Node(7, false, 99, 100),
			Node(7, false, 101, 102), Node(7, false, 103, 104), Node(7, false, 105, 106), Node(7, false, 107, 108), Node(7, false, 109, 110),
			Node(6, false, 111, 112), Node(6, false, 113, 114), Node(6, false, 115, 116), Node(6, false, 117, 118), Node(7, false, 119, 120),
			Node(7, false, 121, 122), Node(7, false, 123, 124), Node(7, false, 125, 126), Node(32, true, 0, 0), Node(7, false, 127, 128),
			Node(7, false, 129, 130), Node(6, false, 131, 132), Node(2, true, 0, 0), Node(3, true, 0, 0), Node(45, true, 0, 0),
			Node(7, false, 133, 134), Node(35, true, 0, 0), Node(7, false, 135, 136), Node(7, false, 137, 138), Node(6, false, 139, 140),
			Node(6, true, 0, 0), Node(7, true, 0, 0), Node(51, true, 0, 0), Node(7, false, 141, 142), Node(8, true, 0, 0),
			Node(9, true, 0, 0), Node(10, true, 0, 0), Node(11, true, 0, 0), Node(12, true, 0, 0), Node(13, true, 0, 0),
			Node(14, true, 0, 0), Node(15, true, 0, 0), Node(38, true, 0, 0), Node(39, true, 0, 0), Node(40, true, 0, 0),
			Node(41, true, 0, 0), Node(7, false, 143, 144), Node(7, false, 145, 146), Node(7, false, 147, 148), Node(7, false, 149, 150),
			Node(16, true, 0, 0), Node(17, true, 0, 0), Node(18, true, 0, 0), Node(19, true, 0, 0), Node(20, true, 0, 0),
			Node(21, true, 0, 0), Node(22, true, 0, 0), Node(23, true, 0, 0), Node(24, true, 0, 0), Node(25, true, 0, 0),
			Node(26, true, 0, 0), Node(27, true, 0, 0), Node(28, true, 0, 0), Node(29, true, 0, 0), Node(30, true, 0, 0),
			Node(31, true, 0, 0), Node(62, true, 0, 0), Node(7, false, 151, 152), Node(65, true, 0, 0), Node(7, false, 153, 154),
			Node(68, true, 0, 0), Node(7, false, 155, 156), Node(71, true, 0, 0), Node(7, false, 157, 158), Node(74, true, 0, 0),
			Node(75, true, 0, 0), Node(76, true, 0, 0), Node(77, true, 0, 0), Node(78, true, 0, 0), Node(79, true, 0, 0),
			Node(80, true, 0, 0), Node(81, true, 0, 0), Node(33, true, 0, 0), Node(34, true, 0, 0), Node(0, true, 0, 0),
			Node(1, true, 0, 0), Node(42, true, 0, 0), Node(7, false, 159, 160), Node(46, true, 0, 0), Node(47, true, 0, 0),
			Node(36, true, 0, 0), Node(37, true, 0, 0), Node(4, true, 0, 0), Node(5, true, 0, 0), Node(48, true, 0, 0),
			Node(7, false, 161, 162), Node(52, true, 0, 0), Node(53, true, 0, 0), Node(54, true, 0, 0), Node(55, true, 0, 0),
			Node(56, true, 0, 0), Node(57, true, 0, 0), Node(58, true, 0, 0), Node(59, true, 0, 0), Node(60, true, 0, 0),
			Node(61, true, 0, 0), Node(63, true, 0, 0), Node(64, true, 0, 0), Node(66, true, 0, 0), Node(67, true, 0, 0),
			Node(69, true, 0, 0), Node(70, true, 0, 0), Node(72, true, 0, 0), Node(73, true, 0, 0), Node(43, true, 0, 0),
			Node(44, true, 0, 0), Node(49, true, 0, 0), Node(50, true, 0, 0)
		};
		//Thin matrices
		bool tmatrix[82][9] = {{1,1,0,1,1,1,0,1,0},{1,1,0,1,1,1,0,1,0},{1,1,0,1,1,1,0,1,0},{1,1,0,1,1,1,0,1,0},{1,1,0,1,1,1,0,1,0},
			{1,1,0,1,1,1,0,1,0},{1,1,0,1,1,1,0,1,0},{1,1,0,1,1,1,0,1,0},{1,1,0,1,1,1,0,1,0},{1,1,0,1,1,1,0,1,0},
			{1,1,0,1,1,1,0,1,0},{1,1,0,1,1,1,0,1,0},{1,1,0,1,1,1,0,1,0},{1,1,0,1,1,1,0,1,0},{1,1,0,1,1,1,0,1,0},
			{1,1,0,1,1,1,0,1,0},{1,1,0,1,1,1,0,1,0},{1,1,0,1,1,1,0,1,0},{1,1,0,1,1,1,0,1,0},{1,1,0,1,1,1,0,1,0},
			{1,1,0,1,1,1,0,1,0},{1,1,0,1,1,1,0,1,0},{1,1,0,1,1,1,0,1,0},{1,1,0,1,1,1,0,1,0},{1,1,0,1,1,1,0,1,0},
			{1,1,0,1,1,1,0,1,0},{1,1,0,1,1,1,0,1,0},{1,1,0,1,1,1,0,1,0},{1,1,0,1,1,1,0,1,0},{1,1,0,1,1,1,0,1,0},
			{1,1,0,1,1,1,0,1,0},{1,1,0,1,1,1,0,1,0},{0,0,1,0,1,0,1,0,0},{0,0,1,1,1,0,1,1,0},{0,0,1,1,1,0,0,1,0},
			{0,0,1,0,1,0,1,0,0},{0,0,1,1,1,0,1,1,0},{0,0,1,1,1,0,0,1,0},{0,0,1,1,1,0,0,1,0},{0,0,1,1,1,0,0,1,0},
			{0,0,1,1,1,0,0,1,0},{0,0,1,1,1,0,0,1,0},{0,1,1,0,1,1,1,0,0},{1,1,1,1,1,1,1,1,0},{1,1,1,1,1,1,0,1,0},
			{0,1,0,0,1,1,1,0,0},{1,1,0,1,1,1,1,1,0},{1,1,0,1,1,1,0,1,0},{0,1,1,0,1,1,1,0,0},{1,1,1,1,1,1,1,1,0},
			{1,1,1,1,1,1,0,1,0},{0,1,0,0,1,1,1,0,0},{1,1,0,1,1,1,1,1,0},{1,1,0,1,1,1,0,1,0},{1,1,1,1,1,1,0,1,0},
			{1,1,1,1,1,1,0,1,0},{1,1,0,1,1,1,0,1,0},{1,1,0,1,1,1,0,1,0},{1,1,1,1,1,1,0,1,0},{1,1,1,1,1,1,0,1,0},
			{1,1,0,1,1,1,0,1,0},{1,1,0,1,1,1,0,1,0},{0,1,0,0,1,1,1,0,0},{1,1,0,1,1,1,1,1,0},{1,1,0,1,1,1,0,1,0},
			{0,1,0,0,1,1,1,0,0},{1,1,0,1,1,1,1,1,0},{1,1,0,1,1,1,0,1,0},{0,1,0,0,1,1,1,0,0},{1,1,0,1,1,1,1,1,0},
			{1,1,0,1,1,1,0,1,0},{0,1,0,0,1,1,1,0,0},{1,1,0,1,1,1,1,1,0},{1,1,0,1,1,1,0,1,0},{1,1,0,1,1,1,0,1,0},
			{1,1,0,1,1,1,0,1,0},{1,1,0,1,1,1,0,1,0},{1,1,0,1,1,1,0,1,0},{1,1,0,1,1,1,0,1,0},{1,1,0,1,1,1,0,1,0},
			{1,1,0,1,1,1,0,1,0},{1,1,0,1,1,1,0,1,0}};
		//Flatten the boolean vectors
		bool ov[m + n];
		int i = 0;
		for(int j = 0; j < m; j++)
		{
			ov[i] = vf[j];
			i++;
		}
		for(int k = 0; k < n; k++)
		{
			ov[i] = vg[k];
			i++;
		}
		i = 0;
		if(option == 1 && num == -1)
		{
			//Basic Determination
			int j = 0;
			bool found = false;
//			clock_t start2 = clock();
//			for(int l = 0; l < 1000000; l++)
//			{
				found = false;
				for(j = 0; j < 82; j++)
				{
					for(int k = 0; k < 8; k++)
					{
						if(evaluateNode(ov, llch[k], n+m) != matrix[j][k])
							break;
						else if(k == 7)
							found = true;
					}
					if(found)
						break;
				}
//			}
//			double t2 = ((double)clock() - start2) / CLOCKS_PER_SEC;
//			cout << t2 << endl;
			return j + 1;
		}
		else if(option == 1 && num > 0)
		{
			//Basic Verification
//			clock_t start2 = clock();
			bool found = false;
//			for(int l = 0; l < 10000000; l++)
//			{
				found = false;
				for(int k = 0; k < 8; k++)
				{
					if(evaluateNode(ov, llch[k], n+m) != matrix[num-1][k])
						break;
					else if(k == 7)
						found = true;
				}
//			}
//			double t2 = ((double)clock() - start2) / CLOCKS_PER_SEC;
//			cout << t2 << endl;
			if(found)
				return num;
			else
				return 0;
		}
		else if(option == 2 && num == -1)
		{
			//MCDT
//			clock_t start1 = clock();
//			int tmp = i;
//			for(int l = 0; l < 1000000; l++)
//			{
//				i = tmp;
				while(!lltr[i].isLeaf)
				{
					if(!evaluateNode(ov, llch[lltr[i].id], n+m))
						i = lltr[i].l;
					else
						i = lltr[i].r;
				}
//			}
//			double t1 = ((double)clock() - start1) / CLOCKS_PER_SEC;
//			cout << t1 << endl;
			return lltr[i].id + 1;
		}
		else if (option == 2 && num > 0)
		{
			//Matrix Thinning
//			clock_t start2 = clock();
			bool found = true;
//			for(int l = 0; l < 10000000; l++)
//			{
				found = true;
				for(int k = 0; k < 8; k++)
				{
					if(tmatrix[num-1][k])
						if(evaluateNode(ov, llch[k], n+m) != matrix[num-1][k])
					{
						found = false;
						break;
					}
				}
//			}
//			double t2 = ((double)clock() - start2) / CLOCKS_PER_SEC;
//			cout << t2 << endl;
			if(found)
				return num;
			else
				return 0;
		}
	}
	int matchlr(int num)
	{
		//Line/Region matrices
		bool matrix[43][9] = { {0,0,1,0,0,0,1,1,1},{0,0,1,0,0,1,1,1,1},{0,0,1,0,1,0,1,1,1},{0,0,1,0,1,1,1,1,1},
			{0,1,0,0,0,0,1,0,1},{0,1,0,0,0,0,1,1,1},{0,1,0,0,1,0,1,1,1},{0,1,1,0,0,0,1,0,1},
			{0,1,1,0,0,0,1,1,1},{0,1,1,0,0,1,1,0,1},{0,1,1,0,0,1,1,1,1},{0,1,1,0,1,0,1,1,1},
			{0,1,1,0,1,1,1,1,1},{1,0,0,0,0,0,1,1,1},{1,0,0,0,1,0,1,1,1},{1,0,0,1,0,0,1,1,1},
			{1,0,0,1,1,0,1,1,1},{1,0,1,0,0,0,1,1,1},{1,0,1,0,0,1,1,1,1},{1,0,1,0,1,0,1,1,1},
			{1,0,1,0,1,1,1,1,1},{1,0,1,1,0,0,1,1,1},{1,0,1,1,0,1,1,1,1},{1,0,1,1,1,0,1,1,1},
			{1,0,1,1,1,1,1,1,1},{1,1,0,0,0,0,1,0,1},{1,1,0,0,0,0,1,1,1},{1,1,0,0,1,0,1,1,1},
			{1,1,0,1,0,0,1,0,1},{1,1,0,1,0,0,1,1,1},{1,1,0,1,1,0,1,1,1},{1,1,1,0,0,0,1,0,1},
			{1,1,1,0,0,0,1,1,1},{1,1,1,0,0,1,1,0,1},{1,1,1,0,0,1,1,1,1},{1,1,1,0,1,0,1,1,1},
			{1,1,1,0,1,1,1,1,1},{1,1,1,1,0,0,1,0,1},{1,1,1,1,0,0,1,1,1},{1,1,1,1,0,1,1,0,1},
			{1,1,1,1,0,1,1,1,1},{1,1,1,1,1,0,1,1,1},{1,1,1,1,1,1,1,1,1} };
		//Characterize 9-intersection elements for line/region
		bool lrch[8][8] = { {true, false, false, false, false, false, false, false},
			{false, true, false, true, false, false, false, false},
			{false, false, true, false, false, false, false, false},
			{false, false, false, false, true, false, false, false},
			{false, false, false, false, false, true, false, false},
			{false, false, false, false, false, false, true, false},
			{true, true, true, true, true, true, true, true},
			{false, false, false, false, false, false, false, true},
		} ;
		//Building the minimum cost decision tree for line/region
		Node lrtr[85] = {Node(3, false, 1, 2), Node(0, false, 3, 4), Node(1, false, 5, 6), Node(4, false, 7, 8), Node(1, false, 9, 10),
			Node(2, false, 11, 12), Node(2, false, 13, 14), Node(5, false, 15, 16), Node(1, false, 17, 18), Node(2, false, 19, 20),
			Node(2, false, 21, 22), Node(4, false, 23, 24), Node(4, false, 25, 26), Node(4, false, 27, 28), Node(4, false, 29, 30),
			Node(2, false, 31, 32), Node(1, false, 33, 34), Node(5, false, 35, 36), Node(2, false, 37, 38), Node(4, false, 39, 40),
			Node(4, false, 41, 42), Node(4, false, 43, 44), Node(4, false, 45, 46), Node(15, true, 0, 0), Node(16, true, 0, 0),
			Node(5, false, 47, 48), Node(5, false, 49, 50), Node(7, false, 51, 52), Node(30, true, 0, 0), Node(5, false, 53, 54),
			Node(5, false, 55, 56), Node(7, false, 57, 58), Node(1, false, 59, 60), Node(1, true, 0, 0), Node(7, false, 61, 62),
			Node(2, true, 0, 0), Node(3, true, 0, 0), Node(6, true, 0, 0), Node(5, false, 63, 64), Node(13, true, 0, 0),
			Node(14, true, 0, 0), Node(5, false, 65, 66), Node(5, false, 67, 68), Node(7, false, 69, 70), Node(27, true, 0, 0),
			Node(5, false, 71, 72), Node(5, false, 73, 74), Node(21, true, 0, 0), Node(22, true, 0, 0), Node(23, true, 0, 0),
			Node(24, true, 0, 0), Node(28, true, 0, 0), Node(29, true, 0, 0), Node(7, false, 75, 76), Node(7, false, 77, 78),
			Node(41, true, 0, 0), Node(42, true, 0, 0), Node(4, true, 0, 0), Node(5, true, 0, 0), Node(0, true, 0, 0),
			Node(7, false, 79, 80), Node(9, true, 0, 0), Node(10, true, 0, 0), Node(11, true, 0, 0), Node(12, true, 0, 0),
			Node(17, true, 0, 0),Node(18, true, 0, 0), Node(19, true, 0, 0), Node(20, true, 0, 0), Node(25, true, 0, 0), 
			Node(26, true, 0, 0), Node(7, false, 81, 82), Node(7, false, 83, 84), Node(35, true, 0, 0), Node(36, true, 0, 0),
			Node(37, true, 0, 0), Node(38, true, 0, 0), Node(39, true, 0, 0), Node(40, true, 0, 0), Node(7, true, 0, 0),
			Node(8, true, 0, 0), Node(31, true, 0, 0), Node(32, true, 0, 0), Node(33, true, 0, 0), Node(34, true, 0, 0),
		};
		//Thin matrices
		bool tmatrix[43][9] = {{1,1,0,0,1,1,0,0,0},{1,1,0,0,1,1,0,0,0},{1,1,0,0,1,1,0,0,0},{1,1,0,0,1,1,0,0,0},{1,0,1,0,0,0,0,1,0},
			{1,0,1,0,1,0,0,1,0},{1,0,1,0,1,0,0,0,0},{1,0,1,0,0,1,0,1,0},{1,1,1,0,1,1,0,1,0},{1,0,0,0,0,1,0,1,0},
			{1,1,0,0,1,1,0,1,0},{1,1,1,0,1,1,0,0,0},{1,1,0,0,1,1,0,0,0},{0,1,1,1,1,0,0,0,0},{0,1,1,1,1,0,0,0,0},
			{0,1,1,1,1,0,0,0,0},{0,1,1,1,1,0,0,0,0},{1,1,1,1,1,1,0,0,0},{1,1,0,1,1,1,0,0,0},{1,1,1,1,1,1,0,0,0},
			{1,1,0,1,1,1,0,0,0},{0,1,1,1,1,1,0,0,0},{0,1,0,1,1,1,0,0,0},{0,1,1,1,1,1,0,0,0},{0,1,0,1,1,1,0,0,0},
			{1,0,1,1,0,0,0,1,0},{1,1,1,1,1,0,0,1,0},{1,1,1,1,1,0,0,0,0},{0,0,1,1,0,0,0,1,0},{0,1,1,1,1,0,0,1,0},
			{0,1,1,1,1,0,0,0,0},{1,0,1,1,0,1,0,1,0},{1,1,1,1,1,1,0,1,0},{1,0,0,1,0,1,0,1,0},{1,1,0,1,1,1,0,1,0},
			{1,1,1,1,1,1,0,0,0},{1,1,0,1,1,1,0,0,0},{0,0,1,1,0,1,0,1,0},{0,1,1,1,1,1,0,1,0},{0,0,0,1,0,1,0,1,0},
			{0,1,0,1,1,1,0,1,0},{0,1,1,1,1,1,0,0,0},{0,1,0,1,1,1,0,0,0}};
			
		//Flatten the boolean vectors
		bool ov[m + n];
		int i = 0;
		for(int j = 0; j < m; j++)
		{
			ov[i] = vf[j];
			i++;
		}
		for(int k = 0; k < n; k++)
		{
			ov[i] = vg[k];
			i++;
		}
		i = 0;
		if(option == 1 && num == -1)
		{
			//Basic Determination
			int j = 0;
			bool found = false;
//			clock_t start2 = clock();
//			for(int l = 0; l < 1000000; l++)
//			{
				found = false;
				for(j = 0; j < 43; j++)
				{
					for(int k = 0; k < 8; k++)
					{
						if(evaluateNode(ov, lrch[k], n+m) != matrix[j][k])
							break;
						else if(k == 7)
							found = true;
					}
					if(found)
						break;
				}
//			}
//			double t2 = ((double)clock() - start2) / CLOCKS_PER_SEC;
//			cout << t2 << endl;
			return j + 1;
		}
		else if(option == 1 && num > 0)
		{
			//Basic Verification
//			clock_t start2 = clock();
			bool found = false;
//			for(int l = 0; l < 10000000; l++)
//			{
				found = false;
				for(int k = 0; k < 8; k++)
				{
					if(evaluateNode(ov, lrch[k], n+m) != matrix[num-1][k])
						break;
					else if(k == 7)
						found = true;
				}
//			}
//			double t2 = ((double)clock() - start2) / CLOCKS_PER_SEC;
//			cout << t2 << endl;
			if(found)
				return num;
			else
				return 0;
		}
		else if(option == 2 && num == -1)
		{
			//MCDT
//			clock_t start1 = clock();
//			int tmp = i;
//			for(int l = 0; l < 1000000; l++)
//			{
//				i = tmp;
				while(!lrtr[i].isLeaf)
				{
					if(!evaluateNode(ov, lrch[lrtr[i].id], n+m))
						i = lrtr[i].l;
					else
						i = lrtr[i].r;
				}
//			}
//			double t1 = ((double)clock() - start1) / CLOCKS_PER_SEC;
//			cout << t1 << endl;
			return lrtr[i].id + 1;
		}
		else if (option == 2 && num > 0)
		{
			//Matrix Thinning
//			clock_t start2 = clock();
			bool found = true;
//			for(int l = 0; l < 10000000; l++)
//			{
				found = true;
				for(int k = 0; k < 8; k++)
				{
					if(tmatrix[num-1][k])
						if(evaluateNode(ov, lrch[k], n+m) != matrix[num-1][k])
					{
						found = false;
						break;
					}
				}
//			}
//			double t2 = ((double)clock() - start2) / CLOCKS_PER_SEC;
//			cout << t2 << endl;
			if(found)
				return num;
			else
				return 0;
		}
	}
	int matchrr(int num)
	{
		//Region matrices
		bool matrix[33][9]={ {false,false,true,false,false,true,true,true,true},{false,false,true,false,true,false,true,true,true},{false,false,true,false,true,true,true,false,true},
			{false,false,true,false,true,true,true,true,true},{true,false,false,false,true,false,false,false,true},{true,false,false,false,true,false,true,true,true},
			{true,false,false,true,false,false,true,true,true},{true,false,false,true,true,false,true,false,true},{true,false,false,true,true,false,true,true,true},
			{true,false,true,false,true,false,true,true,true},{true,false,true,false,true,true,false,false,true},{true,false,true,false,true,true,true,false,true},
			{true,false,true,false,true,true,true,true,true},{true,false,true,true,false,true,true,true,true},{true,false,true,true,true,false,true,false,true},
			{true,false,true,true,true,false,true,true,true},{true,false,true,true,true,true,true,false,true},{true,false,true,true,true,true,true,true,true},
			{true,true,true,false,false,true,false,false,true},{true,true,true,false,false,true,true,true,true},{true,true,true,false,true,false,false,false,true},
			{true,true,true,false,true,false,true,false,true},{true,true,true,false,true,false,true,true,true},{true,true,true,false,true,true,false,false,true},
			{true,true,true,false,true,true,true,false,true},{true,true,true,false,true,true,true,true,true},{true,true,true,true,false,false,true,true,true},
			{true,true,true,true,false,true,true,false,true},{true,true,true,true,false,true,true,true,true},{true,true,true,true,true,false,true,false,true},
			{true,true,true,true,true,false,true,true,true},{true,true,true,true,true,true,true,false,true},{true,true,true,true,true,true,true,true,true} };
		//Characterize 9-intersection elements for region/region
		bool rrch[8][15] = { {0, 1, 0, 0, 1, 1, 1, 0, 1, 0, 0, 1, 1, 1, 0},
			{0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 0, 1, 0},
			{1, 0, 1, 1, 0, 0, 0, 0, 0, 0, 1, 1, 0, 1, 0},
			{0, 0, 0, 0, 1, 0, 1, 0, 0, 0, 0, 0, 0, 0, 0},
			{0, 1, 0, 1, 0, 1, 0, 0, 1, 0, 1, 0, 1, 0, 1},
			{1, 0, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
			{0, 0, 0, 1, 1, 0, 1, 1, 0, 1, 1, 0, 0, 0, 0},
			{0, 0, 0, 0, 0, 0, 0, 1, 0, 1, 0, 0, 0, 0, 0}
		} ;
		//Building the minimum cost decision tree for region/region
		Node rrtr[65] = {Node(1, false, 1, 2), Node(3, false, 3, 4), Node(3, false, 5, 6), Node(5, false, 7, 8), Node(2, false, 9,10),
			Node(5, false,11,12), Node(4, false,13,14), Node(2, false,15,16), Node(0, false,17,18), Node(4, false,19,20),
			Node(5, false,21,22), Node(6, false,23,24), Node(4, false,25,26), Node(5, false,27,28), Node(5, false,29,30),
			Node(6, false,31,32), Node(0, false,33,34), Node(4, false,35,36), Node(6, false,37,38), Node(6, true, 0, 0),
			Node(7, false,39,40), Node(7, false,41,42), Node(4, false,43,44), Node(20, true, 0, 0), Node(7, false,45,46),
			Node(6, false,47,48), Node(6, false,49,50), Node(26, true, 0, 0), Node(7, false,51,52), Node(7, false,53,54),
			Node(7, false,55,56), Node(4, true, 0, 0), Node(5, true, 0, 0), Node(1, true, 0, 0), Node(9, true, 0, 0),
			Node(0, true, 0, 0), Node(7, false,57,58), Node(10, true, 0, 0), Node(7, false,59,60), Node(7, true, 0, 0),
			Node(8, true, 0, 0), Node(14, true, 0, 0), Node(15, true, 0, 0), Node(13, true, 0, 0), Node(7, false,61,62),
			Node(21, true, 0, 0), Node(22, true, 0, 0), Node(18, true, 0, 0), Node(19, true, 0, 0), Node(23, true, 0, 0),
			Node(7, false,63,64), Node(27, true, 0, 0), Node(28, true, 0, 0), Node(29, true, 0, 0), Node(30, true, 0, 0),
			Node(31, true, 0, 0), Node(32, true, 0, 0), Node(2, true, 0, 0), Node(3, true, 0, 0), Node(11, true, 0, 0),
			Node(12, true, 0, 0), Node(16, true, 0, 0), Node(17, true, 0, 0), Node(24, true, 0, 0), Node(25, true, 0, 0)
		};
		//Thin matrices
		bool tmatrix[33][9] = {{1,0,0,0,1,0,0,0,0},{1,0,0,0,0,1,0,0,0},{1,0,0,0,0,0,0,1,0},{1,0,0,0,1,1,0,1,0},{0,0,1,0,0,0,1,0,0},
			{0,0,1,1,0,0,0,1,0},{0,0,1,0,1,0,0,0,0},{0,0,1,0,0,0,1,1,0},{0,0,1,1,1,0,0,1,0},{1,1,1,1,0,1,0,0,0},
			{0,1,0,0,0,1,1,0,0},{1,1,0,1,0,0,1,1,0},{1,1,0,1,0,1,0,1,0},{0,1,0,1,1,1,0,0,0},{0,1,1,0,0,1,0,1,0},
			{0,1,1,1,0,1,0,1,0},{0,1,0,1,0,1,0,1,0},{0,1,0,1,1,1,0,1,0},{0,0,0,0,1,0,1,0,0},{0,1,0,1,1,0,0,1,0},
			{0,0,1,0,0,1,1,0,0},{0,0,0,1,0,1,1,1,0},{0,1,0,1,0,1,0,1,0},{0,1,0,0,1,1,1,0,0},{0,1,0,1,0,1,1,1,0},
			{0,1,0,1,1,1,0,1,0},{0,0,1,0,1,1,0,0,0},{0,0,0,0,1,0,1,1,0},{0,1,0,1,1,1,0,1,0},{0,1,0,1,0,1,0,1,0},
			{0,1,0,1,1,1,0,1,0},{0,1,0,1,1,1,0,1,0},{0,1,0,1,1,1,0,1,0}};
		//Flatten the boolean vectors
		bool ov[m + n];
		int i = 0;
		for(int j = 0; j < m; j++)
		{
			ov[i] = vf[j];
			i++;
		}
		for(int k = 0; k < n; k++)
		{
			ov[i] = vg[k];
			i++;
		}
		i = 0;
		if(option == 1 && num == -1)
		{
			//Basic Determination
			//No MCDT Testing
			int j = 0;
			bool found = false;
//			clock_t start2 = clock();
//			for(int l = 0; l < 1000000; l++)
//			{
				found = false;
				for(j = 0; j < 33; j++)
				{
					for(int k = 0; k < 8; k++)
					{
						if(evaluateNode(ov, rrch[k], n+m) != matrix[j][k])
							break;
						else if(k == 7)
							found = true;
					}
					if(found)
						break;
				}
//			}
//			double t2 = ((double)clock() - start2) / CLOCKS_PER_SEC;
//			cout << t2 << endl;
			return j + 1;
		}
		else if(option == 1 && num > 0)
		{
			//Basic Verification
//			clock_t start2 = clock();
			bool found = false;
//			for(int l = 0; l < 10000000; l++)
//			{
				found = false;
				for(int k = 0; k < 8; k++)
				{
					if(evaluateNode(ov, rrch[k], n+m) != matrix[num-1][k])
						break;
					else if(k == 7)
						found = true;
				}
//			}
//			double t2 = ((double)clock() - start2) / CLOCKS_PER_SEC;
//			cout << t2 << endl;
			if(found)
				return num;
			else
				return 0;
		}
		else if(option == 2 && num < 0)
		{
			//MCDT
//			clock_t start1 = clock();
//			int tmp = i;
//			for(int l = 0; l < 1000000; l++)
//			{
//				i = tmp;
				while(!rrtr[i].isLeaf)
				{
					if(!evaluateNode(ov, rrch[rrtr[i].id], n+m))
						i = rrtr[i].l;
					else
						i = rrtr[i].r;
					//std::cout<<" ("<< i << ") ";
				}
//			}
//			double t1 = ((double)clock() - start1) / CLOCKS_PER_SEC;
			//std::cout<<"\n";
//			cout << t1 << endl;
			int adder=0;
			if (num==-2) {
				// {F0/1, F0/2, F1/0, F1/1, F1/2, F2/0, F2/1}
				//{G0/1, G0/2, G1/0, G1/1, G1/2, G2/0, G2/1, point_in_common}
				if (!(vf[1] || vf[5] || vf[3]) && vg[6])
					adder=100; //0D
				else if ((vf[1] || vf[5] || vf[3]) && vg[6])
					adder=200; //01D
				else if ((vf[1] || vf[5] || vf[3]) && !vg[6])
					adder=300; //1D
			}	
			return (adder + rrtr[i].id + 1);
		}
		else if (option == 2 && num > 0)
		{
			//Matrix Thinning
//			clock_t start2 = clock();
			bool found = true;
//			for(int l = 0; l < 10000000; l++)
//			{
				found = true;
				for(int k = 0; k < 8; k++)
				{
					if(tmatrix[num-1][k])
						if(evaluateNode(ov, rrch[k], n+m) != matrix[num-1][k])
					{
						found = false;
						break;
					}
				}
//			}
//			double t2 = ((double)clock() - start2) / CLOCKS_PER_SEC;
//			cout << t2 << endl;
			if(found)
				return num;
			else
				return 0;
		}
	}
};
}
#endif
