/** This class implements a simple version of a SweepLine
 *  \author Alejandro Pauly, Reasey Praing
 *  \version 0.1
 *  \date    2005-12-02
 *  \version 0.2 fixing for mSlob
 *  \date    2006-10-25
 */

#ifndef SWEEPLINE_H
#define SWEEPLINE_H

#include "statusHseg2D.h"
#include "point2D.h"
#include "num.h"
#include "EventSchedule.h"
#include  "dvector.h"
#include "seg2D.h"

namespace stu{

	using namespace stu;

	struct eventStruct {
		bool isPoi2D;
		void * obj; // pointer to the object;
	};

	template <class T1, class T2, class T3, class T4>
	class SweepLine{
		private:
			bool sweepSingleObject;	
		
		public:
			SweepLine( bool singleObject = false): sweepSingleObject( singleObject) { }
			SweepLine(stu::dvector<statusHseg2D> & _status):status(_status){ }
			~SweepLine(){ }
			stu::dvector<statusHseg2D> status;
			/** Add a new left hseg2D to status structure 
			*/
			//Generic add_left
			bool add_left(statusHseg2D &h, EventSchedule<T1,T2,T3,T4> &es)
			{
				hseg2D segCurr, segAbove, segBelow;
				bool one, two;
				return add_left_Return_Segs( h, es, segCurr, segAbove, one, segBelow, two );
			}
			//Helper for region algorithm to handle coinside situation
			bool add_left_skip(statusHseg2D &h, EventSchedule<T1,T2,T3,T4> &es, bool &skip_event)
			{
				hseg2D segCurr, segAbove, segBelow;
				bool one, two;
				return add_left_Return_Segs_Skip( h, es, segCurr, segAbove, one, segBelow, two, skip_event);
			}
			//Helper for operations
			bool add_left_Return_Segs(statusHseg2D &h, EventSchedule<T1,T2,T3,T4> &es, hseg2D &segCurr, hseg2D &segAbove, bool &segAboveExists,  hseg2D &segBelow, bool &segBelowExists )
			{
				bool skip_event;
// 				bool one, two;
				return add_left_Return_Segs_Skip( h, es, segCurr, segAbove, segAboveExists, segBelow, segBelowExists, skip_event );
			}
			//The actual add_left
			bool add_left_Return_Segs_Skip(statusHseg2D &h, EventSchedule<T1,T2,T3,T4> &es, hseg2D &segCurr, hseg2D &segAbove, bool &segAboveExists,  hseg2D &segBelow, bool &segBelowExists, bool &skip_event)
			{
				segAboveExists = false;
				segBelowExists = false;
				bool intersectWithTop = false;
				bool intersectWithBot = false;
				bool coincideWithTop = false, coincideWithBot = false;
				segCurr = *(h.hseg);
				//insert the segment and check intersection with both of the neighbors
// 				cout<<"---------Inserting Element---------"<<endl;
// 				cout<<"Element: "<< h << endl;
// 				cout<<"Status structure before: " << status << endl;
//				stu::dvector<statusHseg2D>::Iterator itr;
				if(status.isEmpty())
					status.push_back(h);
				else
				{
					//insert the segment and check intersection with both of the neighbors (use binary search)
					vectorBinarySearchWrapper< dvector<statusHseg2D>, statusHseg2D > binaryFinder( &(status) );
					uint index;
					binaryFinder.find(h, index);
					status.insert(index, h);
				
					/*
					//Using Linear Search
					itr = status.getIterator();
					while(itr.next())
					{
//						cout<<" itr.current in while: "<<itr.current<<endl;
//						cout<<"h: "<< h.hseg->getDominating().isUndefined() << "  " << h.hseg->getSubmissive().isUndefined() << endl;
//						cout<<"itr: "<< itr->hseg->getDominating().isUndefined() << "  " << itr->hseg->getSubmissive().isUndefined() << endl;
//						cout<<"("<<h.hseg->getDominating().getX()<<","<< *(itr.get())<<");  " << h.hseg->getDominating().getY()<<endl;
//						cout<<sweepIntersect(h.hseg->getDominating().getX(), *(itr.get()))<<endl;
						if(sweepIntersect(h.hseg->getDominating().getX(), *(itr.get())) < h.hseg->getDominating().getY())
						{
//							cout<<" itr.current 1: "<<itr.current<<endl;
							if(!itr.next())
							{
								status.push_back(h);
								break;
							}
							else
							{
								itr.prev();
								continue;
							}
//							cout<<" itr.current 2: "<<itr.current<<endl;
						}
						else if(sweepIntersect(h.hseg->getDominating().getX(), *(itr.get())) > h.hseg->getDominating().getY())
						{
//							cout<<" itr.current: "<<itr.current<<endl;
							status.insert(itr,h);
							break;
						}
						else
						{
							hseg2D s1 = *(itr.get()->hseg);
							hseg2D s2 = *(h.hseg);
							rat slope1, slope2;
							bool infinity1 = false, infinity2 = false;
							
							if(s1 == s2) //>>>>>>>>>>>>>>>>>>>>>>>>>>>> ADDED 7/27/06 <<<<<<<<<<<<<<<<<<<<<<<<<<<<
							{	
								status.insert(itr,h);
								break;
							}
							
							if(s1.getSubmissive().getX() - s1.getDominating().getX() == 0)
								infinity1 = true;
							else
								slope1 = (s1.getSubmissive().getY() - s1.getDominating().getY()) / (s1.getSubmissive().getX() - s1.getDominating().getX());
		
							if(s2.getSubmissive().getX() - s2.getDominating().getX() == 0)
								infinity2 = true;
							else
								slope2 = (s2.getSubmissive().getY() - s2.getDominating().getY()) / (s2.getSubmissive().getX() - s2.getDominating().getX());
		
							if((!infinity1 && !infinity2 && slope1 < slope2) || (!infinity1 && infinity2))
								if(!itr.next())
								{
									//cout<<"Push: "<<h<<endl;
									status.push_back(h);
									//cout<<"Pushed: "<<*(itr.get())<<endl;
									break;
								}
								else
								{
									itr.prev();
									continue;
								}
							else if((!infinity1 && !infinity2 && slope1 > slope2) || (infinity1 && !infinity2))
							{
								status.insert(itr,h);
								break;
							}
							else if((!infinity1 && !infinity2 && slope1 == slope2) || (infinity1 && infinity2))//slopes are equal and they both left halfsegment => coincide
							{
								//skip overlap num calculation
								skip_event = true;
								//check endpoints for ordering
								if(s1.getSubmissive().getX() < s2.getSubmissive().getX())
									if(!itr.next())
									{
										status.push_back(h);
										break;
									}
									else
									{
										itr.prev();
										continue;
									}
								else if(s1.getSubmissive().getX() > s2.getSubmissive().getX())
								{
									status.insert(itr,h);
									break;
								}
								else
								{
									
									if(s1.getSubmissive().getY() < s2.getSubmissive().getY())
									{
										if(!itr.next())
										{
											status.push_back(h);
											break;
										}
										else
										{
											itr.prev();
											continue;
										}
									}
									else if(s1.getSubmissive().getY() > s2.getSubmissive().getY())
									{
										status.insert(itr,h);
										break;
									}
								}
							}
						}
					}
					*/
					/*
					itr = status.getIterator();
					while(itr.next())
					{
						if(*(itr.get()->hseg) == *(h.hseg))
							break;
					}
					*/
					stu::dvector<statusHseg2D>::Iterator itr = status.getIterator((long)index);
					stu::dvector<statusHseg2D>::Iterator itr1;
					itr1 = itr;
					
					if(status.size() > 1 && !itr.begin())
					{
//						cout<<"--------------CHECK BOTTOM---------------\n";
//						cout<< "{"<<(itr-1)->hseg->getDominating()<<", "<<(itr-1)->hseg->getSubmissive()<<") , "<<(itr-1)->hseg->isLeft()<<", "<<(itr-1)->hseg->isInsideAbove()<<"}"<<(itr-1)->owner<<endl;
						itr.prev();
						segBelow =*(itr.get()->hseg);
						segBelowExists = true;
						coincideWithBot = checkIntersection(h, true, itr1, itr, es, intersectWithTop);
					}
					itr = itr1;
					if(status.size() > 1 && itr.next())
					{
						//cout << "Current Status After: " << status << endl;
//						cout<<"--------------CHECK TOP---------------\n";
//						cout<< "{"<<(itr+1)->hseg->getDominating()<<", "<<(itr+1)->hseg->getSubmissive()<<") , "<<(itr+1)->hseg->isLeft()<<", "<<(itr+1)->hseg->isInsideAbove()<<"}"<<(itr+1)->owner<<endl;
						segAbove = *(itr.get()->hseg);
						segAboveExists = true;
						coincideWithTop = checkIntersection(h, true, itr1, itr, es, intersectWithBot);
					}
				}
// 				cout<<"Status structure after: " << status << endl;
				return( coincideWithBot || coincideWithTop || intersectWithTop || intersectWithBot );
			}

			bool del_right(statusHseg2D &h, EventSchedule<T1,T2,T3,T4> &es){
				statusHseg2D p;
				return del_rightReturn(h,es,p);
			}
			
			bool del_rightReturn(statusHseg2D &h, EventSchedule<T1,T2,T3,T4> &es, statusHseg2D&  left) 
			{
				//remove the segment and check intersection between both of the neighbors (use Bianry Search)
//				cout<<"---------Removing Element---------\n";
//				cout<<"Status structure before: " << status << endl;
				stu::dvector<statusHseg2D>::Iterator itr, ib, ia;
				
				//statusHseg2D hleft(hseg2D(h.hseg->getSubmissive(), h.hseg->getDominating(),!h.hseg->isInsideAbove()), h.owner);
				
				//Using Binary Search
				uint index;
				vectorBinarySearchWrapper< dvector<statusHseg2D>, statusHseg2D > binaryFinder( &status );
				bool found = binaryFinder.find(h, index);  //Will find the left of h since we only compare segments in statusHseg2D
				bool intersectionOccurs = false;
				//Taking care of duplicate entries from different objects
				if(found && !(status[index].owner == h.owner || h.owner=='B' || h.owner=='T'))
				{
					if(index != 0 && status[index-1] == h)
						index--;
					else if(index < status.size() - 1 && status[index+1] == h)
						index++;
				}
				if(found && (status[index].owner == h.owner || h.owner=='B' || h.owner=='T'))
				{
					//Remove the element at index
					itr = status.getIterator(index);
					ia = itr;
					if(!itr.begin() && itr.next())
					{
						ib = itr;
						itr.prev();
						itr.prev();
						if(itr.get()->owner != ib.get()->owner)
							checkIntersection(h,false, itr, ib, es, intersectionOccurs);
					}
//							cout<<"erasing: "<<itr.current<<endl;
					left=*(ia.get());
					status.erase(ia);
					return true;
				}

/*
				//Using Linear Search
				itr = status.getIterator();
				while(itr.next())	//Binary Search will be done later
				{
					bool intersectionOccurs = false;
					if(itr.get()->owner == h.owner || h.owner=='B' || h.owner=='T')
						if(itr.get()->hseg->getSegment() == h.hseg->getSegment())
						{
							//Remove the i'th element
							ia = itr;
							if(!itr.begin() && itr.next())
							{
								ib = itr;
								itr.prev();
								itr.prev();
								if(itr.get()->owner != ib.get()->owner)
									checkIntersection(h,false, itr, ib, es, intersectionOccurs);
							}
//							cout<<"erasing: "<<itr.current<<endl;
							left=*(ia.get());
							status.erase(ia);
							return true;
						}
				}
*/
//				cout<<"Status structure after: " << status << endl;
				return false;
			}
			bool poi_in_seg(const poi2D &p, char o)
			{
				stu::dvector<statusHseg2D>::Iterator i;
				bool result = false;
				i = status.getIterator();
//				cout<<"Status structure: " << status << endl;
				
				while(i.next())
				{
//					cout<<flush;
//					cout<<"i="<<i.current<<endl;
//					cout<<"point: "<<p<<endl;
					//std::cout<< sweepIntersect(p.getX(), *i) << " --- " << p.getY() << "\n";
					if(i.get()->hseg->getSubmissive().getX() - i.get()->hseg->getDominating().getX() == 0)
					{	
						if(p.getX() == i.get()->hseg->getSubmissive().getX() && p.getY() < i.get()->hseg->getSubmissive().getY() && p.getY() > i.get()->hseg->getDominating().getY() && i.get()->owner != o)
						{
							result = true;
							break;
						}
					}
					else if(sweepIntersect(p.getX(), *(i.get())) == p.getY() && p > i.get()->hseg->low() && p < i.get()->hseg->high() && i.get()->owner != o)
					{
						result = true;
						break;
					}
					else if(sweepIntersect(p.getX(), *(i.get())) > p.getY())
						break;
				}
				return result;
			}
			bool poi_on_seg(const poi2D &p, char o)
			{
				stu::dvector<statusHseg2D>::Iterator i;
				bool result = false;
				i = status.getIterator();
				while(i.next())
				{
					if(i.get()->hseg->getSubmissive().getX() - i.get()->hseg->getDominating().getX() == 0)
					{
						if(p.getX() == i.get()->hseg->getSubmissive().getX() && p.getY() <= i.get()->hseg->getSubmissive().getY() && p.getY() >= i.get()->hseg->getDominating().getY() && i.get()->owner != o)
						{
							result = true;
							break;
						}
					}
					else if(sweepIntersect(p.getX(), *(i.get())) == p.getY() && p >= i.get()->hseg->getDominating() && p <= i.get()->hseg->getSubmissive() && i.get()->owner != o)
					{
						result = true;
						break;
					}
					else if(sweepIntersect(p.getX(), *(i.get())) > p.getY())
						break;
				}
				return result;
			}
			bool poi_in_region(const poi2D &p)
			{
				stu::dvector<statusHseg2D>::Iterator i;
				bool result = false;
				i = status.getIterator();
				while(i.next())
				{
					if(sweepIntersect(p.getX(), *(i.get())) == p.getY() && p >= i.get()->hseg->getDominating() && p <= i.get()->hseg->getSubmissive())
					{
						result = false;
						break;
					}
					else if(sweepIntersect(p.getX(), *(i.get())) > p.getY())
					{
						if(!i.begin())
						{
							if(i.prev() && i.get()->n == 1)
							{
								result = true;
								break;
							}
							else
								break;
						}
						else
							break;
					}
				}
				return result;
			}
			bool poi_in_region(const poi2D &p, char o)
			{
				stu::dvector<statusHseg2D>::Iterator i;
				bool result = false;
				i = status.getIterator();
//				cout<<"p: " <<p<<endl;
				while(i.next())
				{
					if(i.get()->owner != o)
					{
						if(sweepIntersect(p.getX(), *(i.get())) == p.getY() && p >= i.get()->hseg->getDominating() && p <= i.get()->hseg->getSubmissive())
						{
							result = false;
							break;
						}
						else if(sweepIntersect(p.getX(), *(i.get())) > p.getY())
						{
							if(!i.get()->hseg->isInsideAbove())//(i->m == 1)
							{
								result = true;
								break;
							}
							else
								break;
						}
					}
				}
				return result;
			}
			bool seg_in_region(const statusHseg2D h) //h must already be in the sweepline  >>>>>>>>>>> Overhauled 7/27/06 <<<<<<<<<<<<<
			{
				stu::dvector<statusHseg2D>::Iterator i;
				bool result = false;
				i = status.getIterator();
				while(i.next())
				{
					//cout<<i->owner<<": {"<<i->hseg->getDominating()<<", "<<i->hseg->getSubmissive()<<") , "<<i->hseg->isLeft()<<", "<<i->hseg->isInsideAbove()<<"}"<<endl;
					if(i.get()->hseg->getSegment() == h.hseg->getSegment() && i.get()->owner == h.owner)
						break;
				}
				if(i.end())
					cerr<<"Error: h must already be in the sweepline.\n";
				else
				{
					while(!i.begin())
					{
						if(i.prev() && i.get()->hseg->getSegment() != h.hseg->getSegment() && i.get()->owner != h.owner)
						{
							//cout<<"HERE\n";
							if(i.get()->hseg->isInsideAbove())// && i->hseg->getSegment().testIntersection(h.hseg->getSegment()) != SEGMENT)
							{
								result = true;
								break;
							}
							else
							{
							//	cout<<"HERE1\n";
								//cout<<"break here\n";
								break;
							}
						}
					}
				}
				return result;
			}
		private:
			bool checkIntersection(statusHseg2D &h, bool writeBack, stu::dvector<statusHseg2D>::Iterator i, stu::dvector<statusHseg2D>::Iterator j, EventSchedule<T1,T2,T3,T4> &es, bool &intersectionOccurs )
			{
//				cout << "Checking Element " <<*(i.get())<<" and "<<*(j.get())<<endl;
				bool coincide = false;
//				cout<<"i owner: "<<i->owner<<"j owner: "<<j->owner<<endl;
				
				
				if(!i.end() && !j.end() && (i.get()->owner == j.get()->owner && !sweepSingleObject) )
					return coincide;	//Segments from the same object
		
				
				
				//Check Intersection
				//System.out.println("Checking Intersection for element " + i + " and " + j + ".");
				hseg2D s1;
				hseg2D s2;
				if(!i.end() && !j.end() )
				{
					s1 = *(i.get()->hseg);
					s2 = *(j.get()->hseg);
				}
				rat X1 = s1.getDominating().getX(), Y1 = s1.getDominating().getY(), X2 = s1.getSubmissive().getX(), Y2 = s1.getSubmissive().getY(),
					A1 = s2.getDominating().getX(), B1 = s2.getDominating().getY(), A2 = s2.getSubmissive().getX(), B2 = s2.getSubmissive().getY();
				rat dx, dy, da, db, s, t, b1, b2, Ix, Iy;
		
				dx = X2 - X1;
				dy = Y2 - Y1;
				da = A2 - A1;
				db = B2 - B1;
		
				
				//REALMIFICATION
				if (da * dy - db * dx == 0)	//Segments are parallel, possibly coincide
				{
					
//					cout << "Element " <<*(i.get())<<" is parallel to "<<*(j.get())<<endl;
					if(dx == 0 || da == 0) //They are vertical
					{
						//Since they are on the Sweep status, they are on the same line
						if(Y1 < B1)
						{
							if(B1 < Y2)
							{
								//insert B1 into i
								insertNewIntersectionPoint(h, writeBack, A1, B1, i, es, intersectionOccurs);
								if(B2 > Y2)
									//insert Y2 into j
									insertNewIntersectionPoint(h, false, X2, Y2, j, es, intersectionOccurs);
							}
							//else if (B1 == Y2)		//Should never be executed
								//overlapNum[0][0][0] = true;
							if(B2 < Y2)
								//insert B2 into i
								insertNewIntersectionPoint(h, writeBack, A2, B2, i, es, intersectionOccurs);
							//else if(B2 == Y2)
							//	overlapNum[0][0][0] = true;
						}
						else if(Y1 == B1)
						{
							//overlapNum[0][0][0] = true;
							coincide = true;
							if(B2 < Y2)
								//insert B2 into i
								insertNewIntersectionPoint(h, writeBack, A2, B2, i, es, intersectionOccurs);
							else if(B2 == Y2)
								;//do nothing
							else if(B2 > Y2)
								//insert Y2 into j
								insertNewIntersectionPoint(h, false, X2, Y2, j, es, intersectionOccurs);
						}
						else if(B1 < Y1)
						{
							if(Y2 < B2)
								//insert Y2 into j
								insertNewIntersectionPoint(h, false, X2, Y2, j, es, intersectionOccurs);
							//else if(Y2 == B2)
							//	overlapNum[0][0][0] = true;
							if(Y1 < B2)
							{
								//insert Y1 into j
								insertNewIntersectionPoint(h, false, X1, Y1, j, es, intersectionOccurs);
								if(Y2 > B2)
									//insert B2 into i
									insertNewIntersectionPoint(h, writeBack, A2, B2, i, es, intersectionOccurs);
							}
							//else if (Y1 == B2)//Should never be executed
							//	overlapNum[0][0][0] = true;
							
						}
					}
					else	//They are not vertical
					{
						
						//System.out.println("Element " + i + " and " + j + " are not vertical.");
		
						b1 = Y1 - (dy / dx) * X1;
						b2 = B1 - (db / da) * A1;
						if( b1 != b2) //They are not on the same line
						{
							//System.out.println("Element " + i + " and " + j + " are not on the same line.");
						
							return coincide;
						}
						else		  //They are on the same line and both are active in sweep status
						{
							
							
							if(X1 < A1)
							{
								if(A1 < X2)
								{
									//insert A1 into i
									insertNewIntersectionPoint(h, writeBack, A1, B1, i, es, intersectionOccurs);
									if(A2 > X2)
										//insert X2 into j
										insertNewIntersectionPoint(h, false, X2, Y2, j, es, intersectionOccurs);
								}
								//else if (A1 == X2)		//Should never be executed
								//	overlapNum[0][0][0] = true;
								if(A2 < X2)
									//insert A2 into i
									insertNewIntersectionPoint(h, writeBack, A2, B2, i, es, intersectionOccurs);
								//else if(A2 == X2)
								//	overlapNum[0][0][0] = true;
							}
							else if(X1 == A1)
							{
								//overlapNum[0][0][0] = true;
								coincide = true;
								if(A2 < X2)
									//insert A2 into i
									insertNewIntersectionPoint(h, writeBack, A2, B2, i, es, intersectionOccurs);
								else if(A2 == X2)
									;//do nothing
								else if(A2 > X2)
									//insert X2 into j
									insertNewIntersectionPoint(h, false, X2, Y2, j, es, intersectionOccurs);
							}
							else if(A1 < X1)
							{
								if(X2 < A2)
									//insert X2 into j
									insertNewIntersectionPoint(h, false, X2, Y2, j, es, intersectionOccurs);
								//else if(X2 == A2)
								//	overlapNum[0][0][0] = true;
								if(X1 < A2)
								{
									//insert X1 into j
									insertNewIntersectionPoint(h, false, X1, Y1, j, es, intersectionOccurs);
									if(X2 > A2)
										//insert A2 into i
										insertNewIntersectionPoint(h, writeBack, A2, B2, i, es, intersectionOccurs);
								}
								//else if (X1 == A2)		//Should never be executed
								//	overlapNum[0][0][0] = true;
								
							}
						}
					}
					return coincide;
				}
				
//				cout<<"Unparallel: "<< *(i.get()) << " ; " << *(j.get()) <<"\n";
		
				s = (dx * (B1 - Y1) + dy * (X1 - A1)) / (da * dy - db * dx);
				t = (da * (Y1 - B1) + db * (A1 - X1)) / (db * dx - da * dy);
				
				//cout << "\n s = " <<s<<endl;
				//cout << "t = "<<t<<endl;
				/*//Handling rational robustness limitation >>>>>>>>>>>>>>>>>>>>> added 6/14/06 >>>>>>>>>>>>>>>>>
				rat diffs, difft;
				rat one(1), zero(0);
				rat threshold(1, 1000000000); //Can be changed depending on the RATIO limit
				if(s.abs() < threshold)
					s = zero;
				if(t.abs() < threshold)
					t = zero;
				diffs = one - s; difft = one - t;
				if(diffs.abs() < threshold)
					s = one;
				if(difft.abs() < threshold)
					t = one;
				*/
				//>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
				
				if (s >= 0 && s <= 1 && t >= 0 && t <= 1)	//There is an intersection
				{
					//System.out.println("Element " + i + " and " + j + " have an intersection.");
					Ix = X1 + t * dx;
					Iy = Y1 + t * dy;
//					cout<<"\nIntersection Point Found: s = "<<s<< "; t = "<<t<<endl;
					if (s > 0 && s < 1 && t > 0 && t < 1)	//There is an interior intersection point
					{

						//insert (Ix, Iy) into i and j
						insertNewIntersectionPoint(h, writeBack, Ix, Iy, i, es, intersectionOccurs);
						insertNewIntersectionPoint(h, false, Ix, Iy, j, es, intersectionOccurs);
					}
					else if (s == 0 && t > 0 && t < 1) 	//intersect at mid i left j
						//insert (A1,B1) into i
						insertNewIntersectionPoint(h, writeBack, A1, B1, i, es, intersectionOccurs);
					else if (s == 1 && t > 0 && t < 1) 	//intersect at mid i right j
						//insert (A2,B2) into i
						insertNewIntersectionPoint(h, writeBack, A2, B2, i, es, intersectionOccurs);
					else if (t == 0 && s > 0 && s < 1) 	//intersect at mid j left i
						//insert (X1,Y1) into j
						insertNewIntersectionPoint(h, false, X1, Y1, j, es, intersectionOccurs);
					else if (t == 1 && s > 0 && s < 1) 	//intersect at mid j right i
						//insert (X2,Y2) into j
						insertNewIntersectionPoint(h, false, X2, Y2, j, es, intersectionOccurs);
				}
//				cout<<"coincide?:"<<coincide<<endl;
				return coincide;
			}
			void insertNewIntersectionPoint(statusHseg2D &shseg, bool writeBack, rat x, rat y, stu::dvector<statusHseg2D>::Iterator h, EventSchedule<T1,T2,T3,T4> &es, bool & intersectionOccurs)
			{
				intersectionOccurs = true;
//				cout<<"&&&&&&&&&&&& INSERTING ("<< x << ", "<<y<<")"<< " into "<<*(h.get())<<endl;
				poi2D newP(x, y);
				statusHseg2D sh;
				if(!h.end())
				{
					sh = statusHseg2D(*(h.get()->hseg), h.get()->owner);
					sh.m = h.get()->m;
					sh.n = h.get()->n;
				}
				hseg2D s = *(sh.hseg);
				if(s.getSegment().disjoint(newP)) //point is disjoint
					return;
				//seg2D s1(s.getDominating(), newP);
				//seg2D s2(newP, s.getSubmissive());
				hseg2D h1(s.getDominating(), newP,  sh.hseg->isInsideAbove());
				hseg2D h2(newP, s.getDominating(), !sh.hseg->isInsideAbove());
				hseg2D h3(newP, s.getSubmissive(), sh.hseg->isInsideAbove());
				hseg2D h4(s.getSubmissive(), newP, !sh.hseg->isInsideAbove());
				eventStruct *e1 = new eventStruct;
				e1->obj = (void *) &h1;
				e1->isPoi2D = false;
				eventStruct *e2 = new eventStruct;
				e2->obj = (void *) &h2;
				e2->isPoi2D = false;
				eventStruct *e3 = new eventStruct;
				e3->obj = (void *) &h3;
				e3->isPoi2D = false;
				eventStruct *e4 = new eventStruct;
				e4->obj = (void *) &h4;
				e4->isPoi2D = false;
				
				statusHseg2D newSh(h1, h.get()->owner);
				statusHseg2D newSh3(h3, h.get()->owner);
				//Preserve original overlap number
				newSh.m = h.get()->m;
				newSh.n = h.get()->n;
				
				//find existing seg in status, handle coincide situations
				bool found = false;
				stu::dvector<statusHseg2D>::Iterator it = status.getIterator();

				//Using Binary Search
				uint index;
				vectorBinarySearchWrapper< dvector<statusHseg2D>, statusHseg2D > binaryFinder( &status );
				found = binaryFinder.find(newSh, index);
				if(found)
				{
					it = status.getIterator(index);
					//reinsert into event sched
					eventStruct *e5 = new eventStruct;
					e5->obj = (void *)(it.get()->hseg);
					e5->isPoi2D = false;
					if(it.get()->owner == 'F')
						es.insertF(*(T3*)(e5->obj));
					else if(it.get()->owner == 'G')
						es.insertG(*(T4*)(e5->obj));
						//erase it from status
					status.erase(it);
						//insert first split into event sched
					if(sh.owner == 'F')
					{
						es.insertF(*(T3*)(e1->obj));
					}
					else if(sh.owner == 'G')
					{
						es.insertG(*(T4*)(e1->obj));
					}
						//remove it from status
					h.prev(); //since the first erase "it" is smaller than h, h moves down
					status.erase(h);
				}
				else
				{
					found = binaryFinder.find(newSh3, index);
					if(found)
					{
						it = status.getIterator(index);
						//reinsert into event sched
						eventStruct *e5 = new eventStruct;
						e5->obj = (void *)(it.get()->hseg);
						e5->isPoi2D = false;
						if(it.get()->owner == 'F')
							es.insertF(*(T3*)(e5->obj));
						else if(it.get()->owner == 'G')
							es.insertG(*(T4*)(e5->obj));
						//erase it from status
						status.erase(it);
						//update the seg in status
						status.erase(h);
						status.insert(h,newSh);
					}
				}
/*
				//Using Linear Search
				while(it.next())
				{
					if(*(it.get()->hseg) == h1)
					{
						found = true;
						//reinsert into event sched
						eventStruct *e5 = new eventStruct;
						e5->obj = (void *)(it.get()->hseg);
						e5->isPoi2D = false;
						if(it.get()->owner == 'F')
							es.insertF(*(T3*)(e5->obj));
						else if(it.get()->owner == 'G')
							es.insertG(*(T4*)(e5->obj));
						//erase it from status
						status.erase(it);
						//insert first split into event sched
						if(sh.owner == 'F')
						{
							es.insertF(*(T3*)(e1->obj));
						}
						else if(sh.owner == 'G')
						{
							es.insertG(*(T4*)(e1->obj));
						}
						//remove it from status
						h.prev(); //since the first erase "it" is smaller than h, h moves down
						status.erase(h);
						break;
					}
					else if(*(it.get()->hseg) == h3)
					{
						found = true;
						//reinsert into event sched
						eventStruct *e5 = new eventStruct;
						e5->obj = (void *)(it.get()->hseg);
						e5->isPoi2D = false;
						if(it.get()->owner == 'F')
							es.insertF(*(T3*)(e5->obj));
						else if(it.get()->owner == 'G')
							es.insertG(*(T4*)(e5->obj));
						//erase it from status
						status.erase(it);
						//update the seg in status
						status.erase(h);
						status.insert(h,newSh);
						break;
					}
				}
*/

				//Update h's left halfsegment in sweepStatus
				if(!found)
				{
					status.erase(h);
					status.insert(h,newSh);
				}

				//Writeback the new split
				if(writeBack)
				{
					shseg = newSh;
				}

				//Update and insert the new halfsegments in event sched
				//Can use binary search for O(log n) time
				if(sh.owner == 'F')
				{
//					cout<< "Adding to F"<<endl;
					es.insertF(*(T3*)(e2->obj));
					es.insertF(*(T3*)(e3->obj));
					es.insertF(*(T3*)(e4->obj));
				}
				else if(sh.owner == 'G')
				{
//					cout<< "Adding to G"<<endl;
					es.insertG(*(T4*)(e2->obj));
					es.insertG(*(T4*)(e3->obj));
					es.insertG(*(T4*)(e4->obj));
				}
			}
			rat sweepIntersect(rat x, statusHseg2D h)
			{
				rat result(-1.0);
				rat slope = rat(0);
				rat b(0.0);
				if(h.hseg->getDominating().getX() == x)
				{
					return h.hseg->getDominating().getY();
				}
				if(h.hseg->getSubmissive().getX() - h.hseg->getDominating().getX() == 0)
					result = h.hseg->getDominating().getY();
				else
				{
					rat y2 = h.hseg->getSubmissive().getY();
					rat y1 = h.hseg->getDominating().getY();
					rat x2 = h.hseg->getSubmissive().getX();
					rat x1 = h.hseg->getDominating().getX();
					rat n = y2 - y1;
					rat d = x2 - x1;
					slope = n/d;
					b = h.hseg->getSubmissive().getY() - slope * h.hseg->getSubmissive().getX();
					result = slope * x + b;
				}
				return result;
			}
	};
}
#endif
