#include "TopPred2D.h"
#include "Algorithm_pp.h"
#include "Algorithm_ll.h"
#include "Algorithm_rr.h"
#include "Algorithm_pl.h"
#include "Algorithm_pr.h"
#include "Algorithm_lr.h"

 /**
  * This file contains the implementation for the TopPred2DImpl class.
  * The implementation handles calculation of topological predicates
  * between spatial data types: point2D, line2D, and region2D.
  * \author Reasey Praing
  * \version 0.1
  * \date 2005-11-01
 * \version 0.2
 * \date 2006-10-31
 * \version 0.3
 * \date	2006-11-06 
 * \note for mslob
  */

namespace stu
{
using namespace stu;
// Unnamed Predicates

// Since we do not have identifiable name for most of these predicates,
// we use the third parameter to specify the ID number of the predicate.

// Predicates verifications
bool TopPred2D::topPred( point2D& p1,  point2D& p2,  int PointPredicateID)
{
	//Check Bounding Box Intersection
//	cout<<"hereE\n";
	Algorithm_pp pp(false);
//	cout<<"hereF\n";
	if(p1.getPointCount() > 1 && p2.getPointCount() > 1)
	{
//		cout<<"hereC\n";
		if(p1.getmbb().disjoint(p2.getmbb()))
		{
//			cout<<"hereD\n";
			if(pp.debug)
				cout << "\n********* Resulted by Minimum Bounding Box **********";
			return (PointPredicateID==1);	//disjoint
		}
	}
	if(pp.topPred(p1, p2, PointPredicateID) == PointPredicateID)
		return true;
	return false;
}

bool TopPred2D::topPred( line2D& l1,  line2D& l2,  int LinePredicateID)
{
	//Checking Bounding Box Intersection does not help since we have multiple disjoint predicates
	Algorithm_ll ll(false);
	if(ll.topPred(l1, l2, LinePredicateID) == LinePredicateID)
		return true;
	return false;
}

bool TopPred2D::topPred( region2D& r1,  region2D& r2,  int RegionPredicateID)
{
	//Check Bounding Box Intersection
	Algorithm_rr rr(false);
	if(r1.getmbb().disjoint(r2.getmbb()))
	{
		if(rr.debug)
			cout << "\n********* Resulted by Minimum Bounding Box **********";
		return (RegionPredicateID==1);	//disjoint
	}
	if(rr.topPred(r1, r2,RegionPredicateID) == RegionPredicateID)
		return true;
	return false;
}

bool TopPred2D::topPred( point2D& p,  line2D& l,  int PointLinePredicateID)
{
	//Checking Bounding Box Intersection does not help since we have multiple disjoint predicates
	Algorithm_pl pl(false);
	if(pl.topPred(p, l, PointLinePredicateID) == PointLinePredicateID)
		return true;
	return false;
}

bool TopPred2D::topPred( point2D& p,  region2D& r,  int PointRegionPredicateID)
{
	//Check Bounding Box Intersection
	Algorithm_pr pr(false);
	if(p.getPointCount() > 1)
	{
		if(p.getmbb().disjoint(r.getmbb()))
		{
			if(pr.debug)
				cout << "\n********* Resulted by Minimum Bounding Box **********";
			return (PointRegionPredicateID==1);	//disjoint
		}
	}
	if(pr.topPred(p, r, PointRegionPredicateID) == PointRegionPredicateID)
		return true;
	return false;
}
bool TopPred2D::topPred( line2D& l,  region2D& r,  int LineRegionPredicateID)
{
	//Checking Bounding Box Intersection does not help since we have multiple disjoint predicates
	Algorithm_lr lr(false);
	if(lr.topPred(l, r, LineRegionPredicateID) ==LineRegionPredicateID )
		return true;
	return false;
}


int TopPred2D::topPred( point2D& p1,  point2D& p2){
	return topPredHelper(p1,p2,-1);
}

int TopPred2D::topPred( line2D& l1,  line2D& l2){
	return topPredHelper(l1,l2,-1);
}

int TopPred2D::topPred( region2D& r1,  region2D& r2){
	return topPredHelper(r1,r2,-1);
}

int TopPred2D::topPred( point2D& p,  line2D& l){
	return topPredHelper(p,l,-1);
}

int TopPred2D::topPred( point2D& p,  region2D& r){
	return topPredHelper(p,r,-1);
}
	
int TopPred2D::topPred( line2D& l,  region2D& r){
	return topPredHelper(l,r,-1);
}


//Predicate Determination
int TopPred2D::topPredHelper(point2D& p1, point2D& p2, int v)
{
	//Check Bounding Box Intersection
//	cout<<"hereG\n";
	Algorithm_pp pp(false);
//	cout<<"hereH\n";
	if(p1.getPointCount() > 1 && p2.getPointCount() > 1)
	{
//		cout<<"hereI\n";
		if(p1.getmbb().disjoint(p2.getmbb()))
		{
//			cout<<"hereJ\n";
			if(pp.debug)
				cout << "\n********* Resulted by Minimum Bounding Box **********";
			return 1;	//disjoint
		}
	}
	return pp.topPred(p1, p2, v); 	//-1 means predicate determination
}
int TopPred2D::topPredHelper( line2D& l1,  line2D& l2, int v)
{
	//Checking Bounding Box Intersection does not help since we have multiple disjoint predicates
	Algorithm_ll ll(false);
	return ll.topPred(l1, l2, v);
}

int TopPred2D::topPredHelper( region2D& r1,  region2D& r2, int v)
{
	//Check Bounding Box Intersection
	Algorithm_rr rr(false);
	if(r1.getmbb().disjoint(r2.getmbb()))
	{
		if(rr.debug)
			cout << "\n********* Resulted by Minimum Bounding Box **********";
		return 1;	//disjoint
	}
	return rr.topPred(r1, r2, v);
}

int TopPred2D::topPredHelper(point2D& p, line2D& l, int v)
{
	//Checking Bounding Box Intersection does not help since we have multiple disjoint predicates
	Algorithm_pl pl(false);
	return pl.topPred(p, l, v);
}

int TopPred2D::topPredHelper( point2D& p,  region2D& r, int v)
{
	//Check Bounding Box Intersection
	Algorithm_pr pr(false);
	if(p.getPointCount() > 1)
	{
		if(p.getmbb().disjoint(r.getmbb()))
		{
			if(pr.debug)
				cout << "\n********* Resulted by Minimum Bounding Box **********";
			return 1;	//disjoint
		}
	}
	return pr.topPred(p, r, v);
}
int TopPred2D::topPredHelper( line2D& l,  region2D& r, int v)
{
	//Checking Bounding Box Intersection does not help since we have multiple disjoint predicates
	Algorithm_lr lr(false);
	return lr.topPred(l, r, v);
}
}
