#ifndef TOPPRED2D_H
#define TOPPRED2D_H

#include "point2D.h"
#include "line2D.h"
#include "region2D.h"
//#include "hseg2D.h"

//BYPASSING DATA TYPE REQUIREMENT
//typedef double point2D;
//typedef float line2D;
//typedef short region2D;
//typedef int hseg2D;

 /**
  * The class interface for 2-dimensional Topological Predicates.
  * The implementation handles calculation of topological predicates
  * between spatial data types: point2D, line2D, and region2D.
  * \author Reasey Praing
  * \version 0.1
  * \date	2005-10-05
  * \version 0.2
  * \date	2005-11-01
 * \version 0.3
 * \date	2006-11-09 
 * \note for mslob
  */

namespace stu
{
using namespace stu;
class TopPred2D
{
public:
	// Unnamed Predicates

	// Since we do not have identifiable name for most of these predicates,
	// we use the third parameter to specify the ID number of the predicate.

	// Predicates verifications
	static bool topPred( point2D&,  point2D&,  int);
	static bool topPred( line2D&,  line2D&,  int);
	static bool topPred( region2D&,  region2D&,  int);
	static bool topPred( point2D&,  line2D&,  int);
	static bool topPred( point2D&,  region2D&,  int);
	static bool topPred( line2D&,  region2D&,  int);

	//Predicate Determination for proper predicates
	static int topPred( point2D&,  point2D&);
	static int topPred( line2D&,  line2D&);
	static int topPred( region2D&,  region2D&);
	static int topPred( point2D&,  line2D&);
	static int topPred( point2D&,  region2D&);
	static int topPred( line2D&,  region2D&);

	//Predicate Determination for dimension refined preds
// 	static int DRtopPred( point2D&,  point2D&);
// 	static int DRtopPred( line2D&,  line2D&);
// 	static int DRtopPred( region2D&,  region2D&);
// 	static int DRtopPred( point2D&,  line2D&);
// 	static int DRtopPred( point2D&,  region2D&);
// 	static int DRtopPred( line2D&,  region2D&);

private:

	static int topPredHelper( point2D&,  point2D&, int v);
	static int topPredHelper( line2D&,  line2D&, int v);
	static int topPredHelper( region2D&,  region2D&, int v);
	static int topPredHelper( point2D&,  line2D&, int v);
	static int topPredHelper( point2D&,  region2D&, int v);
	static int topPredHelper( line2D&,  region2D&, int v);
		
	//static void sweepStatus( hseg2D&); // Plane sweep
	//static int matchPredicate( bool**); // Matching flag to predicate characterizations
	//static int matchRegionPredicate( bool***); // Matching overlap number to predicate characterizations for region objects

};
}
#endif //TOPPRED2D_H
