/** This class is used for representing blocks of line2D objects.
 * A block represents a single connected component of a line2D object.
 *	@see line2D
 *  \author Alejandro Pauly, Mark McKenney
 *  \version 0.1
 *  \date    2005-10-05
 *  \version 0.2
 *  \date    2005-10-14
 *  \version 0.3
 *  \date    2005-11-25 
 *  \version 0.4 Removed Impl stuff, fixing for mSlob
 *  \date    2006-10-13 
*/

#ifndef BLOCK2D_H
#define BLOCK2D_H

#include "hseg2D.h"
#include "seg2D.h"
#include "num.h"
#include "mSlob.h"
#include "line2D.h"
#include "invalidObjectException.h"

#include "vectorBinarySearchWrapper.h"

#include "statusHseg2D.h"
#include "EventSchedule.h"
#include "SweepLine.h"
#include "dvector.h"
//#include "sweepLineSingleStatus.h"
//#include "MEventSchedule.h"

namespace stu{

	using namespace stu;

	class line2D;
	
	class block2D{
		friend class line2D;
		
		public:
			class Iterator{ //iterates over each poi2D in list
				long int current;
				block2D* parent;
				
				public:
					Iterator(block2D* par);
					Iterator( );
					hseg2D* get();
					bool next();
					bool end(); 
					Iterator operator+( int num );
					Iterator operator-( int num );
					Iterator operator++( int dummy );
					Iterator operator--( int dummy );
					Iterator& operator+=( int num );
					Iterator& operator-=( int num );
			};
			
			block2D(); //default
			block2D(const line2D& l, const Locator& b); //create (initialize) empty or simply load if it already has something
			block2D(line2D& l); //add an empty block to l
			block2D(line2D& l, const block2D& b); //copy constructor, copy p as a new block of l

			~block2D();
				
			hseg2D operator[]( uint index );
			uint size() const; //the number of half segments
			
				//constructor ops
			void addSegment(const seg2D &s);
				
				//adds all segs, leaves checking to finalize
			void addSegments(const seg2D ss[], uint length); // throw (finalizedObjectException); 
				
			void removeSegment(const seg2D &s)  throw (undefinedSpatialObjectException);
				
			bool validate();
	
			bool isValid() const; 
				
			uint getSegmentCount() const; //returns the number of segments for the whole block
			rat getLength() const; //returns the perimeter. sum of lengths of segments
				
			bool isEmpty() const; //returns getSize()==0
		
			Iterator getIterator() const; //sorted in half-segment order
			
			//Output operator
			friend std::ostream &operator << (std::ostream &ostr, const block2D &rhs) throw( invalidObjectException );
		
			void walkBlock( hseg2D currHseg, uint &numVisited, vectorBinarySearchWrapper< vector<hseg2D>, hseg2D > &alreadyVisitedWrapper, vector<hseg2D> &alreadyVisitedVector, vectorBinarySearchWrapper< block2D, hseg2D > &blockSearchWrapper );
			
			bool addhseg(const hseg2D &h) throw (invalidObjectException);	
		
		private:
			//global is a locator to a global mSlob that represents a whole line, same as pointed by parent below
			Locator global;
			//elements is a locator to an element of a secondary structure of global..
			Locator elements;
			bool valid;
			bool changed; //to keep whether the object was changed and so the finalized flag needs to be rewritten
			bool inBatch; ////to avoid writing valid flag more than necessary
			line2D *parent; //pointer to its parent...necessary to add to primary
			
			static short int FLAGS;

			void saveValid();
			void loadValid();

			void removeMe(); //empties all subobjects of this block
				
			
/*			poi2D& findFirstHalfSeg();
			uint connectedFromHere(const hseg2D& origin, std::vector<hseg2D> & nothere);
			std::vector<hseg2D>& findConnectedHsegs(const hseg2D& current, std::vector<hseg2D> & nothere );
			void checkIntersections( hseg2D &H1, hseg2D &H2, std::vector<hseg2D> &vec,
									 MEventSchedule<hseg2D, hseg2D> EE, sweepLineSingleStatus<hseg2D> &SLSS );
			hseg2D& findTwin( hseg2D &sibling);*/
	};
}
#endif

