#include "convexHull.h"


namespace stu{
	
		using namespace stu;
		
		/* struct used for angle comparion for sorting on angle */
		struct _ST_COMPARE_ANGLE{
			poi2D pivot;
			bool operator ()(const poi2D& p1,const poi2D& p2) const
			{
				return ((p1.getX()-pivot.getX())*(p2.getY()-pivot.getY())
				    > (p2.getX()-pivot.getX())*(p1.getY()-pivot.getY()));
			}
	
			void setPivot(const poi2D& p) 
			{ 
				pivot.setXY(p.getX(), p.getY()); 
			}
			
		}ST_COMPARE_ANGLE;		



				

		void CConvexHull::getConvexHull(vector<poi2D>& vec, region2D& reg)
		{
	
			initialize(vec);

			int index = findPivot();

			/* remove pivot point */
			poi2D poi(m_poi_set.at(index));
			m_pivot.setXY(poi.getX(), poi.getY());
			m_poi_set.erase(m_poi_set.begin() + index);
			ST_COMPARE_ANGLE.setPivot(m_pivot);
	
			/* sort all the points based on the angle to pivot point*/
			sort(m_poi_set.begin(),m_poi_set.end(),ST_COMPARE_ANGLE);
			
			/* remove duplicate points on the same angle and only keep the farthest one */
			removeDuplicatesOnAngle();

			vector<poi2D> stack;
			stack.push_back(m_pivot);
			vector <poi2D>::iterator iter =m_poi_set.begin();
			stack.push_back(*iter);
			rat cross_result;
			int stack_size;
			iter++;
			while(iter!=m_poi_set.end()){
				stack_size = stack.size();
				cross_result = crossProduct(stack.at(stack_size-2),
							     stack.at(stack_size-1),
							     *iter);
				
				if(cross_result > rat(0)){
					/* good point add the point to the stack */	
					stack.push_back(*iter);
					iter++;
				}else{
					/* bad point: pop up the most recent point */
					stack.pop_back();
				}	
			}
			
			/* build a outer region for all the resulting points */
			buildOuterRegion(stack, reg);
		}
		
		int CConvexHull::findPivot()
		{
			/* scan the whole list to find the lowest right most point*/
			rat min_y;
			rat max_x;
			int iter_index =0;
			int index = 0;
			vector <poi2D>::iterator iter =m_poi_set.begin();
			min_y= iter->getY();
			max_x= iter->getX();
			iter++;
			for(;iter!=m_poi_set.end(); iter++){
				index++;
				if(iter->getY()< min_y	){
					min_y = iter->getY();
					max_x = iter->getX();
					iter_index = index;
				}else if(iter->getY() == min_y && iter->getX() > max_x){
					max_x = iter->getX();
					iter_index = index;
				}else{
				
				}
			}
			return iter_index;
		}


		void CConvexHull::removeDuplicatesOnAngle()
		{
			vector <poi2D>::iterator iter1 =m_poi_set.begin();
			vector <poi2D>::iterator iter2 = m_poi_set.begin();
			iter2++;
			rat dist1,dist2;
			for(;iter2!=m_poi_set.end(); iter2++){
				if((iter1->getX()-m_pivot.getX())*(iter2->getY()-m_pivot.getY()) == 
					(iter2->getX()-m_pivot.getX())*(iter1->getY()-m_pivot.getY())){
					/* duplicates */
					dist1= m_pivot.distance(*iter1);
					dist2= m_pivot.distance(*iter2);
					if(dist1>dist2){
						m_poi_set.erase(iter2);
					}else{
						m_poi_set.erase(iter1);
					}
					iter2= iter1;
				}else{
					iter1++;
				}
			}
			
		}


		void CConvexHull::buildOuterRegion(vector<poi2D>& vec, region2D& reg)
		{
			cout<<"########build region####"<<endl;
			vector <poi2D>::iterator iter1 =vec.begin();
			vector <poi2D>::iterator iter2 =vec.begin();
			iter2++;
			poly2D poly;
			seg2D seg;
			while(iter2!=vec.end()){
				seg.setPQ(*iter1, *iter2);
				cout<<seg<<endl;
				if(!poly.addSegment(seg)){
					cout<<"add seg error"<<endl;
				}
				
				iter1++;
				iter2++;
			}
			iter2= vec.begin();
			seg.setPQ(*iter1, *iter2);
			poly.addSegment(seg);
			
			poly.validate();
		
			reg.addFace().addOuterCycle(poly);
			reg.validate();
		}
		

	
		void CConvexHull::initialize(vector<poi2D>& vec)
		{

			vector<poi2D>::iterator iter = vec.begin();
			for(;iter!= vec.end(); iter++){
				m_poi_set.push_back(*iter);
			}

		}
		

		/** for debug
		**/
		void CConvexHull::print(vector<poi2D>& vec)
		{
			cout<<"########print_point_set####"<<endl;
			vector <poi2D>::iterator iter =vec.begin();
			for(;iter!=vec.end(); iter++){
				cout<<*iter<<endl;
			}
		}

	

}


