/** This class is for computing convex hull of all the points 
    in point2D, line2D, region2D.
 *  \author Jing sun
 *  \version 0.1
 *  \date    2007-07-26
 */

#ifndef CONVEX_H
#define CONVEX_H

#include "constants.h"
#include "num.h"
#include "invalidObjectException.h"
#include "hseg2D.h"
#include "mSlob.h"
#include "poly2D.h"

namespace stu{
	using namespace stu;

	class region2D;
	class poi2D;


	class CConvexHull{
	
		public:
	
			CConvexHull() {;}
			~CConvexHull() {;}
			void getConvexHull(vector<poi2D>& vec, region2D& reg);
			
			/*pivot point is the most rightdown point */ 
			poi2D getPivot() {return m_pivot;}
		
		private:
			/* sort on points on the angle(the angle is formed from any point v to the pivot point) */
			void sortPoiOnAngle();
			
			int findPivot();
			
			/* all the points that have the same angle value, 
			remove duplicates and only keep the most farthest one */
			void removeDuplicatesOnAngle();
			
			rat crossProduct(const poi2D& p1,const poi2D& p2,const poi2D& p3)
			{
				return (p2.getX() - p1.getX())*(p3.getY() - p1.getY()) - 
					(p3.getX() - p1.getX())*(p2.getY() - p1.getY());
			}

			void print(vector<poi2D>& vec);
			void initialize(vector<poi2D>& vec);

			/* build region for the final result */
			void buildOuterRegion(vector<poi2D>& vec, region2D& reg);
			
			/* pivot point */
			poi2D m_pivot;
			
			/* all the points to be computed in the algorithm: input point set */
			vector<poi2D> m_poi_set;
	};

}
#endif


