#include "delaunay.h"


#define MAX_POINT_VALUE 300
#define MBB_SIZE_MIN_X  0
#define MBB_SIZE_MAX_X  100
#define MBB_SIZE_MIN_Y  0
#define MBB_SIZE_MAX_Y 	100  


namespace stu{
	
	using namespace stu;

	/******************************************************************************
	Implementation for Edge
	Edge has two points and Edge has a pointer to Triangles means that this edge 
	belongs to which triangle.
	*******************************************************************************/
	void CEdge::removeTriPtr(const CTriangle* p)
	{
		bool hit = false;
		for (int i =0; i<2; i++){
			if(mp_tri_ptr[i] == p){
				mp_tri_ptr[i] = NULL;
				hit = true;
				break;
			}
		}
		if(!hit) cout<<*this<<"<removeTriPtr> Not find the tri pointer"<<endl;
	
	}
	
	void CEdge::addTriPtr(const CTriangle* p)
	{
		
		bool hit = false;
		if(mp_tri_ptr[0]==p ||mp_tri_ptr[1]==p){
			// already existed
			return;
		}
		for (int i =0; i<2; i++){
			if(mp_tri_ptr[i] == NULL){
				mp_tri_ptr[i] =(CTriangle*) p;
				hit = true;
				break;
			}
		}
		if(!hit) {
			cout<<*this<<"<addTriPtr> error no space to set the tri pointer"<<endl;
			cout<<mp_tri_ptr[0]<<"\t"<<mp_tri_ptr[1]<<"\t"<<p<<endl;
		}
	}
	
	
	
	CTriangle*  CEdge::findNeigb(const CTriangle* p)
	{
		if(p == mp_tri_ptr[0])
			return mp_tri_ptr[1];
		if(p == mp_tri_ptr[1])
			return mp_tri_ptr[0];
		return NULL;
	}
	
	bool CEdge::getNeigbTriVertex(poi2D& p1, poi2D& p2)
	{
	
		if(!mp_tri_ptr[0]||!mp_tri_ptr[1])
			return false;
		
		p1 = mp_tri_ptr[0]->getAnotherVertex(m_p1, m_p2);
		p2 = mp_tri_ptr[1]->getAnotherVertex(m_p1, m_p2);
		return true;
		
	}
	
	
	/******************************************************************************
	Implementation for EdgeList
	The list for all Edges
	*******************************************************************************/
	CEdgeList m_edge_list_inst;
	CEdgeList * CEdgeListGetInstance(){
		return &m_edge_list_inst;
	}
	
	
	CEdge* CEdgeList::addEdge(const poi2D p1, const poi2D p2)
	{
		int index = m_list.size();
		CEdge* e= new CEdge(p1,p2, index,true);
		m_list.push_back(e);
		return e;
	}
	
	
	void CEdgeList::clearEdges()
	{
		vector <CEdge*>::iterator iter =m_list.begin();
		for(;iter!= m_list.end();iter++){
			delete *iter;
		}
		m_list.clear();
	}
	
	
	void CEdgeList::print()
	{
		cout<<"***** edge list****"<<endl;
		vector <CEdge*>::iterator iter =m_list.begin();
		for(;iter!= m_list.end();iter++){
			cout<<**iter;
		}
	
	}
	
	/******************************************************************************
	Implementation for CTriangle
	Triangle has 3 edges which are pointed to the edges in EdgeList
	*******************************************************************************/
	
	CTriangle::CTriangle(const CEdge* e1, const CEdge* e2, const CEdge* e3)
	{
		setEdges(e1,e2,e3);
	
	}
	
	
	void CTriangle::setEdges(const CEdge* e1, const CEdge* e2, const CEdge* e3)
	{
		m_edge[0] = (CEdge*)e1;
		m_edge[1] = (CEdge*)e2;
		m_edge[2] = (CEdge*)e3;
	
	}
	
	
	void CTriangle::getEdges(CEdge** e1,  CEdge** e2, CEdge** e3) const
	{
	
		*e1 = this->m_edge[0];
		*e2 = this->m_edge[1];
		*e3 = this->m_edge[2];
	}
	
	
	void CTriangle:: getPoints(poi2D& p1, poi2D& p2, poi2D& p3) const 
	{ 
		poi2D poi[4];
	
		m_edge[0]->getPoints(poi[2],poi[3]);
		p1 = poi[2];
		p2 = poi[3];
		m_edge[1]->getPoints(poi[0],poi[1]);
		if(poi[0] != poi[2] && poi[0]!= poi[3]){
			p3= poi[0];
		}else{
			p3 = poi[1];
		}
	
	}
	
	
	void CTriangle::addChild(CTriangle* p) 
	{
		m_child.push_back(p);
	}
	
	poi2D CTriangle::getAnotherVertex(poi2D p1, poi2D p2) const
	{
		poi2D poi[3];
		getPoints(poi[0], poi[1], poi[2]);
		int k = -1;
		bool hit1 = false;
		bool hit2 = false;
		for(int i =0; i<3; i++){
			if(poi[i] != p1 && poi[i] != p2){
				k =i;
			}else if(poi[i] == p1){
				hit1 = true;
			}else if(poi[i] == p2){
				hit2 = true;
			}
		}
		if(!hit1||!hit2 || k<0){
			cout<<"[getAnotherVertex] !!illegal point"<<p1<<p2<<*this<<endl;
			return poi2D(-100,-100);
		}
		return poi[k];
	}
	
	
	
	CEdge* CTriangle::getEdge(const poi2D p1, const poi2D p2) const
	{
		poi2D poi[2];
		for(int i =0; i<3; i++){
			m_edge[i]->getPoints(poi[0],poi[1]);
			if((poi[0] == p1 && poi[1] == p2) ||
				(poi[0] == p2 && poi[1] == p1))
				return m_edge[i];
		}
		cout<<"[getEdge] error no edge exist!"<<p1<<p2<<*this<<endl;
		return NULL;
	}
	
	
	void CTriangle::removeEdgeTriPtr()
	{
		for(int i =0; i<3; i++){
			m_edge[i]->removeTriPtr(this);
		}
	
	}
	
	bool CTriangle::isPoiInCircleTriangle(poi2D pl)
	{
	
		poi2D pi,pj,pk;
		getPoints(pi,pj,pk);
		poi2D a, b,c;
		if(pointToLine(pk,pi,pj)== pointToLine(pl, pi,pj)) {
			a= pi; c =pj; b = pk;
		}else if(pointToLine(pj,pi,pk)== pointToLine(pl, pi,pk)){
			a= pi; c =pk; b = pj;
		}else if(pointToLine(pi,pj,pk)== pointToLine(pl, pj,pk)){
			a= pj; c =pk; b = pi;
		}else{
			cout<<"error pl not on the same side of pk" <<endl;
		}

		rat ab= distance(a,b);
		rat bc= distance(b,c);
		rat ac = distance(a,c);
		rat pa = distance(pl, a);
		rat pc = distance(pl, c);
		rat ang = ab + bc - ac;
		rat pang = pa +pc -ac;
		if(ang > 0 && pang <=0) return true;
		if(ang <=0  && pang >=0) return false;

		rat da = ang*(rat(pa*pc).sqrt());
		rat dp = pang*(rat(ab*bc).sqrt());
		rat dc = da - dp;
		if(dc>0) return true;
		else return false;
			
	}
	
	
	bool CTriangle::isPoiOnEdges(poi2D pr, CEdge** pe)
	{
	
		poi2D poi[2];
		for(int i =0; i<3; i++){
			m_edge[i]->getPoints(poi[0],poi[1]);
			if(pointToLine(pr,poi[0],poi[1])==0 
				&& inBetween(pr.getX(),poi[0].getX(),poi[1].getX())){
				cout<<"point on edge"<<pr<<*m_edge[i]<<endl;
				*pe = m_edge[i];
				return true;
			}
		}
		return false;
	}
	
	/**
	0: on the edges of triangle
	1: inside the triangle
	-1: outside the triangle
	**/
	bool CTriangle::isPoiInOnTri( const poi2D pr, int& res, CEdge** e) 
	{
		
		poi2D p0,p1,p2; 
		getPoints(p0,p1,p2);
		if(isPoiOnEdges(pr,e)) 
		{ 
			res = 0; 
			//cout<<pr<<"on"<<*this;
			return true;
		}
		if(pointInAngle(pr,p0,p1,p2)&&pointInAngle(pr,p1,p0,p2)){
			//cout<<pr<<"in"<<*this;
			res = 1;
			return true;
		}
		//cout<<pr<<"outside"<<*this;
		return false;
	}
	
	

	/* 
	primitive test on whether q(xq,yq) falls in the angle from 
	op1((xo,yo)->(x1,y1)) to op2((xo,yo)->(x2,y2))
	
	false	outside
	true inside
	*/
	bool CTriangle::pointInAngle(poi2D pr, poi2D p0,poi2D p1,poi2D p2)
	{
	        poi2D p11(p1.getX()-p0.getX(), p1.getY()-p0.getY());
		poi2D p22(p2.getX()-p0.getX(), p2.getY()-p0.getY());

		rat a = dot(pr,p22)-dot(p0,p22);
		rat c = dot(p11,p22);
		rat b = dot(pr,p11)-dot(p0,p11);
		if(c > 0){
			if(a > 0 && b < 0) return true;
		}else if(c < 0){
			if(a < 0 && b > 0) return true;
		}
		return false;
	}
	
	
	/**
	is xq in between x1,x2, assume xq not eq x1 or x2
	**/
	bool CTriangle::inBetween(rat xq,rat x1,rat x2)
	{
	
		return((xq-x1)*(xq-x2)<=0);
	}
	
		
	/**
	Point on the side of radial line: (x,y)'s position according to 
	radial line (ox,oy) to (x1,y1)
	return value: 
	-1 left
	0 on line
	1 right
	**/
	int	CTriangle::pointToLine(const poi2D pr, const poi2D po, const poi2D p1)
	{
		
		rat x = pr.getX();
		rat y = pr.getY();
		rat ox = po.getX();
		rat oy = po.getY();
		rat x1 = p1.getX();
		rat y1 = p1.getY();

		// translation, o(x1,y1) as the new Oriental point
		x=x-ox;	
		y=y-oy;
		x1=x1-ox;
		y1=y1-oy;
	
		rat D=y1*x-x1*y;
		if (D != 0)
			return (D > 0) ? 1 : -1;
		else
			return 0;
 
	}
	

	
	
	/******************************************************************************
	Implementation for class CDelaunay
	Used to compute Delaunay Triangulation
	*******************************************************************************/

	CDelaunay gp_delaunay;
	CDelaunay * CDelaunayGetInstance()
	{
		return &gp_delaunay;
	}

	
	
	void CDelaunay::insertPoint(const poi2D pr)
	{
		CTriangle* p_tri;
		CTriangle* neigb_tri; 
		poi2D pi,pj, pk, pl; 
		CEdgeList* p_list = CEdgeListGetInstance();
		CEdge * pe;
		int res;
		p_tri = findTriContainsPoi(pr, res,&pe);
		if(!p_tri) {
			cout<<pr<<"<insertPoint> !ERROR not find the right triangle"<<endl;
			return;
		}
		if(res == 0 ){
			/* find the neighbour triangle of p_tri share the edge (pi, pj)
				return another point pl
			*/
			pe->getPoints(pi,pj); 
			neigb_tri = pe->findNeigb(p_tri);
			if(!neigb_tri) {
				cout<< " error!<insertPoint> there is no neighbor of tri"<<endl;
				return;
			}
			/* find another vertex of triangle except pi, pj */
			pk= p_tri->getAnotherVertex(pi,pj);
			pl= neigb_tri->getAnotherVertex(pi,pj);

			/* splitting the 2 triangle into 4 */
			CEdge* e1 = p_list->addEdge(pi,pr);
			CEdge* e2 = p_list->addEdge(pj,pr);
			CEdge* e3 = p_list->addEdge(pk,pr);
			CEdge* e4 = p_list->addEdge(pl,pr);
			pe->setValid(false);
	
			CEdge* edge[4];
			CTriangle* tri[4];
			int i;
			edge[0] = p_tri->getEdge(pk, pi);
			edge[1] = p_tri->getEdge(pk, pj);
			edge[2] = neigb_tri->getEdge(pl, pi);
			edge[3] = neigb_tri->getEdge(pl, pj);
	
			p_tri->removeEdgeTriPtr();
			neigb_tri->removeEdgeTriPtr();
	
			tri[0] = makeTriangle(edge[0], e1,e3);
			tri[1] = makeTriangle(edge[1], e3,e2);
			p_tri->addChild(tri[0]);
			p_tri->addChild(tri[1]);
			tri[2] = makeTriangle(edge[2], e1,e4);
			tri[3] = makeTriangle(edge[3], e2,e4);
			neigb_tri->addChild(tri[2]);
			neigb_tri->addChild(tri[3]);
	
	
			for( i =0; i<4; i++){
				legalizeEdge(pr, edge[i],tri[i]);
			}
	
		}else{
			/* pr inside the triangle  split into 3 triangles*/
			p_tri->getPoints(pi,pj,pk);
	
			CEdge* e1 = p_list->addEdge(pi,pr);
			CEdge* e2 = p_list->addEdge(pj,pr);
			CEdge* e3 = p_list->addEdge(pk,pr);
		
			CEdge* edge[3];
			CTriangle* tri[3];
			int i;
			p_tri->removeEdgeTriPtr();
	
			edge[0] = p_tri->getEdge(pi, pj);
			tri[0] = makeTriangle(edge[0], e1,e2);
			edge[1] = p_tri->getEdge(pj, pk);
			tri[1] = makeTriangle(edge[1], e2,e3);
			edge[2] = p_tri->getEdge(pk, pi);
			tri[2] = makeTriangle(edge[2], e3,e1);
			for(i=0; i<3; i++)
				p_tri->addChild(tri[i]);
	
			for( i =0; i<3; i++){
				legalizeEdge(pr, edge[i],tri[i]);
			}
		}
	
	}
	
	
	CTriangle* CDelaunay::makeTriangle(CEdge* e1, CEdge* e2, CEdge* e3)
	{
		
		CTriangle* tri= new CTriangle(e1,e2,e3);
		//cout<<"make triangle"<<tri<<*tri<<endl;
		e1->addTriPtr(tri);
		e2->addTriPtr(tri);
		e3->addTriPtr(tri);
	
		return tri;
	}
	
	
		
	/*after add a point, legalize the edge e of the traingle that contains the point 
	with adjacent triangle on the edge e. */
	void CDelaunay::legalizeEdge(const poi2D pr, CEdge* eij, CTriangle* p_tri)
	{
		poi2D pi,pj,pk,pl;
		int di, dj,dk, dl;
		eij->getPoints(pi,pj);
		pl = pr;
		CTriangle* neib_tri= eij->findNeigb(p_tri);
		if(!neib_tri) return;
		pk = neib_tri->getAnotherVertex(pi,pj);

		if(!neib_tri||neib_tri == p_tri) return;
		pk = neib_tri->getAnotherVertex(pi,pj);
		if(pk == pl){
			cout<<"error! the same triangle"<<endl;
			cout<<*p_tri<<*neib_tri<<endl;
		}
		di = getSpecialPointIndex(pi);
		dj = getSpecialPointIndex(pj);
		dk = getSpecialPointIndex(pk);
		dl = getSpecialPointIndex(pl);
		bool legal = true;

		if(di <0 && dj<0) {
			/* edge of special triangle legal edge */
		
		}else if(dk>0 && dl>0){
			if(di>0 && dj>0){
				/* all positive, illegal if pl lies inside the cycle of neib_tri */
				if(neib_tri->isPoiInCircleTriangle(pl))
					legal = false;
			}else if (di * dj <0){
				/* exact one point is negative index */
				if(neib_tri->isPoiInCircleTriangle(pl))
					legal = false;
			}
		}else if(dk*dl <0 && di*dj <0){
			if(neib_tri->isPoiInCircleTriangle(pl))
				legal = false;
		}

		if(legal) return;
	
		/* illegal Edge to do flipEdges(i--->j) fliped to (l---->k) */
		p_tri->removeEdgeTriPtr();
		neib_tri->removeEdgeTriPtr();
		eij->setValid(false);
		CEdge* ekl = CEdgeListGetInstance()->addEdge(pl,pk);
		CEdge* eil = p_tri->getEdge(pi,pl);
		CEdge* ejl = p_tri->getEdge(pj,pl);
		CEdge* eik = neib_tri->getEdge(pi,pk);
		CEdge* ejk = neib_tri->getEdge(pj,pk);
		
		CTriangle* tri[2];
		tri[0] = makeTriangle(eil, eik, ekl);
		tri[1] = makeTriangle(ejl,ejk, ekl);
		for(int i =0; i<2; i++){
			p_tri->addChild(tri[i]);
			neib_tri->addChild(tri[i]);
		}
		legalizeEdge(pl, eik, tri[0]);
		legalizeEdge(pl, ejk, tri[1]);
		
	}
	
	
	/**
	locate the Triangle that contains the point pr
	Search the tree until the leaf node that contains the point is found.
	0: on the edges of triangle
	1: inside the triangle
	-1: outside the triangle
	**/
	CTriangle* CDelaunay::findTriContainsPoi(const poi2D pr, int& res, CEdge** e)
	{
		CTriangle* p = &m_tri_tree;
		if(p->isPoiInOnTri(pr,res,e) && p->isLeaf()) return p;
		CTriangle* r;
		while(1){
			// scan the list of children
			int i;
			bool hit = false;
			for(i =0; i<p->getChildNum(); i++){
				r = p->getIthChild(i);
				if(r->isPoiInOnTri(pr,res,e)){
					p = r;
					if(r->isLeaf())
						return r;
					hit = true;
					break;
				}
			}
			if(!hit){
				cout<<pr<< "Error! can not locate the point"<<endl;
				res = -1;
				return NULL;
			}
		}
	
		return NULL;
	}
	
	
	void CDelaunay::initialize(vector<poi2D>& vec)
	{

		if(m_poi_set.size()) m_poi_set.clear();
		vector<poi2D>::iterator iter = vec.begin();
		for(;iter!= vec.end(); iter++){
			m_poi_set.push_back(*iter);
		}
		setBoundaryTri();
	}
	
	/* set boundary triangle that contains all the point set in it
	3 points:
	(MAX_POINT_VALUE, 0),
	(0, MAX_POINT_VALUE),
	(-MAX_POINT_VALUE, -MAX_POINT_VALUE)	
	*/
	void CDelaunay::setBoundaryTri()
	{

	#if 0
		/* find the absolute vale of the points to make a biggest triangle*/
		int max=0;
		int value;
		vector <poi2D>::iterator iter =m_poi_set.begin();
	
		for(;iter!=m_poi_set.end(); iter++){
			value = abs(iter->getX());
			if(value > max){
				max = value;
			}
			value = abs(iter->getY());
			if(value > max){
				max = value;
			}
		}
		max = max * 3;
	#endif
		
		int max = MAX_POINT_VALUE;
		m_max_value = max;
		CEdgeList* p_list = CEdgeListGetInstance();
		CEdge* e1 = p_list->addEdge(poi2D(rat(-max,1),rat(-max,1)), poi2D(rat(0,1),rat(max,1)));
		CEdge* e2 = p_list->addEdge(poi2D(rat(-max,1),rat(-max,1)), poi2D(rat(max,1),rat(0,1)));
		CEdge* e3 = p_list->addEdge(poi2D(rat(0,1),rat(max,1)), poi2D(rat(max,1),rat(0,1)));
		
		m_tri_tree.setEdges(e1, e2, e3);
		e1->addTriPtr(&m_tri_tree);
		e2->addTriPtr(&m_tri_tree);
		e3->addTriPtr(&m_tri_tree);
	
		
		
	}
	
	int CDelaunay::getSpecialPointIndex(const poi2D& poi)
	{
		poi2D poi1(rat(m_max_value,1),rat(0,1));
		poi2D poi2(rat(0,1),rat(m_max_value,1));
		poi2D poi3(rat(-m_max_value,1),rat(-m_max_value,1));
	
		if(poi == poi1) 
			return -1;
		if(poi == poi2 ) 
			return -2;
		if(poi == poi3 )
			return -3;
		return 1;
	}



	void CDelaunay::triangulation(vector<poi2D>& vec, line2D& reg)
	{
		initialize(vec);
		vector <poi2D>::iterator iter =m_poi_set.begin();
		for(;iter!=m_poi_set.end(); iter++)
		{
			insertPoint(*iter);
		}
		buildRegion(reg);
		finalize();
	}
	
	
	void CDelaunay::voronoi(vector<poi2D>& vec, line2D& ps)
	{
		cout<<"voronoi"<<endl;
		initialize(vec);
		vector <poi2D>::iterator iter =m_poi_set.begin();
		for(;iter!=m_poi_set.end(); iter++)
		{
			insertPoint(*iter);
		}
		poi2D poi[4];
		CEdgeList* p_list = CEdgeListGetInstance();
		CEdge* e = p_list->begin();
		CEdge* eend = p_list->end();
		block2D b1 = ps.addBlock();
		while( e!= eend){
			if(!e->isValid()) {
				e = p_list->next();			
				continue;
			}
			e->getPoints(poi[0], poi[2]);
			
			int idx0 = getSpecialPointIndex(poi[0]);
			int idx2 = getSpecialPointIndex(poi[2]);
			if(idx0 <0 || idx2 <0){
				e = p_list->next();	
				continue;
			}
			if(!e->getNeigbTriVertex(poi[1],poi[3])){
				e = p_list->next();
				continue;
			}
			
			poi2D poi1 = getBisectPoint(poi[0],poi[1],poi[2]);
			poi2D poi2 = getBisectPoint(poi[0],poi[2],poi[3]);
			/* set the boundary for the intersect points */
/*
			poi2D poi3, poi4;
			if(boundaryMbb(poi1, poi2, poi3)&&boundaryMbb(poi2, poi3,poi4)){
				seg2D ns(poi3,poi4);
				b1.addSegment(ns);
			}
*/
			seg2D ns(poi1,poi2);
			b1.addSegment(ns);
			e = p_list->next();
		}
		b1.validate();
		ps.validate();	
		finalize();
	}

	bool CDelaunay::boundaryMbb(const poi2D poi1,const poi2D poi2,poi2D& poi3)		
	{
		rat x1,x2,y1,y2;
		x1 = poi1.getX();
		y1 = poi1.getY();
		x2 = poi2.getX();
		y2 = poi2.getY();
		rat x3, y3;

		if((x1>=MBB_SIZE_MIN_X && x1<=MBB_SIZE_MAX_X ) &&
		   (y1>=MBB_SIZE_MIN_Y && y1<=MBB_SIZE_MAX_Y)){
			poi3.setXY(x1, y1);
			return true;
		}
		if(x1< MBB_SIZE_MIN_X && x2< MBB_SIZE_MIN_X) return false;
		if(x1> MBB_SIZE_MAX_X && x2> MBB_SIZE_MAX_X) return false;
		if(y1< MBB_SIZE_MIN_Y && y2< MBB_SIZE_MIN_Y) return false;
		if(y1> MBB_SIZE_MAX_Y && y2> MBB_SIZE_MAX_Y) return false;

		rat slope = (y2-y1)/(x2-x1);
		rat slope2 = rat(1)/slope;
		if(x1< MBB_SIZE_MIN_X){
			x3 = MBB_SIZE_MIN_X;
			y3 = y1-slope*x1;
		
		}else if(x1> MBB_SIZE_MAX_X){
			x3 = MBB_SIZE_MAX_X;
			y3 = y1+slope*(x3-x1);
		}
		x1 = x3;
		y1 = y3;
		if(y1< MBB_SIZE_MIN_Y){
			y3 = MBB_SIZE_MIN_Y;
			x3 = x1+ slope2*(y3-y1);
		
		}else if(y1> MBB_SIZE_MAX_Y){
			y3 = MBB_SIZE_MAX_Y;
			x3 = x1+ slope2*(y3-y1);
		}
		poi3.setXY(x3,y3);
		
		return true;
	}


	poi2D CDelaunay::getBisectPoint(const poi2D& p00, const poi2D& p11, const poi2D& p22)
	{
	
		poi2D p1(p11.getX()-p00.getX(), p11.getY()-p00.getY());
		poi2D p2(p22.getX()-p00.getX(), p22.getY()-p00.getY());
		rat xc, yc;
		rat x1,y1,x2,y2;
		x1 = p1.getX();
		y1 = p1.getY();
		x2 = p2.getX();
		y2 = p2.getY();
		
		rat dist1= x1*x1+y1*y1;
		rat dist2= x2*x2+y2*y2;
		rat cross = x1*y2- y1*x2;
		if(cross == 0) {
			cout<<"colinear lines"<<endl;
			cout<<p00<<p11<<p22<<endl;
			return poi2D(0,0);
		}
		if(y1 == 0){
			xc = x1/rat(2);
			yc = (dist2-rat(2)*x2*xc)/(rat(2)*y2);
		}else if(y2 == 0){
			xc = x2/rat(2);
			yc = (dist1- rat(2)*x1*xc)/(rat(2)*y1);
		}else{	
			xc = (y2*dist1 -y1*dist2)/(rat(2)*cross);
			yc = (dist1-rat(2)*xc*x1)/(rat(2)*y1);
		}
		
		rat xcc= xc+p00.getX();
		rat ycc= yc+p00.getY();
	
		
		return poi2D(xcc, ycc);
	}


	void CDelaunay::print()
	{
		
		CTriangle *p = &m_tri_tree;
		print(p);
		CEdgeListGetInstance()->print();
	
	}

	void CDelaunay::buildRegion(line2D& ps)
	{
		cout<<"!!!!!!build region begin"<<endl;

		CEdgeList* p_list = CEdgeListGetInstance();
		CEdge* e = p_list->begin();
		CEdge* eend = p_list->end();
		poi2D poi[2];	
		int idx1, idx2;
		block2D b1 = ps.addBlock();
		while( e!= eend){
			if(!e->isValid()) {
				e = p_list->next();			
				continue;
			}
			e->getPoints(poi[0], poi[1]);
			idx1 = getSpecialPointIndex(poi[0]);
			idx2 = getSpecialPointIndex(poi[1]);
			if(idx1 <0 || idx2 <0){
				e = p_list->next();	
				continue;
			}
			seg2D ns(poi[0],poi[1]);
			b1.addSegment(ns);
			e = p_list->next();
		}
		b1.validate();
		ps.validate();	
		
		cout<<"!!!!!build region end"<<endl;
	}


	void CDelaunay::getAllLeafTri( CTriangle* p, vector<CTriangle*> &vec)
	{
		if(p->isLeaf()) {
			vec.push_back(p);
			return;
		}
		int i;
		CTriangle* r;
		for(i =0; i<p->getChildNum(); i++){
			r = p->getIthChild(i);
			getAllLeafTri(r, vec);
		}
		
	}
	
	
	
	void CDelaunay::print(CTriangle* p)
	{
		CTriangle* r;
		int i;
		if(p->isLeaf()) cout<<*p;
		for(i =0; i<p->getChildNum(); i++){
			r = p->getIthChild(i);
			print(r);
		}
	}
	
	
	void CDelaunay::finalizeTree(CTriangle* p)
	{
		for(int i =0; i<p->getChildNum(); i++){
			p = p->getIthChild(i);
			finalizeTree(p);
		}
		p->clearTree();
	
	}
	
	void CDelaunay::finalize()
	{
		m_poi_set.clear();
		finalizeTree(&m_tri_tree);
		CEdgeListGetInstance()->clearEdges();
	
	}
	
	
}





