/** This class is for computing delaunay triangluation of all the points in point2D, line2D, region2D.
 *  \author Jing sun
 *  \version 0.1
 *  \date    2007-07-26
 */


#ifndef H_DELAUNAY
#define H_DELAUNAY


#include "constants.h"
#include "num.h"
#include "hseg2D.h"
#include "mSlob.h"
#include "poly2D.h"
#include "face2D.h"
#include "point2D.h"
#include "line2D.h"


namespace stu{
	using namespace stu;

	class poly2D;
	class face2D;
	class line2D;
	class poi2D;
	class CTriangle;


	class CEdge
	{
	public:
		CEdge() {mp_tri_ptr[0]= mp_tri_ptr[1] =NULL;}
		CEdge(const poi2D p1, const poi2D p2):m_p1(p1),m_p2(p2)
				{mp_tri_ptr[0]= mp_tri_ptr[1] =NULL;}
		
		CEdge(const poi2D p1, const poi2D p2, int i, bool va)
		:m_p1(p1), m_p2(p2), m_index(i),m_valid(va) 
				{mp_tri_ptr[0]= mp_tri_ptr[1] =NULL;}
		
		~CEdge() {;}
		
		void setPoints(const poi2D p1, const poi2D p2)
		{this->m_p1=p1; this->m_p2=p2; }
		
		void getPoints(poi2D& p1, poi2D& p2)const
		{p1= this->m_p1; p2= this->m_p2;}
		
		/* the index for edge */
		int  getIndex() { return m_index;}
		void setIndex(int i) { m_index = i;}
		void setValid(bool vl){m_valid = vl;}
		bool isValid() { return m_valid;}
		
		/*if the edge is valid and is used in one triangle
		  add a triPointer to edge
		  each edge can have at most 2 triangle pointer which are neigboured triangle.
		  This struct is used to relate the relation between edge and triangle.
		  It is easy to find a neib of any triangle.  */
		void removeTriPtr(const CTriangle* p);
		void addTriPtr(const CTriangle* p);
		CTriangle* findNeigb(const CTriangle* p);
		bool getNeigbTriVertex(poi2D& p1, poi2D& p2);

		friend ostream &operator << (ostream &ostr, const CEdge &rhs)
		{
			poi2D poi[2];
			rhs.getPoints(poi[0],poi[1]);
			ostr<<"EDGE: "<<poi[0]<<"----"<< poi[1]<<endl;
			return ostr;
		}
		
	private:
		poi2D m_p1;
		poi2D m_p2;
		int m_index;
		bool m_valid;
		CTriangle* mp_tri_ptr[2];
	};



	class CEdgeList{
	
	public:
		CEdgeList() {;}
		~CEdgeList() {;}
		CEdge* addEdge(const poi2D p1, const poi2D p2);
		void removeEdge(CEdge* p) {p->setValid(false);}
		void clearEdges(); 
		void print();
		CEdge* begin() { m_list_iter = m_list.begin(); return *m_list_iter;}
		CEdge* next()  { m_list_iter++; return * m_list_iter;}
		CEdge* end()   { return *(m_list.end());}
	private:
		vector<CEdge*> m_list;
		vector<CEdge*>::iterator m_list_iter;
		
	};
	
	extern  CEdgeList * CEdgeListGetInstance();
	
	/* the tree structure for triangle 
	   Each triangle has 3 edges
	   Each triangle can have children which are triangulated from this.
	*/
	class CTriangle{
	
	public:
		CTriangle() {}
		CTriangle(const CEdge* p1, const CEdge* p2, const CEdge* p3); 
	
		~CTriangle() {;}
	
		void setEdges(const CEdge* e1, const CEdge* e2, const CEdge* e3);
		void getEdges(CEdge** e1, CEdge** e2, CEdge** e3) const;
		void  getPoints(poi2D& p1, poi2D& p2, poi2D& p3) const;

		int getChildNum() const {return this->m_child.size();}
		CTriangle* getIthChild(int i)  {return (this->m_child.at(i));}// 0,1,2
		bool hasChild() const {return m_child.size()!=0;}
		bool isLeaf() const  {return m_child.size()==0;}
		void addChild(CTriangle* p);

		/* get the edge object of the given 2 points */
		CEdge* getEdge(const poi2D p1, const poi2D p2) const;
		
		/* get another vertex of the triangle */	
		poi2D getAnotherVertex (poi2D p1, poi2D p2) const ;
		
		/* remove all the edge-- triangle pointer for all the edges of this triangle */
		void removeEdgeTriPtr();
		
		/* to determine if the given point is in circumcircle of the current triangle */
		bool isPoiInCircleTriangle(poi2D pl);
		
		/* to determine if the poi is in or on the triangle 
		If point is on one edge of the triangle, return that edge object pointer in e.
		res:
		0: on the edges of triangle
		1: inside the triangle
		-1: outside the triangle
		*/
		bool isPoiInOnTri( const poi2D poi, int& res, CEdge** e);
		
		/*
		If point is on one edge of the triangle, return that edge object pointer in e.
		*/
		bool isPoiOnEdges(const poi2D pr, CEdge** pe);

		void clearTree() {m_child.clear();}
		
		friend ostream &operator << (ostream &ostr, const CTriangle &rhs)
		{
	
			CEdge* pe[3];
			rhs.getEdges(&pe[0], &pe[1], &pe[2]);
			ostr<<"!!!Triangle"<<endl;
			ostr<<*pe[0]<<*pe[1]<<*pe[2]<<endl;
			return ostr;
		}
	private:
		
		/* 
		primitive test on whether q(xq,yq) falls in the angle from 
		op1((xo,yo)->(x1,y1)) to op2((xo,yo)->(x2,y2))
		false	outside
		true inside
		*/
		bool pointInAngle(poi2D pq, poi2D po, poi2D p1, poi2D p2);
		bool inBetween(rat xq,rat x1,rat x2);
		int pointToLine(const poi2D pr,const poi2D po, const poi2D p1);
		
		/* calculate det product of 2 points */
		rat dot(const poi2D& p1,const poi2D& p2)
		{
			return p1.getX()*p2.getY()- p1.getY()*p2.getX();
		}

		/* calculate distances between 2 points  */
		rat distance(const poi2D& a, const poi2D& b)
		{
			return (a.getX()-b.getX())*(a.getX()-b.getX())+ 
					(a.getY()- b.getY())*(a.getY()- b.getY());
		}


	private:
	
		CEdge* m_edge[3]; // index for edge
		vector<CTriangle*> m_child;
	};
	
	
	class CDelaunay{
	
	public:
		CDelaunay() {}
	
		~CDelaunay() {;}
	
		
		void triangulation(vector<poi2D>& vec, line2D& reg);
		void voronoi(vector<poi2D>& vec, line2D& reg);
	
	private:
		void initialize(vector<poi2D>& vec);
		void finalize();
		void print();
		void insertPoint(const poi2D pr);
		
		/* set the biggest boundary triangle which contains all the points */
		void setBoundaryTri();
		
		/* the special point index -1, -2, -3 for the biggest boundary triangle */
		int getSpecialPointIndex(const poi2D& poi);

		/* trace the Triangle tree to find a leaf triangle that contains the point */
		CTriangle* findTriContainsPoi(const poi2D pr, int& res, CEdge** pe);

		/* using the edge to make triangle */
		CTriangle* makeTriangle(CEdge* e1, CEdge* e2, CEdge* e3);
		
		/*after add a point, legalize the edge e of the traingle that contains the point with adjacent triangle on the edge e. */
		void legalizeEdge(const poi2D pr, CEdge* e, CTriangle* p_tri);
	
		poi2D getBisectPoint(const poi2D& p00, const poi2D& p11, const poi2D& p22);
		bool  boundaryMbb(const poi2D poi1,const poi2D poi2,poi2D& poi3);

		void buildRegion(line2D& reg);
		void finalizeTree(CTriangle* p);

		/* for print and debug*/	
		void print(CTriangle* p);
		void getAllLeafTri(CTriangle* p, vector<CTriangle*> &vec);


	private:
		/* all the input points from other module, original input */
		vector<poi2D> m_poi_set;
		
		/*triangle tree */	
		CTriangle m_tri_tree;

		/*the max value to make the biggest boundary triangle */
		int m_max_value;
	};

	extern  CDelaunay * CDelaunayGetInstance();

}



#endif

