/** This class is for handling the dynamic structure of spatial objects in plane sweep.
 *  \author Reasey Praing
 *  \version 0.1
 *  \date    2006-10-23
 */
 
#ifndef DVECTOR_H
#define DVECTOR_H

#include "num.h"
#include "constants.h"
//#include "poi2D.h"
//#include "hseg2D.h"
#include "mSlob.h"
#include "undefinedSpatialObjectException.h"
#include "unrepairableObjectException.h"
#include "user_types.h"

namespace stu
{
	using namespace stu;
	
	template <class T>
	class dvector{
			
		public:
			
			class Iterator{ //iterates over each T in list
				public:
				long long int current;
				dvector* parent;
				Iterator(){}
				Iterator(dvector* par):current(-1), parent(par){}
				Iterator(dvector* par, long cur):current(cur), parent(par){}
				
				T* get(){
					if (current<0) cerr << "Need to move to next element to retrieve from iterator. " << endl;
					else if(current>=static_cast<long long int>(parent->elements.count())) cerr<< "Cannot get object beyond the last element."<<endl;
					//cout<<"current="<<current<<endl;
					Locator p(parent->elements.locate(current));
					//cout<<"current located\n";
					uint len=p.length(); //note that length only "really" works if poi is base object...
					unsigned char *buffer=new unsigned char[len];
					uint nread=0; //number of bytes read
					imSlobstream reader=p.getStream();
					reader.read(buffer,len,nread);
					return (new T(buffer,len)); //constructor "deserializes" poi2D object currently in buffer	
				}
				bool next(){
					if(!end())
						current++;
					return (!end());
				}
				bool prev(){
					if(current>0)
					{
						current--;
						return true;
					}
					else
						return false;
				}
				bool begin(){
					return (current==0 && parent->elements.count() > 0);
				}
				bool end(){
					return ((current>=static_cast<long long int>(parent->elements.count())) || (parent->elements.count()==0));
				}
				bool operator== (const Iterator &rhs){
					return (current == rhs.current && parent == rhs.parent);
				}
				Iterator operator= (const Iterator &rhs){
					parent = rhs.parent;
					current = rhs.current;
					//return this->getIterator();
					return parent->getIterator();
				}
				void toEnd()
				{
					current = parent->elements.count();
				}
			};
			
			dvector():elements(false), global(true)
			{
				//cout << "GLOBAL before: "; global.debug();
				global=global.insert(0,OBJECT_LEVEL);
				//cout << "GLOBAL after: "; global.debug();
				elements=global.insert(0, NONE_LEVEL);
				//cout << "Elements: "; elements.debug();
			}
			dvector(const Locator& m):elements(false), global(m)
			{
				if (global.getElements()==0) //then its an empty OBJECT NODE, insert primary struct
				{
					elements=global.insert(0);
				}
				else
				{
					elements=global.locate(0); //get primary struct
				}
			}
				
			~dvector(){}

			T operator[]( uint index )
			{
				Iterator bIter( this, index );
				return *bIter.get();
			}			
			
			void insert(Iterator itr, T &p){
				//if there are no elements yet
				uint sp=p.serialSize();
				unsigned char *data=new unsigned char[sp];		
				uint len=p.serialize(data);
				//cout << "Will insert in: "; elements.debug();
				if (elements.count()==0)
				{
					elements.append(data,len);//add p to the end.
					return;
				}
				if(itr.current > -1)
				{
					elements.insert(data,len,(uint)itr.current);//add p to the end
				}
				//cout << "Inserted: "; elements.debug();
			}

			void insert(uint idx, T &p){
				//if there are no elements yet
				uint sp=p.serialSize();
				unsigned char *data=new unsigned char[sp];		
				uint len=p.serialize(data);
				//cout << "Will insert in: "; elements.debug();
				if (elements.count()==0)
				{
					elements.append(data,len);//add p to the end.
					return;
				}else elements.insert(data,len,idx);//add p to the end
				//cout << "Inserted: "; elements.debug();
			}

			void push_back(T &p){
				//cout << "Pushing Back: "; elements.debug();
				//if there are no elements yet
				uint sp=p.serialSize();
				unsigned char *data=new unsigned char[sp];
				uint len=p.serialize(data);
				elements.append(data,len);//add p to the end.
				//cout << "Pushed Back: "; elements.debug();
			}
			
			void add_front(T &p){
				//cout << "Pushing Back: "; elements.debug();
				//if there are no elements yet
				uint sp=p.serialSize();
				unsigned char *data=new unsigned char[sp];
				uint len=p.serialize(data);
				elements.insert(data,len,0);//add p to the beginning.
				//cout << "Pushed Back: "; elements.debug();
			}

			void erase(Iterator itr)	{
				//cout << "Erasing: "; elements.debug();
				if(itr.current > -1 && !itr.end())
				{
					elements.remove((uint)itr.current);
				}
				//cout << "Erased: "; elements.debug();
			}

			void erase(uint idx)	{
				//cout << "Erasing: "; elements.debug();
				elements.remove(idx);
				//cout << "Erased: "; elements.debug();
			}

			const uint size() const //returns the size/number of elements
			{
				return elements.count();
			}
			bool isEmpty() const //returns getSize()==0
			{
				return (elements.count()==0);
			}

			Iterator getIterator(long start=-1) const
			{
				typename dvector<T>::Iterator p(const_cast<dvector<T>*>(this),start);
				return p;
			}
			friend std::ostream& operator << (std::ostream &ostr, const dvector<T> &rhs) throw( undefinedSpatialObjectException )
			{
				cout << "There are " << rhs.elements.count() << " objects in this structure" << endl;
				Iterator p1=rhs.getIterator();
				T pp1;
	
				ostr << "[";
				while (p1.next()){
					pp1=*(p1.get());
					ostr  << pp1 << ",";
				}
				ostr << "]";
				return ostr;
			}
		//	friend std::ostream &operator << (std::ostream &ostr, typename dvector<T> &rhs);
				
		
		private:
			Locator elements; //represents primary sequence
			Locator global; //represents whole object
	};
};
#endif

