#include "face2D.h"

#define min(a,b)=(a<b?a:b)
#define max(a,b)=(a>b?a:b)

namespace stu{
	
		using namespace stu;
		
		uint face2D::FLAGS=1;
		
		////////////face iterator methods
		face2D::PolyIterator::PolyIterator():current(-1), parent(NULL) {}
		face2D::PolyIterator::PolyIterator(face2D* par):current(-1), parent(par) {}
	
		bool face2D::PolyIterator::next(){ 
			if (current<0) current=0; 
			else current++;			
			return (!end());
		}
	
		poly2D* face2D::PolyIterator::get(){
		
			poly2D *temp;
		
			if (current<0) throw runtime_error("Need to move to next element to retrieve from iterator. ");
		
			Locator poly(parent->polys.locate(current));
/*			cout << "Loading poly: "; poly.debug();*/
			temp=new poly2D(*parent,poly);
			return temp;
		}
	
		bool face2D::PolyIterator::end(){
			//take into account that first element should always be valid flag
			return (current>static_cast<long>(parent->polys.count())-1 || parent->polys.count()<1);
		}

		
		face2D::Iterator::Iterator():current(-1), parent(NULL) {}
		face2D::Iterator::Iterator(face2D* par):current(-1), parent(par) {}
		face2D::Iterator::Iterator(face2D* par, long start):current(start), parent(par) {}
	
		bool face2D::Iterator::next(){ 
			if (parent==NULL) throw runtime_error("Undefined face2D iterator");
			if (current<0) current=0; //no flags to skip
			else current++;			
			return (!end());			
		}
		bool face2D::Iterator::prev(){		//added  --Reasey
			if(current>0)
			{
				current--;
				return true;
			}
			else
				return false;
		}	
		hseg2D* face2D::Iterator::get(){
		
		
			if (parent==NULL) throw runtime_error("Undefined face2D iterator");
			if (current<0) throw runtime_error("Need to move to next element to retrieve from iterator. ");
		
			Locator hseg(parent->hsegs.locate(current));
			uint len=hseg.length(); //note that length only "really" works if hseg is base object...
			/*			cout << "Will load hseg with len: " << len << " Loc: "; hseg.debug();*/
			unsigned char *buffer=new unsigned char[len];
			uint nread=0; //number of bytes read
			imSlobstream reader=hseg.getStream();
			reader.read(buffer,len,nread);
			hseg2D *temp=new hseg2D(buffer,len); //constructor "deserializes" hseg2D object currently in buffer	
			/*			cout << "The hseg: " << *temp << endl;*/
			
			return temp;
		}
	
		bool face2D::Iterator::end(){
			if (parent==NULL) throw runtime_error("Undefined face2D iterator");
			//take into account that first element should always be valid flag
			return (current>static_cast<long long int>(parent->hsegs.count())-1 || static_cast<long long int>( parent->hsegs.count())<1);
		}
		
		face2D::Iterator face2D::Iterator::operator+( int num )
		{
			Iterator temp = *this;
			if (temp.current<0) temp.current=num;
			else temp.current = temp.current+num;
			return temp;
		}
		face2D::Iterator face2D::Iterator::operator-( int num )
		{
			Iterator temp = *this;
			temp.current = temp.current-num;
			if (temp.current<0) temp.current=0;
			return *this;	
		}
		face2D::Iterator face2D::Iterator::operator++( int dummy )
		{
			Iterator temp = *this;
			if (current<0) current=0;
			else current++;
			return temp;
		}
		face2D::Iterator face2D::Iterator::operator--( int dummy )
		{
			Iterator temp = *this;
			current--;
			if (current<0) current=0;
			return temp;
		}
		face2D::Iterator& face2D::Iterator::operator+=( int num )
		{
			if (current<0) current=num;
			else current = current+num;
			return *this;
		}
		face2D::Iterator& face2D::Iterator::operator-=( int num )
		{
			current = current - num;
			if( current < 0 ) current = 0;
			return *this;
		}
	
		
		hseg2D face2D::operator[]( uint index )
		{
			face2D::Iterator pIter( this );
			pIter.next( );
			pIter = pIter+index;
			return *pIter.get();
		}
		
		uint face2D::size() const
		{
			return getSegmentCount()*2;
		}
		
		///////////////////////////////////
		
		void face2D::loadValid(){
			changed=false;
			Locator flg(elements.locate(0));
			uint len=flg.length(); //the flag length should be 1!
			unsigned char *buffer=new unsigned char[len];
			uint nread=0; //number of bytes read
			imSlobstream reader=flg.getStream();
			reader.read(buffer,len,nread);
			valid=(*buffer=='V');
// 			cout << "Loaded Valid for Face: " << *buffer << endl;
			delete buffer;
		}

		void face2D::saveValid(){
			if (changed){
				//throw runtime_error("calling save valid");
				if (elements.count()>0) elements.remove(0);
				unsigned char *buffer=new unsigned char;
				if (valid) *buffer='V'; 
				else *buffer='I';
				elements.insert(buffer,1,0);
// 				cout << "Saved Valid for Face: " << *buffer << endl;
				delete buffer;
			}
			changed=false;
		}
		
		
		face2D::face2D():global(false), elements(false), polys(false), hsegs(false),  parent(NULL), valid(false), changed(true), inBatch(false){
			parent=new region2D(); //create parent
	
			global=parent->global;
			elements=parent->addFace().elements;
			hsegs=elements.locate(FLAGS);
			polys=elements.locate(FLAGS+1);
			validate();
		}
		
		
		face2D::face2D(const region2D& r, const Locator& f):global(r.global), elements(f), polys(false), hsegs(false), parent(const_cast<region2D*>(&r)), valid(false), changed(true), inBatch(false) {
			if (elements.count()>0){
				loadValid();
				hsegs=elements.locate(FLAGS);
				polys=elements.locate(FLAGS+1);
				//cout << "Built facy: "; hsegs.debug();
			}
			else{
				saveValid(); //first save
				hsegs=elements.insert(FLAGS,NONE_LEVEL);
				polys=elements.insert(FLAGS+1,NONE_LEVEL);
				validate();
			}
/*			cout << "Created face: "; elements.debug();*/
		}
		
		face2D::face2D(const region2D& r):global(r.global), elements(false), polys(false), hsegs(false), parent(const_cast<region2D*>(&r)), valid(false), changed(true), inBatch(false) {
			elements=parent->addFace().elements; //create me as an empty face of my parent
			hsegs=elements.locate(FLAGS);
			polys=elements.locate(FLAGS+1);	
			validate();
		}		
		
		face2D::~face2D(){}
		
		//constructor ops

		poly2D face2D::addOuterCycle(const poly2D &p) throw (runtime_error, invalidObjectException)
		{	//b should be from a different mSLob, so copy it	
			return addPoly(p,0,true);
		}
		
		poly2D face2D::addHole(const poly2D &p) throw (invalidObjectException)
		{	//b should be from a different mSLob, so copy it
			
			return addPoly(p, polys.count(), false);
		}
		
		//adds all polys, leaves checking to finalize
		void face2D::addHoles(const poly2D bs[], uint length) throw (invalidObjectException)
		{
			inBatch=true;
			for (uint i=0; i<length; i++)
				addHole(bs[i]);
			inBatch=false;
			saveValid();
		}
		
		poly2D face2D::addOuterCycle()
		{	//add an empty outercycle
			
			//no need to invalidate as new empty block does nothing wrong
			if (polys.count()>0) //then check to make sure first poly is not an outer
			{
				poly2D temp(*this, polys.locate(0));
				if (temp.isOuterCycle())
					throw runtime_error("Adding outer cycle to face that has one already. Remove first.");
			}
			
			Locator newout(polys.insert(0,NONE_LEVEL)); //outer cycle should always be in position 0
						
			poly2D toReturn(*this,newout,true);
			//toReturn->grandpa=parent;
			
/*			cout << "Added empty Outer! "; newout.debug();*/
			
			return toReturn;
		}
		
		poly2D face2D::addHole()
		{	//add an empty hole
			
			//no need to invalidate as new empty block does nothing wrong
			Locator newhole(polys.insert(polys.count(),NONE_LEVEL));
						
			poly2D toReturn(*this, newhole,false);
/*			cout << "Added empty Hole! "; newhole.debug();*/
			//toReturn->grandpa=parent; not necessary, happens in construction in previous line
			
			return toReturn;
		}
		
		
		
		poly2D face2D::addPoly(const poly2D& p, uint idx, bool isOuterCycle)  throw (runtime_error, invalidObjectException){
			
			if (!p.isValid()) //should p be valid?
				throw runtime_error("Adding broken polygon to face...u crazy?");
			
			Locator newpolyl(polys.insert(idx,NONE_LEVEL));
			
			
			//first add all the polygon flags
/*			for (uint i=0; static_cast<long long int>(i)<poly2D::FLAGS; i++){
				Locator flg(p.elements.locate(i));
				uint len=flg.length(); //the flag length should be 1!
				unsigned char *buffer=new unsigned char[len];
				uint nread=0; //number of bytes read
				imSlobstream reader=flg.getStream();
				reader.read(buffer,len,nread);
				newpolyl.insert(buffer,len,i);	
				delete buffer;
			}*/
			
/*			cout << "Poly Added: "; newpolyl.debug();*/
			
			poly2D newpoly(*this, newpolyl, isOuterCycle);
			
			poly2D::Iterator p1=p.getIterator();
			hseg2D *t;
			
			while (p1.next())
			{
				t=p1.get();
				newpoly.addhseg(*t); //&& could save time by appending cause they are read in order anyway
			}
			
			//invalidate(); already invalidated by addhseg above
			
/*			if (p.isOuterCycle()!=newpoly.isOuterCycle())
			{	
				newpoly.invalidate();*/
				newpoly.validate();
/*			}*/
/*			else
				valid=p.isValid();*/
			
			invalidate();
						
/*			cout << "Poly Added After: "; newpolyl.debug();*/
			return newpoly;
		}
		
		bool face2D::removePoly(uint idx)
		{
// 			Locator ply(polys.locate(idx));

/*			cout << "will remove poly: "; ply.debug();*/
			poly2D up(*this, polys.locate(idx));
			
			up.removeMe();
/*			cout << "removed" << endl;*/
			if (!polys.remove(idx))
				return false;
// 			global.getMySlob()->sinx->debug();

			return true;//should be empty by now
		}

		bool face2D::removeOuterCycle(){
			if (polys.count()>0){
				poly2D p(*this, polys.locate(0));
				if (p.isOuterCycle()){
					if (!removePoly(0)){
						invalidate();
						return false;
					}else{
						invalidate();
						return true;
					}
				}
				else
					return false;
			}
			else return false;
		}
		
		bool face2D::removeHole(uint idx){
			return removePoly(idx);
		}
		
		void face2D::invalidate(){
			if (valid){
				valid=false;
				changed=true;
				if (!inBatch) saveValid();
				parent->invalidate();
			}
		}
		
		bool face2D::validate()
		{ 
			if (!valid){
				
/*				cerr << "Validating face!" << endl;*/
				//first check that all children are valid
				face2D::PolyIterator p1=getPolyIterator();
				poly2D* pp1;
				while (p1.next())
				{	
					pp1=p1.get();
					if (!pp1->isValid()) //then attempt to validate
						if (!pp1->validate())
						{
							cerr << "Could not validate polygon: " << *pp1 << endl;
							return false;
						}
				}
				
				if (hsegs.count()>0){
					
					// check intersection of segments and agreement on insideAbove flags
					whichObj object;
					whatStatus status;
					dvector<hseg2D> dyn1;
					dvector<hseg2D> dyn2;
					face2D tempPl;
					EventSchedule<face2D, face2D, hseg2D, hseg2D> eventSched( *this, tempPl, dyn1, dyn2 );
					SweepLine<face2D, face2D, hseg2D, hseg2D> S( true );
					statusHseg2D sh;
					eventSched.select_first(object, status);
					hseg2D segCurr, segAbove, segBelow;
					bool aboveExists, belowExists;
					
					//cout << "======================Start!!!======================" << endl;
					
					while(status != end_of_both )
					{
						//cout << "Looking" << endl;
						if(object == first || object == both )
						{
							sh = statusHseg2D(*(eventSched.i1), 'F');
							if(eventSched.i1->isLeft())
							{
								bool inter=S.add_left_Return_Segs( sh, eventSched, segCurr, segAbove, aboveExists, segBelow, belowExists );
								if( inter ) //then there is an intersection
								{
/*									cerr<<"Not a valid face: intersecting segments"<<endl;*/
									return false;
								}else //no intersection, check ia of segs above and below
								{
									hseg2D next;
									aboveExists=false;
									belowExists=false;
									if (eventSched.look_aheadF(&segCurr,next)){
										//cout << "PRE-Looked ahead: " << next << " Current: " << segCurr << endl;
										if (next.getDominating()==segCurr.getDominating()){
											if (segCurr.isLeft())
											{
												segAbove=next;
												aboveExists=true;
											}else
											{
												segBelow=next;
												belowExists=true;
											}

											//cout << "Looked ahead: " << next << " Current: " << segCurr << endl;
										}
									}
									if (aboveExists && segCurr.isInsideAbove()==segAbove.isInsideAbove()) //then error
									{
										cerr << "Face InsideAbove does not match with above segment. Curr: " << segCurr << " Above: " << segAbove << endl;
										return false;
									}
										
									if (belowExists && segCurr.isInsideAbove()==segBelow.isInsideAbove()) //then error
									{
										cerr << "Face InsideAbove does not match with below segment. Curr: " << segCurr << " Below: " << segBelow << endl;
										return false;
									}
								}
							}
							else
							{
								S.del_right(sh, eventSched);
							}
						}
						eventSched.select_next(object, status);
					}
				}
				
				valid=true;
				changed=true;
				saveValid();
			}
			return valid;
		}
		
		uint face2D::getSegmentCount() const //returns the number of segments for the whole face
		{ return (hsegs.count()/2);}

		uint face2D::getHoleCount() const//number of polys
		{
			if (polys.count()>1)
				return polys.count()-1; //first element is outer cycle, so dont count
			else
				return 0; //watch for wrap arounds of unsigned values
		}

		rat face2D::getLength() const //returns the perimeter. sum of lenghts of segments
		{ 
			//because every segment is stored twice, then sum all hseg lengths and divide by 2.
			rat tlength(0);
			poly2D *t;
			//rat two(2);

			PolyIterator li=getPolyIterator();
			
			while (li.next()){
				t=li.get();
				tlength+=t->getLength();
			}

			return tlength;
		}

		rat face2D::getArea() const //returns the perimeter. sum of lenghts of segments
		{ 
			throw runtime_error("getArea is not implemented!");
		}
		
		bool face2D::isEmpty() const //returns getSize()==0
		{ return (hsegs.count()==0);}

		bool face2D::isValid() const
		{	return valid;	} 
		
// 		int face2D::getType()
// 		{
// 			return LINE2D;
// 		}

		mbb2D face2D::getmbb() const throw (invalidObjectException)
		{
			if (!valid)
				throw invalidObjectException("Invalid Face: Cannot compute MBB.");
	
// 			rat minx(0);
// 			rat miny(0);
// 			rat maxx(0);
// 			rat maxy(0);
//			bool first=true;
	
			PolyIterator li=getPolyIterator();
			poly2D* outer=li.get(); 
			return outer->getmbb(); //mbb for a face is the mbb of its outer cycle
		}
		
		
		face2D::Iterator face2D::getIterator() const //sorted in half-segment order.		
		{
			face2D::Iterator p(const_cast<face2D*>(this));
			return p;;
		}
		
		face2D::PolyIterator face2D::getPolyIterator() const //used to iterate over each poly
		{
			face2D::PolyIterator p(const_cast<face2D*>(this));
			return p;;
		}
		
		Locator face2D::addhseg(const hseg2D &h) throw (invalidObjectException)
		{
			if (!h.isValid())
				throw invalidObjectException();
		
			Locator added(false);
//			uint loc=FLAGS; //loc starts here cause 0 is valid flag and 1 is outer flag
//			hseg2D *t;

			if (parent!=NULL){
				added=(parent->addhseg(h));
			}else{
				throw runtime_error("Faces must belong to regions.");
			}
			//now add reference to secondary structure of line (this poly)
			
// 			face2D::Iterator bi=getIterator();
// 			loc=0;
// 			
// 			while (bi.next()){
// 				t=bi.get();
// 				if (*t==h)
// 					return added; //do not add, duplicate
// 				if (*t>h)
// 					break; //then add here		
// 				loc++;
// 			}
// 		
// 			if (bi.end()) //reached end so insert last
// 				hsegs.insert(added,hsegs.count());
// 			else
// 				hsegs.insert(added,loc);//add p to the end.
		
			vectorBinarySearchWrapper< face2D, hseg2D > finder ( this ); 
			
			uint found=0;
			
			if (finder.find(h,found)){
				long loc=static_cast<long>(found);
				return added; //do not add, duplicate
			}else
			{
				long loc=static_cast<long>(found);
				hsegs.insert(added,loc);//add h
			}	
			
			
// 			if (finder.find(h,found)){
// 				return false; //do not add, duplicate
// 			}else
// 			{
// 				long loc=static_cast<long>(found)+FLAGS;
// 				elements.insert(added,loc);//add h
// 			}	
			
			
/*			cout << "Added hseg to face, hsegs: "; hsegs.debug();*/
			//cout << "This face now " << *this << endl; 
			invalidate();
			return added;
		}
		

		std::ostream& operator << (std::ostream &ostr, const face2D &rhs) throw( invalidObjectException )
		{
			//const_cast<face2D*>(&rhs)->validate();
			if( !rhs.isValid( ) ) {
				ostr<< "--(Invalid Face)--" ;
			} 
			//cout << "There are " << rhs.elements.count() << " pois in this point" << endl;
			face2D::Iterator p1=rhs.getIterator();
			hseg2D* pp1;
			
			ostr << "[";
			while (p1.next()){
				pp1=p1.get();
				//ostr << "=poly=";
				ostr << *pp1;
				//ostr << endl;
			}
			ostr << "]";
			return ostr;
		}

}


