/** This class is for representing complex faces.
 * A face is constructed by way of a set of polys 
 * @see poly2D.
 *  \author Alejandro Pauly, Mark McKenney
 *  \version 0.1
 *  \date    2005-10-05
 *  \version 0.2
 *  \date    2005-10-14 
 *  \version 0.4 Removed Impl stuff, fixing for mSlob
 *  \date    2006-10-16 
 */

#ifndef FACE2D_H
#define FACE2D_H

#include "constants.h"
#include "num.h"
#include "hseg2D.h"
#include "mSlob.h"
#include "poly2D.h"
#include "region2D.h"
#include "invalidObjectException.h"
//#include "TopPred2D.h"
// #include "sweepLineSingleStatus.h"
// #include "MEventSchedule.h"

namespace stu{
	using namespace stu;

	class poly2D;
	class region2D;
	
	class face2D{
	
		public:
			friend class poly2D;
			friend class region2D;
						
			class PolyIterator{ //iterates over each polygon
				long current;
				face2D* parent;
				
				public:
					PolyIterator();
					PolyIterator(face2D* par);
					poly2D* get();
					bool next();
					bool end(); 
			};
			
			class Iterator{ //iterates over each hseg2D in list
				long current;
				face2D* parent;
				
				public:
					Iterator();
					Iterator(face2D* par);
					Iterator(face2D* par, long start);
					hseg2D* get();
					bool next();
					bool prev();
					bool end(); 
					Iterator operator+( int num );
					Iterator operator-( int num );
					Iterator operator++( int dummy );
					Iterator operator--( int dummy );
					Iterator& operator+=( int num );
					Iterator& operator-=( int num );
			};
			
			face2D();
			face2D(const region2D& r, const Locator& f); //load f as a face of r
			face2D(const region2D& r); //create this as an empty face of r

			
			~face2D();
				
			hseg2D operator[]( uint index );
			uint size() const; //the number of half segments
			
			//constructor ops
			poly2D addOuterCycle(const poly2D &p)  throw (runtime_error, invalidObjectException);
			poly2D addHole(const poly2D &p)  throw (invalidObjectException);
			poly2D addOuterCycle();
			poly2D addHole();
			void addHoles(const poly2D ps[], const uint length)  throw (invalidObjectException);
	
			bool removeOuterCycle();
			bool removeHole(uint idx);
	
			bool validate();
			bool isValid( ) const;
			
			uint getSegmentCount() const; //returns the number of segments for the whole face
	
			uint getHoleCount() const; //returns the number of holes for the whole face
	
			bool isEmpty() const; //returns getSize()==0
	
			rat getLength() const; //length of boundary, sum of segments		
	
			rat getArea() const; //all area....sum of areas of faces.
			
			mbb2D getmbb() const throw (invalidObjectException);
			
			Iterator getIterator() const;
			PolyIterator getPolyIterator() const;
			
			//Output operator
			friend std::ostream &operator << (std::ostream &ostr, const face2D &rhs) throw( invalidObjectException );


		private:
			Locator global; //locator to a global mSlob representing a region2D
			Locator elements; //top of hierarchy for a face, consists of flags and two subobjects, polys and hsegs
			Locator polys; //each subelement is a poly2D
			Locator hsegs; //each subelement is an hseg2D
			region2D* parent; //need it to send it to poly2Ds
			
			bool valid;
			bool changed; //to keep whether the object was changed and so the valid flag needs to be rewritten
			bool inBatch; ////to avoid writing valid flag more than necessary

			static uint FLAGS;
			
			void saveValid();
			void loadValid();
			void invalidate(); //used to set valid=false when changes occur
			
			//returns locator to newly added hseg
/*			Locator addhseg(const hseg2D &h) throw (validObjectException, invalidObjectException);*/
			poly2D addPoly(const poly2D& p, uint idx, bool isOuterCycle=true) throw (runtime_error, invalidObjectException);
			bool removePoly(uint idx);
			Locator addhseg(const hseg2D &h)throw (invalidObjectException);
			
// 			uint getBlockSegmentCount(uint bid);
			
/*			std::vector<hseg2D>& findConnectedHsegs(const hseg2D& current, std::vector<hseg2D> & nothere, ulong bid);
			uint connectedFromHere(const hseg2D& origin, std::vector<hseg2D> & nothere, ulong bid);
			poi2D& findFirstHalfSeg(ulong bid);
			void checkIntersections( hseg2D &H1, hseg2D &H2, std::vector<hseg2D> &vec,
			MEventSchedule<hseg2D, hseg2D> EE, sweepLineSingleStatus<hseg2D> &SLSS );
			std::vector<hseg2D>::iterator findTwin( hseg2D &sibling);*/
			
	};

}
#endif


