/** This class is for representing complex point objects.
 * A complex point represents a set of single points in the plane.
 	@see poi2D.
 *  \author Alejandro Pauly, Mark McKenney
 *  \version 0.1
 *  \date    2005-10-05
 *  \version 0.2
 *  \date    2005-10-14
 *  \version 0.3
 *  \date    2005-11-25 
 *  \version 0.4 Removed Impl stuff, fixing for mSlob
 *  \date    2006-09-28 
 * 
 */
 
#ifndef POINT2D_H
#define POINT2D_H

#include "num.h"
#include "constants.h"
//#include "line2D.h"
//#include "region2D.h"
#include "poi2D.h"
#include "hseg2D.h"
#include "mbb2D.h"
#include "mSlob.h"
#include "undefinedSpatialObjectException.h"
#include "vectorBinarySearchWrapper.h"
#include "user_types.h"

//#include "delaunay.h"
//#include "convexHull.h"
//#include "line2D.h"
//#include "region2D.h"

namespace stu
{
	using namespace stu;
	using namespace std;
	class region2D;
	class CDelaunay;
	class CConvexHull;
	class line2D;

	class point2D{
	

		public:
			
			class Iterator{ //iterates over each poi2D in list
				long current;
				point2D* parent;
				
				public:
				Iterator();
				Iterator(point2D* par);
				poi2D* get();
				bool next();
				bool prev();	//added  --Reasey
				bool begin();	//added  --Reasey
				bool end();
				Iterator operator+( int num );
				Iterator operator-( int num );
				Iterator operator++( int dummy );
				Iterator operator--( int dummy );
				Iterator& operator+=( int num );
				Iterator& operator-=( int num );			
			};
			
			point2D(bool create=true);
			point2D(const Locator& m);
			point2D(const Locator& m, const poi2D &p);
			point2D(const Locator& m, poi2D ps[], uint length);
			point2D(const Locator& m, const point2D& ps);
	
			~point2D();
	
			poi2D operator[]( uint index );
			uint size() const; //the number of points

			
			//constructor ops
			void addPoint(poi2D &p) throw (undefinedSpatialObjectException); //adds point checks it didn't exist already
			void addPoints(poi2D ps[], uint length); //adds all points, leaves checking to finalize
	
			void removePoint(const poi2D &p)  throw (runtime_error, undefinedSpatialObjectException); //removes a point, no checking needed
		
			const uint getPointCount() const; //returns the size/number of points
	
			bool isEmpty() const; //returns getSize()==0
			bool validate();
			bool isValid() const;
	
			Iterator getIterator() const;
			mbb2D getmbb() const;

			//homogeneous ops
			point2D intersection(const point2D &other) const;
			point2D sunion(const point2D &other) const;		
			point2D difference(const point2D &other) const;
	
			rat  distance(const point2D &other) const; //returns the minimum distance between this and other
	
			//heterogeneous ops
// 			point2D & intersection(const line2D& other) const;
// 			point2D & intersection(const region2D& other) const;
// 			point2D & difference(const line2D& other) const;
// 			point2D & difference(const region2D& other) const;
// 	
// 			rat &distance(const line2D &other) const;
// 			rat &distance(const region2D &other) const;
			
			//Output operator
			friend std::ostream &operator << (std::ostream &ostr, const point2D &rhs) throw( undefinedSpatialObjectException );


			/* compute convex Hull of all the points 
			   using Granham's algorithm complexity O(nlogn)
			*/
			region2D convexHull();
			/* compute delaunay triangulation of all the points
			   using incremental algorithm with an average complexity of O(nlogn)
			*/
			line2D delaunayTriangulation();
			
			/* compute voronoi diagram */	
			line2D voronoi();	
		private:
			/*
			get all points for convex hull and delaunay triangulation algorithm
			*/
			void getAllDominates(vector<poi2D>& m_poi_set);


			Locator elements; //represents primary sequence
			Locator global; //represents whole object
			bool valid;	
			static short int FLAGS;
			
	
	};
}
#endif

