#include "poly2D.h"

namespace stu{
	
		using namespace stu;

		short int poly2D::FLAGS=2;
		
		/////////////poly iterator methods
		
		poly2D::Iterator::Iterator():current(-1), parent(NULL) {}
		poly2D::Iterator::Iterator(poly2D* par):current(-1), parent(par) {}
		poly2D::Iterator::Iterator(poly2D* par, long start):current(start), parent(par) {}
	
		bool poly2D::Iterator::next(){ 
			if (parent==NULL) throw runtime_error("Undefined poly2D iterator");
			if (current<0) current=FLAGS; //skip valid and outer flags!
			else current++;
			return (!end());
		}
		bool poly2D::Iterator::prev(){		//added  --Reasey
			if(current>poly2D::FLAGS)
			{
				current--;
				return true;
			}
			else
				return false;
		}	
		hseg2D* poly2D::Iterator::get(){
		
		
			if (parent==NULL) throw runtime_error("Undefined poly2D iterator");
			if (current<0) throw runtime_error("Need to move to next element to retrieve from iterator. ");
		
			Locator hseg(parent->elements.locate(current));
			uint len=hseg.length(); //note that length only "really" works if hseg is base object...
/*			cout << "Will load hseg with len: " << len << " Loc: "; hseg.debug();*/
			unsigned char *buffer=new unsigned char[len];
			uint nread=0; //number of bytes read
			imSlobstream reader=hseg.getStream();
			reader.read(buffer,len,nread);
			hseg2D *temp=new hseg2D(buffer,len); //constructor "deserializes" hseg2D object currently in buffer	
/*			cout << "The hseg: " << *temp << endl;*/
			
			return temp;
		}
	
		bool poly2D::Iterator::end(){
			if (parent==NULL) throw runtime_error("Undefined poly2D iterator");
			//take into account that first element should always be valid flag
			return (current>static_cast<long long int>(parent->elements.count())-1 || static_cast<long long int>( parent->elements.count())<=FLAGS);
		}
		
		poly2D::Iterator poly2D::Iterator::operator+( int num )
		{
			Iterator temp = *this;
			if (temp.current<FLAGS) temp.current=FLAGS+num; //skip valid flag!
			else temp.current = temp.current+num;
			return temp;
		}
		poly2D::Iterator poly2D::Iterator::operator-( int num )
		{
			Iterator temp = *this;
			temp.current = temp.current-num;
			if (temp.current<FLAGS) temp.current=FLAGS; //skip valid flag!
			return temp;	
		}
		poly2D::Iterator poly2D::Iterator::operator++( int dummy )
		{
			Iterator temp = *this;
			if (current<FLAGS) current=FLAGS; //skip valid flag!
			else current++;
			return temp;
		}
		poly2D::Iterator poly2D::Iterator::operator--( int dummy )
		{
			Iterator temp = *this;
			current--;
			if (current<FLAGS) current=FLAGS; //skip valid flag!
			return temp;
		}
		poly2D::Iterator& poly2D::Iterator::operator+=( int num )
		{
			if (current<FLAGS) current=FLAGS+num; //skip valid flag!
			else current = current+num;
			return *this;
		}
		poly2D::Iterator& poly2D::Iterator::operator-=( int num )
		{
			current = current - num;
			if( current < FLAGS ) current = FLAGS;
			return *this;
		}
		hseg2D poly2D::operator[]( uint index )
		{
			poly2D::Iterator pIter( this );
			pIter.next( );
			pIter = pIter+index;
			return *pIter.get();
		}
		
		uint poly2D::size() const
		{
			return (elements.count()-FLAGS);
		}
		
		///////////////////////////flag handling functions
		
		void poly2D::loadOuter(){
			Locator flg(elements.locate(1));
			uint len=flg.length(); //the flag length should be 1!
			unsigned char *buffer=new unsigned char[len];
			uint nread=0; //number of bytes read
			imSlobstream reader=flg.getStream();
			reader.read(buffer,len,nread);
			isOuter=(*buffer=='O');
// 			cout << "Loaded Poly Outer: " << *buffer << endl;
			delete buffer;
		}
		
		void poly2D::saveOuter(){
			//throw runtime_error("calling save outer poly");
			if (elements.count()>1) elements.remove(1);
			unsigned char *buffer=new unsigned char;
			if (isOuter) *buffer='O'; 
			else *buffer='H';
			elements.insert(buffer,1,1);
			delete buffer;
		}
		
		void poly2D::loadValid(){
			changed=false;
			Locator flg(elements.locate(0));
			uint len=flg.length(); //the flag length should be 1!
			unsigned char *buffer=new unsigned char[len];
			uint nread=0; //number of bytes read
			imSlobstream reader=flg.getStream();
			reader.read(buffer,len,nread);
			valid=(*buffer=='V');
// 			cout << "Loaded Poly Valid: " << *buffer << endl;
			delete buffer;
		}

		void poly2D::saveValid(){
			if (changed){
				//throw runtime_error("calling validate poly");
				if (elements.count()>0) elements.remove(0);
				unsigned char *buffer=new unsigned char;
				if (valid) *buffer='V'; 
				else *buffer='I';
				elements.insert(buffer,1,0);
				delete buffer;
			}
			changed=false;
		}
		
		void poly2D::loadFlags(){
			loadOuter();
			loadValid();
		}

		void poly2D::saveFlags(){
				saveValid();
				saveOuter();
		}
		
		////////////////////////////end of flag handling functions
		
		
		poly2D::poly2D(bool isOuterCycle):global(false), myface(false), elements(false), parent(NULL), grandpa(NULL), isOuter(isOuterCycle), valid(false), changed(true), inBatch(false){
			grandpa=new region2D();
			parent = new face2D(*grandpa);
			//parent=grandpa->addFace(); //create parent
/*			poly2D *temp=&parent->addOuterCycle(); //create me*/
			
			global=grandpa->global;
			myface=parent->elements;
			elements=parent->addOuterCycle().elements;
		}

		//from the given locator
		poly2D::poly2D(const face2D& f, const Locator& p, bool isOuterCycle ):global(f.global), myface(f.elements), elements(p), parent(const_cast<face2D*>(&f)), grandpa(const_cast<region2D*>(f.parent)), isOuter(isOuterCycle), valid(false), changed(true), inBatch(false) {
			if (elements.count()>0){
				loadFlags();
// 				cout << "Creating here!! "; p.debug();
			}
			else{
// 				cout << "Here it is" << endl;
				saveFlags(); //first save
			}
			//cout << "Created poly: "; elements.debug();
		}
		
		poly2D::poly2D(const face2D& f):global(f.global), myface(f.elements), elements(false), parent(const_cast<face2D*>(&f)), grandpa(const_cast<region2D*>(f.parent)), isOuter(true), valid(false), changed(true), inBatch(false) {
			elements=parent->addOuterCycle().elements; //create me as an outer cycle of a face
			saveValid(); //first save
		}

		poly2D::~poly2D(){}
		
		//constructor ops
		bool poly2D::addSegment(const seg2D &s)
		{
			hseg2D p(s.getP(),s.getQ(),true);
			if (!p.validate()) return false;
			if (addhseg(p)) invalidate();
			
			hseg2D q(s.getQ(),s.getP(),false);
			if (!q.validate()) return false;
			if (addhseg(q)) invalidate();
			
			return true;
		}
		
		//adds all segs, leaves checking to finalize
		bool poly2D::addSegments(const seg2D ss[], uint length) // throw (validObjectException);
		{
			for (uint i=0; i<length; i++)
				if (!addSegment(ss[i]))
					return false; //error adding at least one, skip!
				
			return true; //all added succesfully
		}
		
		void poly2D::removeSegment(const seg2D &s)  throw (undefinedSpatialObjectException)
		{			
			if (s.isUndefined())
				throw undefinedSpatialObjectException();

			hseg2D p(s.getP(),s.getQ(),true);

			long loc=-1;
			hseg2D q=findBrother(p,loc);
			if (loc>-1){
				grandpa->removeSegment(s);
				invalidate();
			}//else the segment does not exist in this polygon, so don't remove from here
		}
		
		void poly2D::removeMe() { //removes all of this poly's hsegs from the primary structure...
			unsigned char *buffer=NULL;
			while (static_cast<long long int>(elements.count())>FLAGS){
				Locator hseg(elements.locate(FLAGS));
				uint len=hseg.length(); //note that length only "really" works if hseg is base object...
				if (buffer!=NULL) delete buffer;
				buffer=new unsigned char[len];
				uint nread=0; //number of bytes read
				imSlobstream reader=hseg.getStream();
				reader.read(buffer,len,nread);
				hseg2D *temp=new hseg2D(buffer,len); //constructor "deserializes" hseg2D object currently in buffer	
			
				grandpa->removeHalfSegment(*temp);
			}
			
			//then its just the flags...remove...
			while (elements.count()>0){
				elements.remove(0);
			}
		}
		
		hseg2D poly2D::findBrother(const hseg2D& cur, long& loc){
			vectorBinarySearchWrapper< poly2D, hseg2D > polySearchWrapper( this ); 
			
			uint found=0;
			hseg2D bro(cur.getSubmissive(), cur.getDominating(), !cur.isInsideAbove());
			
			if (polySearchWrapper.find(bro,found)){
				Locator hseg(elements.locate(found+FLAGS));
				uint len=hseg.length(); //note that length only "really" works if hseg is base object...
				unsigned char *buffer=new unsigned char[len];
				uint nread=0; //number of bytes read
				imSlobstream reader=hseg.getStream();
				reader.read(buffer,len,nread);
				hseg2D *orig=new hseg2D(buffer,len); //constructor "deserializes" hseg2D object currently in buffer	
				hseg2D toret=*orig; //copy
				delete buffer;
				loc=static_cast<long>(found+FLAGS);
				return toret;
			}
			else{
				loc=-1;
				return hseg2D(); //not found.
			}
		}
		
		long poly2D::findNextsInChain(const hseg2D& cur, long curloc, dvector<hseg2D>& result, stuVector<long>& locs){
		/*returns the number of hsegs next in chain from cur (-1 if none). result has the next in chains,
		and locs has the indices where those were found in the sequence*/
			
			long offset=0;
			long locbro=-1;
			long count=0;

			hseg2D bro=findBrother(cur,locbro);
			if (locbro>-1){ //found the brother, now find actual next in chain
				poly2D::Iterator fwd=getIterator(locbro);
				poly2D::Iterator bck=getIterator(locbro);
				
				hseg2D *b=NULL;
				hseg2D *f=NULL;

				offset=0;
				while (true){
					b=NULL;
					offset++;
					if (bck.prev()){
						b=bck.get();
						if (b->getDominating()==cur.getSubmissive()){
							locs.add_front(static_cast<long>(locbro-offset));
							result.add_front(*b);
							count++;
						}else //no more found
							break;	
					}
					if (b==NULL)
						break;
				}

				offset=0;
				while (true){
					f=NULL;
					if (fwd.next()){
						f=fwd.get();
						if (f->getDominating()==cur.getSubmissive()){
							locs.push_back(static_cast<long>(locbro+offset));
							result.push_back(*f);
							count++;
						}else //no more found
							break;	
					}
					if (f==NULL)
						break;
				}
			
			}
			
			return count;
		}
		
		
		void poly2D::invalidate(){
			if (valid){
				valid=false;
				changed=true;
				if (!inBatch) saveValid();
				parent->invalidate();
			}
		}
		
		bool poly2D::validate()
		{
// 			valid=true;
// 			changed=true;
// 			saveValid();
//			return valid;
			
			
			if (!valid)
			{
				
/*				cerr << "Validating polygon!" << endl;*/
				
				// first use the sweep line to check if any segments intersect
				whichObj object;
				whatStatus status;
				dvector<hseg2D> dyn1;
				dvector<hseg2D> dyn2;
				poly2D tempPl;
				EventSchedule<poly2D, poly2D, hseg2D, hseg2D> eventSched( *this, tempPl, dyn1, dyn2 );
				SweepLine<poly2D, poly2D, hseg2D, hseg2D> S( true );
				statusHseg2D sh;
				eventSched.select_first(object, status);
				while(status != end_of_both )
				{
					if(object == first || object == both )
					{
						sh = statusHseg2D(*(eventSched.i1), 'F');
						if(eventSched.i1->isLeft())
						{
							if( S.add_left(sh, eventSched) )
							{
/*								valid = false;
								changed = true;
								saveValid( );*/
								cerr<<"Not a valid polygon: intersecting segments"<<endl;
								return false;
							}
						}
						else
						{
							S.del_right(sh, eventSched);
						}
					}
					eventSched.select_next(object, status);
				}
				
				//cerr << "Okey, no intersect" << endl;
				long loc=FLAGS;
				long locbro=-1;
				
				poly2D::Iterator pi=getIterator();
				
				hseg2D current;
				hseg2D next;
				hseg2D nextBro;
				hseg2D first; //used to know when the cycle was completed
				hseg2D firstBro;
				long hsegsfound=1; //used to verify if cycle will ever complete
				bool value=this->isOuterCycle(); //start with default value
				
				//structures used to count number of visits per point
				vector<poi2D> endPoints;
				vector<poi2D>::iterator currentPoiPosition;
				vector<uint> endPointFrequency;
				vector<uint>::iterator currentFreqPosition;
				vectorBinarySearchWrapper< vector<poi2D>, poi2D > binSearcher( &endPoints );
				poi2D currPoi;
				uint index;
				
				if (pi.next()) current=*pi.get(); //get first hseg, always left
				first=current;
				
				if (first.isInsideAbove()!=value) //then its wrong, reassign insideAbove
				{
					grandpa->removeHalfSegment(first);
							
					first.setInsideAbove(value);
					first.validate();
					addhseg(first); //&&can optimize by giving address where it was found (loc)
// 					uint sp=first.serialSize();
// 					unsigned char *data=new unsigned char[sp];
// 					uint len=first.serialize(data);
// 				
// 					Locator added(false);
// 					added=grandpa->hsegs.insert(data,len,priloc);//re-add hseg
// 					elements.insert(added,loc);
// 					delete data;
				}

				firstBro=findBrother(first,locbro);
				
				if (locbro==-1)
					throw runtime_error("Could not find brother of first.");
				
				if (firstBro.isInsideAbove()==first.isInsideAbove()) //then its wrong, reassign insideAbove
				{
					grandpa->removeHalfSegment(firstBro);
					
					firstBro.setInsideAbove(!first.isInsideAbove());

					firstBro.validate();
					addhseg(firstBro); //&&can optimize by giving address where it was found (locbro)
//  					uint sp=firstBro.serialSize();
// 					unsigned char *data=new unsigned char[sp];
// 					uint len=firstBro.serialize(data);
// 				
// 					Locator added(false);
// 					added=grandpa->hsegs.insert(data,len,priloc);//re-add hseg
// 					elements.insert(added,locbro);
// 					delete data;
				}

				//now update point frequency vector first and firstBro
				currPoi = first.getDominating( );
				if( binSearcher.find( currPoi, index ) )
				{
						// if the point is found, update the frequencey
					endPointFrequency[index]++;
				} else
				{
						// if the endpoint is not found, insert it and make room 
						// in the frequencey vector, setting the frequency to 1
					currentPoiPosition = endPoints.begin();
					currentPoiPosition += index;
					endPoints.insert( currentPoiPosition, currPoi );
					currentFreqPosition = endPointFrequency.begin();
					currentFreqPosition += index;
					endPointFrequency.insert( currentFreqPosition, 1 );
				}
						
				currPoi = firstBro.getDominating( );
				if( binSearcher.find( currPoi, index ) )
				{
						// if the point is found, update the frequencey
					endPointFrequency[index]++;
				} else
				{
						// if the endpoint is not found, insert it and make room 
						// in the frequencey vector, setting the frequency to 1
					currentPoiPosition = endPoints.begin();
					currentPoiPosition += index;
					endPoints.insert( currentPoiPosition, currPoi );
					currentFreqPosition = endPointFrequency.begin();
					currentFreqPosition += index;
					endPointFrequency.insert( currentFreqPosition, 1 );
				}
				
				//now move through the chain
				while (true){
					//send brother as next
					dvector<hseg2D> result;
					stuVector<long> locs;

					long count=findNextsInChain(current, loc, result, locs);

					if (count>1){
						invalidate(); //too many hsegs traversed
/*						cerr << "Found articulation point. Number segs coming from it: " << count << endl;*/
// 						cerr << "First: " << result[0] << endl;
// 						cerr << "Second: " << result[1] << endl;
// 						cerr << "Third: " << result[2] << endl;
						cerr << "Too many hsegs traversed." << endl;
						return valid;
					}else if (count==0){
						cerr << "Broken chain, no next found." << endl;
						return false;
					}

					next=result[0];

					hsegsfound++;
					if (hsegsfound>(((static_cast<long long int>(elements.count())-FLAGS)/2)+1)){
						invalidate(); //too many hsegs traversed
						cerr << "Too many hsegs: " << hsegsfound << " elements.count: " << elements.count() << endl;
						return valid;
					}
					if (loc==-1){// could not continue the chain
						invalidate();
						cerr << "Chain is broken!" << endl;
						return valid;
					}else{
						
/*						cout << "Next found: " << next << endl;*/
						
						if (next==first){ //get out of loop, chain completed
							break;
						}
							
						if (current.isLeft()!=next.isLeft()){
							value=(!value); //there is a turn, reinitialize value
						}
						
						current=next; //keep the chain going
						
						hseg2D* left;
						hseg2D* right;
						long locleft;
						long locright;
						
						nextBro=findBrother(next,locbro);
						
						if (locbro==-1){//there is a problem with the polygon, do not validate
							throw runtime_error("Corrupted poly2D, cannot validate,");
						}
						
						if (next.isLeft()){ //use the location of the original found
							left=&next;
							locleft=loc;
							right=&nextBro;
							locright=locbro;
						}
						else{
							right=&next;
							locright=loc;
							left=&nextBro;
							locleft=locbro;
						}
							
						//now check left
						if (left->isInsideAbove()!=value) //then its wrong, reassign insideAbove
						{
							grandpa->removeHalfSegment(*left);
							
							left->setInsideAbove(value);
							left->validate();
							
							addhseg(*left); //&&can optimize by giving address where it was found (locleft)

							// 							uint sp=left->serialSize();
// 							unsigned char *data=new unsigned char[sp];
// 							uint len=left->serialize(data);
// 							
// 							Locator added(false);
// 							added=grandpa->hsegs.insert(data,len,priloc);//re-add hseg
// 							
// 							elements.insert(added,locleft);
// 							delete data;
						}
						
						//now check right
						if (right->isInsideAbove()==left->isInsideAbove()) //then its wrong, reassign insideAbove
						{
							grandpa->removeHalfSegment(*right);
							
							right->setInsideAbove(!left->isInsideAbove());
							right->validate();
							
							addhseg(*right); //&&can optimize by giving address where it was found (locright)

// 							uint sp=right->serialSize();
// 							unsigned char *data=new unsigned char[sp];
// 							uint len=right->serialize(data);
// 							
// 							Locator added(false);
// 							added=grandpa->hsegs.insert(data,len,priloc);//re-add hseg
// 							
// 							elements.insert(added,locright);
// 							delete data;
						}
						
						
						//now update point frequency vector for left and right
						currPoi = left->getDominating( );
						if( binSearcher.find( currPoi, index ) )
						{
						// if the point is found, update the frequencey
							endPointFrequency[index]++;
						} else
						{
						// if the endpoint is not found, insert it and make room 
						// in the frequencey vector, setting the frequency to 1
							currentPoiPosition = endPoints.begin();
							currentPoiPosition += index;
							endPoints.insert( currentPoiPosition, currPoi );
							currentFreqPosition = endPointFrequency.begin();
							currentFreqPosition += index;
							endPointFrequency.insert( currentFreqPosition, 1 );
						}
						
						currPoi = right->getDominating( );
						if( binSearcher.find( currPoi, index ) )
						{
						// if the point is found, update the frequencey
							endPointFrequency[index]++;
						} else
						{
						// if the endpoint is not found, insert it and make room 
						// in the frequencey vector, setting the frequency to 1
							currentPoiPosition = endPoints.begin();
							currentPoiPosition += index;
							endPoints.insert( currentPoiPosition, currPoi );
							currentFreqPosition = endPointFrequency.begin();
							currentFreqPosition += index;
							endPointFrequency.insert( currentFreqPosition, 1 );
						}
					}
				}
				
				//cerr << "Will check counts now" << endl;
				
				if (hsegsfound<(((static_cast<long long int>(elements.count())-FLAGS)/2)+1)){
					invalidate();
					cerr << "Too few hsegs!!" << endl;
					return valid;
				}
				
				
				// now we have constructed the frequency vector
				// so now we must traveser it and find EXACTLY 2 points
				// with frequency 1, and the rest with frequency 2
				for( uint i = 0; i < endPointFrequency.size(); i++ )
				{
					//cerr << "EndPoint: " << endPoints[i] << " found " << endPointFrequency[i] << endl;
					if( endPointFrequency[i] != 2 ) //only dominating points counted...1 time for each hseg
					{
						valid = false;
						cerr<<"Not a valid poly: point appeared " << endPointFrequency[i] << " times." <<endl;
						return false;
					}
				}
				
				//else{
				valid=true;
				changed=true;
				if (!inBatch) saveValid();
				//}
				
			}else
				cerr << "Did not validate, already valid" << endl;
			
			return valid;
		}
		
		
		bool poly2D::isValid() const
		{
			return valid;
		}

		uint poly2D::getSegmentCount() const //returns the number of segments for the whole poly
		{
			return ((elements.count()-FLAGS)/2); //subtract number of flags
		}
						
		rat poly2D::getLength() const //returns the perimeter. sum of lenghts of segments
		{
			rat result(0);
			
			poly2D::Iterator bi=getIterator();
			hseg2D *t;
			
			
			while (bi.next())
			{
				t=bi.get();
				if (t->isLeft()) result+=t->getLength();
			}

			return result;
		}
		
		bool poly2D::isEmpty() const //returns getSize()==0
		{
			return (elements.count()<2); //first element could be valid flag
		}

		mbb2D poly2D::getmbb() const 
		{
			if (!valid)
				throw runtime_error("Invalid poly2D. No MBB Available");
	
			rat minx(0);
			rat miny(0);
			rat maxx(0);
			rat maxy(0);
			bool first=true;
	
			Iterator li=getIterator();
			
			while (li.next()){ //for each
				hseg2D *temp=li.get();
				
				if (first) //first object, assign to all
				{
					minx=(temp->getDominating().getX()<temp->getSubmissive().getX()?temp->getDominating().getX():temp->getSubmissive().getX());
					miny=(temp->getDominating().getY()<temp->getSubmissive().getY()?temp->getDominating().getY():temp->getSubmissive().getY());
					maxx=(temp->getDominating().getX()>temp->getSubmissive().getX()?temp->getDominating().getX():temp->getSubmissive().getX());
					maxy=(temp->getDominating().getY()>temp->getSubmissive().getY()?temp->getDominating().getY():temp->getSubmissive().getY());
					first=false;
				}
				else
				{
					rat minsegx=(temp->getDominating().getX()<temp->getSubmissive().getX()?temp->getDominating().getX():temp->getSubmissive().getX());
					rat minsegy=(temp->getDominating().getY()<temp->getSubmissive().getY()?temp->getDominating().getY():temp->getSubmissive().getY());
					rat maxsegx=(temp->getDominating().getX()>temp->getSubmissive().getX()?temp->getDominating().getX():temp->getSubmissive().getX());
					rat maxsegy=(temp->getDominating().getY()>temp->getSubmissive().getY()?temp->getDominating().getY():temp->getSubmissive().getY());
	
					if (minsegx<minx)
						minx=minsegx;
					if (minsegy<miny)
						miny=minsegy;
					if (maxsegx>maxx)
						maxx=maxsegx;
					if (maxsegy>maxy)
						maxy=maxsegy;
				}	
			}
			mbb2D r(poi2D(minx,miny),poi2D(maxx,maxy));
			return r;
		}

		bool poly2D::isOuterCycle() const {return isOuter; }
		
		poly2D::Iterator poly2D::getIterator(long start) const //sorted in half-segment order.		
		{
			poly2D::Iterator p(const_cast<poly2D*>(this),start);
			return p;
		}
		
		bool poly2D::addhseg(const hseg2D &h) throw (runtime_error, invalidObjectException)
		{
			if (!h.isValid())
				throw invalidObjectException();
		
			Locator added(false);
			uint loc=FLAGS; //loc starts here cause 0 is valid flag and 1 is outer flag
			hseg2D *t;

			if (grandpa!=NULL){
				added=(parent->addhseg(h)); //add this hseg to parent face (which in turn should add to parent region)
			}else{
				throw runtime_error("Polygons must belong to faces->regions.");
			}
			//now add reference to secondary structure of line (this poly)
			
// 			poly2D::Iterator bi=getIterator();
// 			loc=FLAGS;
// 			
// 			while (bi.next()){
// 				t=bi.get();
// 				if (*t==h)
// 					return false; //do not add, duplicate
// 				if (*t>h)
// 					break; //then add here		
// 				loc++;
// 			}
// 		
// 			if (bi.end()) //reached end so insert last
// 				elements.insert(added,elements.count());
// 			else
// 				elements.insert(added,loc);//add p to the end.
			
			vectorBinarySearchWrapper< poly2D, hseg2D > finder ( this ); 
			
			uint found=0;
			
			if (finder.find(h,found)){
				return false; //do not add, duplicate
			}else
			{
				long loc=static_cast<long>(found)+FLAGS;
				elements.insert(added,loc);//add h
			}	
			
/* 			cout << "Added hseg to poly, elements: "; elements.debug();
			cout << "The poly so far: " << *this << endl;*/
			
			return true;
		}

		
		std::ostream& operator << (std::ostream &ostr, const poly2D &rhs) throw( invalidObjectException )
		{
			if( !rhs.isValid( ) ) {
				ostr<< "--(Invalid Polygon)--" ;
			}
			//cout << "There are " << rhs.elements.count() << " pois in this point" << endl;
			poly2D::Iterator p1=rhs.getIterator();
			hseg2D* pp1;
			
			ostr << "[";
			while (p1.next()){
				pp1=p1.get();
				ostr << *pp1;
			}
			ostr << "]";
			return ostr;
		}
} //namespace
