/**********************************************************************/
/******************************************/
/* A Very Simple OpenGL Example!          */
/******************************************/

/* this code just creates a window and draws a rectangle in it */

#include <iostream>
#include <vector>
#include <gl.h>
#include <glut.h>
#include "segment.h"
#include "fileReader.h"
	
//funcs for glutness
void init(void);
void display(void);
void keyboard(unsigned char, int, int);
void bitmap_output(int, int, char *, void *);

//default viewable area of the plane
float myGlXMin = 0.0;
float myGlXMax = 100.0;
float myGlYMin = 0.0;
float myGlYMax = 100.0;
float myGlZMin = -100.0;
float myGlZMax = 100.0;
//default zoom (no zooming)
float zoomFactor = 1.0;
//default is show help menu
bool showHelp = true;

vector<segment> greenSegs, blueSegs;

int main (int argc, char **argv)
{
	string fileName1, fileName2;
	if( argc < 3 )
	{
		cout<<"file name 1 -+> ";
		cin>> fileName1;
		cout<<"file name 2 -+> ";
		cin>> fileName2;
	}
	else
	{
		fileName1.assign( argv[1] );
		fileName2.assign( argv[2] );
	}
		
	// read in the segment data
	fileReader freader1( fileName1 );
	freader1.readFile( greenSegs );
	fileReader freader2( fileName2 );
	freader2.readFile( blueSegs );
	
	// initialize the glutness
	glutInit(&argc, argv);
	glutInitDisplayMode(GLUT_DOUBLE | GLUT_RGB);
	glutInitWindowSize(600, 600);
	glutInitWindowPosition(100, 100);
	glutCreateWindow("Lines, lines, pretty little lines...");
	init();
	glutDisplayFunc(display);
	glutKeyboardFunc(keyboard);
	glutMainLoop();
	return 0;
}

void init(void)
{
	glClearColor(1.0, 1.0, 1.0, 1.0);
	glColor3f(0.0, 0.0, 1.0);
	glMatrixMode(GL_PROJECTION);	
	glLoadIdentity();
	glOrtho( myGlXMin, myGlXMax, myGlYMin, myGlYMax, myGlZMin, myGlZMax );
}

void display(void)
{
	//reset the scene, set the view size and zoom
	glLoadIdentity();
	glOrtho( myGlXMin, myGlXMax, myGlYMin, myGlYMax, myGlZMin, myGlZMax );
	glScalef( zoomFactor, zoomFactor, zoomFactor );
	//set some color for the lines and points
	glColor3f(0.0, 1.0, 0.0);
	glClear(GL_COLOR_BUFFER_BIT);
	glLineWidth(2.0);
	glPointSize(8.0);
	
	// draw the green lines
	glBegin(GL_LINES);
	for( int i = 0; i < greenSegs.size(); i++ )
	{
		glVertex3f(greenSegs[i].getLX(), greenSegs[i].getLY(), 0.0);
		glVertex3f(greenSegs[i].getRX(), greenSegs[i].getRX(), 0.0);
	}
	glEnd();
	
	//draw the green vertices
	glBegin(GL_POINTS);
	for( int i = 0; i < greenSegs.size(); i++ )
	{
		glVertex3f(greenSegs[i].getLX(), greenSegs[i].getLY(), 0.0);
		glVertex3f(greenSegs[i].getRX(), greenSegs[i].getRX(), 0.0);
	}
	glEnd();
	
	// draw the blue lines
	glColor3f(0.0, 0.0, 1.0);
	glBegin(GL_LINES);
	for( int i = 0; i < blueSegs.size(); i++ )
	{
		glVertex3f(blueSegs[i].getLX(), blueSegs[i].getLY(), 0.0);
		glVertex3f(blueSegs[i].getRX(), blueSegs[i].getRX(), 0.0);
	}
	glEnd();
	
	//draw the blue vertices
	glBegin(GL_POINTS);
	for( int i = 0; i < blueSegs.size(); i++ )
	{
		glVertex3f(blueSegs[i].getLX(), blueSegs[i].getLY(), 0.0);
		glVertex3f(blueSegs[i].getRX(), blueSegs[i].getRX(), 0.0);
	}
	glEnd();
	
	// show the help menu
	if( showHelp )
	{
		glPushMatrix();                             // New frame of reference
			glLoadIdentity();
			glOrtho( myGlXMin, myGlXMax, myGlYMin, myGlYMax, myGlZMin, myGlZMax );
			glTranslatef(myGlXMax-18, myGlYMin+5.0, 0);              // Move to new location
			glColor3f(0.0, 0.0, 0.0);
			bitmap_output(0,0, "w,a,s,d: move around", GLUT_BITMAP_HELVETICA_10);
			glTranslatef(0.0, -2.0, 0);  
			bitmap_output(0,0, "z,x,c: zoom, center", GLUT_BITMAP_HELVETICA_10);
			glTranslatef(0.0, -2.0, 0);  
			bitmap_output(0,0, "q, h: quit,help", GLUT_BITMAP_HELVETICA_10);
		glPopMatrix();                              // Return to old reference
	}
	
	// put it on the screen
	glutSwapBuffers();
}

void keyboard(unsigned char key, int x, int y)
{
		/* this is the keyboard event handler
		the x and y parameters are the mouse 
		coordintes when the key was struck */
	switch (key)
	{
		case 'i':
		case 'I':
			glRotatef(3.0, 1.0, 0.0, 0.0); /* rotate up */
			break;
		case 'k':
		case 'K':
			glRotatef(-3.0, 1.0, 0.0, 0.0); /* rotate down */
			break;
		case 'l':
		case 'L':
			glRotatef(3.0, 0.0, 1.0, 0.0); /* rotate left */
			break;
		case 'j':
		case 'J':
			glRotatef(-3.0, 0.0, 1.0, 0.0); /* rotate right */
			break;
		case 'w':
		case 'W':
			myGlYMin -= 2;
			myGlYMax -= 2;
			break;
		case 's':
		case 'S':
			myGlYMin += 2;
			myGlYMax += 2;
			break;
		case 'a':
		case 'A':
			myGlXMin += 2;
			myGlXMax += 2;
			break;
		case 'd':
		case 'D':
			myGlXMin -= 2;
			myGlXMax -= 2;
			break;
		case 'z':
		case 'Z':
/* 			myGlXMin += 0;
 			myGlXMax -= 2;
 			myGlYMin += 0;
 			myGlYMax -= 2;*/
			zoomFactor += 0.1;
			if( zoomFactor == 0 ) zoomFactor += 0.1;
// 			glScalef( 1.1, 1.1, 1.1 );
			break;
		case 'x':
		case 'X':
// 			myGlXMin += 2;
// 			myGlXMax += 2;
// 			myGlYMin += 2;
// 			myGlYMax += 2;
			zoomFactor -= 0.1;
			if( zoomFactor == 0 ) zoomFactor -= 0.1;
// 			glScalef( .9, .9, .9 );
			break;
		case 'c':
		case 'C':
			myGlXMin = 0.0;
			myGlXMax = 100.0;
			myGlYMin = 0.0;
			myGlYMax = 100.0;
			zoomFactor = 1.0;
			break;
		case 'h':
		case 'H':
			showHelp = !showHelp;
			break;
		case 'q':
		case 'Q':
			exit( 1 );
			break;
	}
	display(); /* repaint the window */
}

//draws text on the screen
void bitmap_output(int x, int y, char *string, void *font)
{  
	int len, i;
	glRasterPos3f(x, y, 0);            // Locate Raster Position in 3-space
	len = (int) strlen(string);        // Find length of string
	for (i = 0; i < len; i++)
	{
		// Loop through plotting all characters in font style
		glutBitmapCharacter(font, string[i]);
	}
}
/**********************************************************************/

