#ifndef LINEWRAPPER_H
#define LINEWRAPPER_H

#include <gl.h>
#include <glut.h>

#include <vector>
#include <string>
#include <sstream>
//slob includes
#include "block2D.h"
#include "line2D.h"
#include <occi.h>
#include "constants.h"

#include "spatialObjectWrapper.h"

class lineWrapper: public spatialObjectWrapper
{
	private:
		line2D *lPtr;
		vector<hseg2D> vec;
		bool reReadDb;
		vector<seg2D> insertedSegVec;
		// if constructed from a pointer, don't free the memory
		bool needToDelete;
		poi2D prevEndPoint;
		poi2D newEndPoint;
		bool addingNewBlock;
		
	public:
		lineWrapper(  OCIEnv *envHP, OCIError *errHP, OCISession *authP, OCIServer *srvHP, OCISvcCtx *svcHP, uint pk, bool useDefaults = true ): spatialObjectWrapper( envHP, errHP, authP, srvHP, svcHP, pk ), lPtr( NULL ), reReadDb( true ), needToDelete( true ), addingNewBlock( false )
		{
			if( !useDefaults )
				changeDatabaseSelectStatement( pk );
			lPtr = new line2D( readFromDB( pk ) );
			lPtr->validate();
			cout<<*lPtr<<endl;
		}
		
		lineWrapper(  OCIEnv *envHP, OCIError *errHP, OCISession *authP, OCIServer *srvHP, OCISvcCtx *svcHP,  bool useDefaults = true ): spatialObjectWrapper( envHP, errHP, authP, srvHP, svcHP, 0 ), lPtr( NULL ), reReadDb( true ), needToDelete( true ), addingNewBlock( false )
		{
			if( !useDefaults )
				changeDatabaseSelectStatement(  );
			lPtr = new line2D( readFromDB( ) );
			lPtr->validate();
			cout<<*lPtr<<endl;
		}
		
		lineWrapper( line2D *pointer ):  lPtr( pointer ), reReadDb( true ), needToDelete( false ), addingNewBlock( false )
		{
			lPtr->validate();
			cout<<*lPtr<<endl;
		}
		
		~lineWrapper( )
		{
			clean( );
		}
		
		void clean( )
		{
			cout<<"cleaning line wrapper"<<endl;
			if( pm!= NULL ){ pm->commitBatch(); pm = NULL; }
			if( lPtr != NULL && needToDelete )
			{	
				delete lPtr;
				lPtr = NULL;
			}
			spatialObjectWrapper::clean( );
		}
		
		bool isLine( ){ return true; }
		
		virtual void display( bool showCoords, uint color )
		{

			if( reReadDb )
			{
				//re read lines from the DB
				copyLineData(  );
				cout<<"REREAD:"<<*lPtr<<endl;
				reReadDb = false;
			}
			
			//draw the lines
			
			if( color == COLOR_GREEN )
				glColor3f(0.0, 1.0, 0.0);
			else if( color == COLOR_BLUE )
				glColor3f(0.0, 0.0, 1.0);
			else if( color == COLOR_RED )
				glColor3f(1.0, 0.0, 0.0);
			
			glBegin(GL_LINES);
			for( uint i = 0; i < vec.size(); i++ )
			{
				glVertex3f( vec[i].getDominating().getX().toDouble(), vec[i].getDominating().getY().toDouble(), 0.0 );
				glVertex3f( vec[i].getSubmissive().getX().toDouble(), vec[i].getSubmissive().getY().toDouble(), 0.0 );
			}
			glEnd();
			
			
			
			glBegin(GL_LINES);
			for( uint i = 0; i < insertedSegVec.size(); i++ )
			{
				glVertex3f( insertedSegVec[i].getP().getX().toDouble(), insertedSegVec[i].getP().getY().toDouble(), 0.0 );
				glVertex3f( insertedSegVec[i].getQ().getX().toDouble(), insertedSegVec[i].getQ().getY().toDouble(), 0.0 );
			}
			glEnd();
				
			// show segment from the last inserted segment to the current mouse pos
			if( addingNewBlock )
			{
				glBegin(GL_LINES);
				glVertex3f( prevEndPoint.getX().toDouble(),prevEndPoint.getY().toDouble(), 0.0 );
				glVertex3f( newEndPoint.getX().toDouble(), newEndPoint.getY().toDouble(), 0.0 );
				glEnd();
			}
			
			// draw the vertices
			glColor3f(0.0, 0.0, 0.0);
			glPointSize(3.0);
			glBegin(GL_POINTS);
			for( uint i = 0; i < vec.size(); i++ )
			{
				glVertex3f( vec[i].getDominating().getX().toDouble(), vec[i].getDominating().getY().toDouble(), 0.0 );
			}
			glEnd();
			
			if( showCoords )
			{
				
				float fx, fy;
				int ix, iy;
				string outputStr;
				
				for( uint i = 0; i < vec.size(); i++ )
				{
					fx = vec[i].getDominating().getX().toDouble();
					fy = vec[i].getDominating().getY().toDouble();
					ix = static_cast<int>(fx);
					iy = static_cast<int>(fy);
						
					glColor3f(0.0, 0.0, 0.0);
					ostringstream oss;
					oss<<'('<<fx<<','<<fy<<')';
					outputStr.assign( oss.str() );
					glColor3f(0.0, 0.0, 0.0);
					bitmap_output(ix,iy,const_cast<char*>( outputStr.c_str() ), GLUT_BITMAP_HELVETICA_10);
					glColor3f(0.0, 1.0, 0.0);
				}
			}
		}	
		
		line2D* getPtr( )
		{
			return lPtr;
		}
		
		void copyLineData(  )
		{
			vec.clear();
			line2D::Iterator lIter= lPtr->getIterator();
			while (lIter.next())
			{
				vec.push_back( *lIter.get() );
			}
		}
		
		line2D::Iterator getIterator( )
		{
			return lPtr->getIterator( );
		}
		
		void updateNewEndpoint( int plotX, int plotY )
		{
			poi2D temp( plotX, plotY );
			newEndPoint = temp;
		}
		
		void insertBlockHSeg( int plotX, int plotY )
		{
			// needs to keep a buffer of the new block to add
			// then add the block when finished

			updateNewEndpoint( plotX, plotY );
			
			if( insertedSegVec.size( ) == 0 && !addingNewBlock )
			{
				addingNewBlock = true;
				//first new point inserted
				prevEndPoint = newEndPoint;
				return;
			}
			else 
			{
				insertedSegVec.push_back( seg2D( prevEndPoint, newEndPoint ) );
				prevEndPoint = newEndPoint;
				return;
			}
			
		}
		
		void commitChanges( )
		{
			addingNewBlock = false;
			// take all the segs from the buffer and make a block
			block2D newBlock = lPtr->addBlock( );
			uint arraySize = insertedSegVec.size();
			seg2D segArray[ arraySize ];
			
			// create an array to use as a function argument
			for( uint i = 0; i < arraySize; i++ )
				segArray[i] = insertedSegVec[i];
			
			// actually add the new segments
			newBlock.addSegments( segArray, arraySize );
			newBlock.validate();
			lPtr->validate();
			// clear the buffer;
			insertedSegVec.clear( );
			
			reReadDb=true;
		}
		
		void removeBlock( int plotX, int plotY )
		{
			rat x( plotX );
			rat y( plotY );
			poi2D poiToRemove( x, y );
			
			uint blockIndexToRemove = 0;
			if( getSelectedBlockID( poiToRemove, blockIndexToRemove ) )
			{
				lPtr->removeBlock( blockIndexToRemove );
				reReadDb = true;
			}
		}
		
		bool getSelectedBlockID( poi2D selectPoint, uint & selectedBlockID )
		{
			block2D blocky;
			
			rat tmpDist;
			rat zero( 0 );
			rat negOne( -1 );
			rat buffer( 2 );
			
			uint numBlocks = lPtr->getBlockCount( );
			for( uint i = 0; i < numBlocks  ; i++ )
			{
				blocky = lPtr->getBlock( i );
				block2D::Iterator bIter = blocky.getIterator();
				while( bIter.next( ) )
				{
					tmpDist = bIter.get( )->getSegment( ).distance( selectPoint );
					if( tmpDist < buffer )
					{
						selectedBlockID = i;
						return true;
					}
				}
			}
			return false;
		}
		
		//draws text on the screen
		void bitmap_output(int x, int y, char *string, void *font)
		{  
			int len, i;
			glRasterPos3f(x, y, 0);            // Locate Raster Position in 3-space
			len = (int) strlen(string);        // Find length of string
			for (i = 0; i < len; i++)
			{
				// Loop through plotting all characters in font style
				glutBitmapCharacter(font, string[i]);
			}
		}
};

#endif

