#ifndef MYGLMENUSUBWINDOW_H
#define MYGLMENUSUBWINDOW_H

#include <gl.h>
#include <glut.h>
#include "button.h"
#include <vector>
#include "myGlWorkspaceSubWindow.h"
#include "constants.h"


class myGlMenuSubWindow
{
	protected:
		int winID; //glut window id
		int parentWinID; // glut id of parent window
		//default viewable area of the plane
		int myGlXMin;
		int myGlXMax;
		int myGlYMin;
		int myGlYMax ;
		int myGlZMin;
		int myGlZMax;
		const uint initOrthoWidth; //initial ortho window width
		const uint initOrthoHeight; //initial ortho window height
		
		const uint initWindowWidth; //initial actual window width
		const uint initWindowHeight; //initial actual window height
		
		uint windowWidth; //current actual window width
		uint windowHeight; //current actual window height
		
		const uint initWindowPositionX; //initial window position
		const uint initWindowPositionY;
		
		float zoomFactor; //default zoom (no zooming)
		
		int mousePosX; //last recorded mouse poisition in the window
		int mousePosY; //last recorded mouse poisition in the window
		bool mouseInScreen; // true if mouse is over this window
		
		//menu buttons
		vector<button> objectSelectButtons;
		vector<button> actionButtons;
		vector<button> modifyButtons;
		vector<button> stateButtons;
		vector<button> fileButtons;
		vector<button> operationButtons;
		
		bool showMouseCoords;
		
		//pointer to let the workspace know which button is selected
		myGlWorkspaceSubWindow *workWin;
		
	public:
		myGlMenuSubWindow( int parentID, uint initPosX, uint initPosY, uint winWidth, uint winHeight,  int orthoMinX, int orthoMaxX, int orthoMinY, int orthoMaxY, float zoomFac, myGlWorkspaceSubWindow * workWindow ): parentWinID( parentID ), myGlXMin(orthoMinX), myGlXMax(orthoMaxX), myGlYMin(orthoMinY), myGlYMax(orthoMaxY), myGlZMin(-100), myGlZMax(100),  initOrthoWidth( myGlXMax - myGlXMin ), initOrthoHeight( myGlYMax - myGlYMin ), initWindowWidth( winWidth ), initWindowHeight( winHeight ),  windowWidth(initWindowWidth ), windowHeight( initWindowHeight),  initWindowPositionX( initPosX  ), initWindowPositionY( initPosY ), zoomFactor(zoomFac), mousePosX(0), mousePosY(0),  mouseInScreen( false ), showMouseCoords( false ),  workWin( workWindow )
		{
			objectSelectButtons.push_back( button("Select Green", orthoMinX, orthoMaxY-(0*BUTTON_HEIGHT), BUTTON_WIDTH, BUTTON_HEIGHT, SELECT_GREEN_BUTTON) );
			objectSelectButtons.push_back( button("Select Blue", orthoMinX, orthoMaxY-(1*BUTTON_HEIGHT), BUTTON_WIDTH, BUTTON_HEIGHT, SELECT_BLUE_BUTTON ) );
			actionButtons.push_back( button("+Zoom", orthoMinX, orthoMaxY-(2*BUTTON_HEIGHT), BUTTON_WIDTH/2, BUTTON_HEIGHT, ZOOM_IN_BUTTON ) );
			actionButtons.push_back( button("-Zoom", orthoMinX+BUTTON_WIDTH/2, orthoMaxY-(2*BUTTON_HEIGHT), BUTTON_WIDTH/2, BUTTON_HEIGHT, ZOOM_OUT_BUTTON) );
			actionButtons.push_back( button("Move View", orthoMinX, orthoMaxY-(3*BUTTON_HEIGHT), BUTTON_WIDTH, BUTTON_HEIGHT, MOVE_VIEW_BUTTON) );
			//modifyButtons.push_back( button("delaunay", orthoMinX, orthoMaxY-(4*BUTTON_HEIGHT), BUTTON_WIDTH, BUTTON_HEIGHT, DELAUNAY_TRIANGULATION_BUTTON ) );
			modifyButtons.push_back( button("I:point", orthoMinX, orthoMaxY-(5*BUTTON_HEIGHT), BUTTON_WIDTH/2, BUTTON_HEIGHT, INSERT_POINT_BUTTON ) );
			
			modifyButtons.push_back( button("R:point", orthoMinX+BUTTON_WIDTH/2, orthoMaxY-(5*BUTTON_HEIGHT), BUTTON_WIDTH/2, BUTTON_HEIGHT, REMOVE_POINT_BUTTON ) );
			modifyButtons.push_back( button("I:block", orthoMinX, orthoMaxY-(6*BUTTON_HEIGHT), BUTTON_WIDTH/2, BUTTON_HEIGHT, INSERT_BLOCK_BUTTON ) );
			modifyButtons.push_back( button("R:block", orthoMinX+BUTTON_WIDTH/2, orthoMaxY-(6*BUTTON_HEIGHT), BUTTON_WIDTH/2, BUTTON_HEIGHT, REMOVE_BLOCK_BUTTON ) );
			modifyButtons.push_back( button("I:face", orthoMinX, orthoMaxY-(7*BUTTON_HEIGHT), BUTTON_WIDTH/2, BUTTON_HEIGHT, INSERT_FACE_BUTTON ) );
			modifyButtons.push_back( button("R:face", orthoMinX+BUTTON_WIDTH/2, orthoMaxY-(7*BUTTON_HEIGHT), BUTTON_WIDTH/2, BUTTON_HEIGHT, REMOVE_FACE_BUTTON ) );
			modifyButtons.push_back( button("I:hole", orthoMinX, orthoMaxY-(8*BUTTON_HEIGHT), BUTTON_WIDTH/2, BUTTON_HEIGHT, INSERT_HOLE_BUTTON ) );
			modifyButtons.push_back( button("R:hole", orthoMinX+BUTTON_WIDTH/2, orthoMaxY-(8*BUTTON_HEIGHT), BUTTON_WIDTH/2, BUTTON_HEIGHT, REMOVE_HOLE_BUTTON ) );
			operationButtons.push_back( button("Validate", orthoMinX, orthoMaxY-(9*BUTTON_HEIGHT), BUTTON_WIDTH, BUTTON_HEIGHT, VALIDATE_BUTTON) );
			
			operationButtons.push_back( button("Inter", orthoMinX, orthoMaxY-(10*BUTTON_HEIGHT), BUTTON_WIDTH/2, BUTTON_HEIGHT, INTERSECTION_BUTTON ) );
			operationButtons.push_back( button("Union", orthoMinX+BUTTON_WIDTH/2, orthoMaxY-(10*BUTTON_HEIGHT), BUTTON_WIDTH/2, BUTTON_HEIGHT, UNION_BUTTON ) );
			operationButtons.push_back( button("Diff", orthoMinX, orthoMaxY-(11*BUTTON_HEIGHT), BUTTON_WIDTH/2, BUTTON_HEIGHT, DIFFERENCE_BUTTON ) );
			operationButtons.push_back( button("Dist", orthoMinX+BUTTON_WIDTH/2, orthoMaxY-(11*BUTTON_HEIGHT), BUTTON_WIDTH/2, BUTTON_HEIGHT, MIN_DISTANCE_BUTTON ) );
			operationButtons.push_back( button("Clear", orthoMinX, orthoMaxY-(12*BUTTON_HEIGHT), BUTTON_WIDTH/2, BUTTON_HEIGHT, CLEAR_OPERATION_BUTTON ) );
			operationButtons.push_back( button("Convex", orthoMinX+BUTTON_WIDTH/2, orthoMaxY-(12*BUTTON_HEIGHT), BUTTON_WIDTH/2, BUTTON_HEIGHT, CONVEX_HULL_BUTTON ) );
			operationButtons.push_back( button("Area", orthoMinX, orthoMaxY-(13*BUTTON_HEIGHT), BUTTON_WIDTH/2, BUTTON_HEIGHT, AREA_OPERATION_BUTTON ) );
			operationButtons.push_back( button("Length", orthoMinX+BUTTON_WIDTH/2, orthoMaxY-(13*BUTTON_HEIGHT), BUTTON_WIDTH/2, BUTTON_HEIGHT, LENGTH_OPERATION_BUTTON ) );
			operationButtons.push_back( button("Voronoi", orthoMinX, orthoMaxY-(14*BUTTON_HEIGHT), BUTTON_WIDTH/2, BUTTON_HEIGHT, VORONOI_BUTTON ) );
			operationButtons.push_back( button("Delaunay", orthoMinX+BUTTON_WIDTH/2, orthoMaxY-(14*BUTTON_HEIGHT), BUTTON_WIDTH/2, BUTTON_HEIGHT, DELAUNAY_TRIANGULATION_BUTTON ) );
			
			fileButtons.push_back( button("Load point", orthoMinX, orthoMaxY-(15*BUTTON_HEIGHT), BUTTON_WIDTH, BUTTON_HEIGHT, LOAD_POINT_BUTTON ) );
			fileButtons.push_back( button("Load line", orthoMinX, orthoMaxY-(16*BUTTON_HEIGHT), BUTTON_WIDTH, BUTTON_HEIGHT, LOAD_LINE_BUTTON ) );
			fileButtons.push_back( button("Load region", orthoMinX, orthoMaxY-(17*BUTTON_HEIGHT), BUTTON_WIDTH, BUTTON_HEIGHT, LOAD_REGION_BUTTON ) );
			
			stateButtons.push_back( button("Show coords", orthoMinX, orthoMaxY-(18*BUTTON_HEIGHT), BUTTON_WIDTH, BUTTON_HEIGHT, SHOW_MOUSE_COORDS_BUTTON ) );
		
			stateButtons.push_back( button("Quit", orthoMinX, orthoMaxY-(19*BUTTON_HEIGHT), BUTTON_WIDTH, BUTTON_HEIGHT, QUIT_BUTTON ) );
			
			winID = glutCreateSubWindow(parentWinID, initPosX, initPosY, winWidth, winHeight );
			glutSetWindow( winID );
			//glMatrixMode(GL_PROJECTION);	
			setupWindow( );
		}
		
		void setupWindow( )
		{
			glutSetWindow( winID );
			glLoadIdentity();
			glScalef( zoomFactor, zoomFactor, zoomFactor );
			glOrtho( myGlXMin, myGlXMax, myGlYMin, myGlYMax, myGlZMin, myGlZMax );
			glClearColor(1.0, 1.0, 1.0, 1.0);
			glClear(GL_COLOR_BUFFER_BIT);
			
			/* Draw border */
			glColor3f (1.0F, 0.0F, 0.0F);
			glBegin (GL_LINE_LOOP);
			glVertex2f( myGlXMin, myGlYMax-.1);
			glVertex2f (myGlXMax-.1, myGlYMax-.1);
			glVertex2f (myGlXMax-.1, myGlYMin);
			glVertex2f (myGlXMin, myGlYMin);
			glEnd ();
		}
		
		void display( )
		{
			setupWindow( );
			for( uint i = 0; i < objectSelectButtons.size(); i++ )
				objectSelectButtons[i].display( );
			for( uint i = 0; i < actionButtons.size(); i++ )
				actionButtons[i].display( );
			for( uint i = 0; i < modifyButtons.size(); i++ )
				modifyButtons[i].display( );
			for( uint i = 0; i < stateButtons.size(); i++ )
				stateButtons[i].display( );
			for( uint i = 0; i < fileButtons.size(); i++ )
				fileButtons[i].display( );
			for( uint i = 0; i < operationButtons.size(); i++ )
				operationButtons[i].display( );
			
			glutSwapBuffers();
		}
		
		
		
		void mouseEnterExit( int state )
		{
			if (state == GLUT_LEFT)
				mouseInScreen = false;
			else
				mouseInScreen = true;
		}
		
		void mouse( int button, int state, int x, int y )
		{
			// if mouse is not in the screen, then ignore the clicks
			
		
			if( ! mouseInScreen )
			return;
		
			glutSetWindow( winID );
			//first, translate the mouse coordinates to plotting coordinates 
			
			int plotX = convertXMouseCoordToScreenCoord( x );
			int plotY = convertYMouseCoordToScreenCoord( y );
			
			if( button == GLUT_LEFT_BUTTON && state == GLUT_UP )
			{
				for( uint i = 0; i < operationButtons.size(); i++ )
				{
					// clear any selected operation buttons
					operationButtons[i].select( false );
				}
				//redisplay the scene
				postRedisplay( );
			}
			
			uint modifyButtonToggledOffID = 0;
			bool modifyButtonToggledOff = false;
			if( button == GLUT_LEFT_BUTTON && state == GLUT_DOWN )
			{
				//handle if a modify button is selected
				// first check if a modify button is currently selected
				// if it is, then tell the work window to commit any
				// changes made to that object
				for( uint i = 0; i < modifyButtons.size( ); i++ )
				{
					if( modifyButtons[i].isSelected( ) )
					{
						workWin->commitChanges( );
						workWin->setActionCode( NO_ACTION );
						modifyButtons[i].select( false ) ;
						modifyButtonToggledOff = true;
						modifyButtonToggledOffID = i;
						break;
					}
				}
				
				
				//first handle the object select buttons
				//if one is clicked, select it and deselect the others
				for( uint i = 0; i < objectSelectButtons.size(); i++ )
				{
					if( objectSelectButtons[i].clicked(plotX, plotY ) )
					{
						//mark all the object select buttons as false
						for( uint j = 0; j < objectSelectButtons.size(); j++ )
							objectSelectButtons[j].select( false );
						//mark this button as true
						objectSelectButtons[i].select( true );
						// let the work window know which object is selected
						workWin->setSelectedTarget( objectSelectButtons[i].getCode( ) );
						postRedisplay( );
						return;
					}
				}
				
				//handle if an action button has been selected
				for( uint i = 0; i < actionButtons.size(); i++ )
				{
					if( actionButtons[i].clicked( plotX, plotY ) )
					{
						//select the button
						actionButtons[i].select();
						//notify the work window of the action selected
						workWin->setActionCode( actionButtons[i].getCode( ) );
						// now update the button pressed state of the other action buttons
						for( uint j = 0; j < actionButtons.size(); j++ )
						{
							if( i != j )
								actionButtons[j].select( false );
						}
						
						//finally, take care of any action that does not 
						// require a click in the work window
						switch( actionButtons[i].getCode( ) )
						{
							case ZOOM_IN_BUTTON:
								workWin->increaseZoom( );
								break;
							case ZOOM_OUT_BUTTON:
								workWin->decreaseZoom( );
								break;
						}
						
						//redisplay the scene
						postRedisplay( );
						return;
					}
				}
				
				
				
				// now handle the case that we are selecting a modify button
				for( uint i = 0; i < modifyButtons.size( ); i++ )
				{
					if( modifyButtons[i].clicked( plotX, plotY ) )
					{
						if( modifyButtonToggledOff && modifyButtonToggledOffID == i )
						{
							postRedisplay( );
							return;
						}
						modifyButtons[i].select( );
						workWin->setActionCode( modifyButtons[i].getCode( ) );
						postRedisplay( );
						return;
					}
				}
				
				//handle if an operation button has been selected
				for( uint i = 0; i < operationButtons.size(); i++ )
				{
					if( operationButtons[i].clicked( plotX, plotY ) )
					{
						//select the button
						operationButtons[i].select();
						
						// clear any other selected operation buttons
						for( uint j = 0; j < operationButtons.size(); j++ )
						{
							if( i != j )
								operationButtons[j].select( false );
						}
						display( );
						// perform the requested operation in the work window
						switch( operationButtons[i].getCode( ) )
						{
							case VALIDATE_BUTTON:
								workWin->performValidation( );
								break;
							case INTERSECTION_BUTTON:
								workWin->performIntersection( );
								break;
							case UNION_BUTTON:
								workWin->performUnion( );
								break;
							case DIFFERENCE_BUTTON:
								workWin->performDifference( );
								break;
							case MIN_DISTANCE_BUTTON:
								workWin->performMinDistance( );
								break;
							case CLEAR_OPERATION_BUTTON:
								workWin->performClearResult( );
								break;
							case AREA_OPERATION_BUTTON:
								workWin->performArea( );
								break;
							case LENGTH_OPERATION_BUTTON:
								workWin->performLength( );
								break;
							case CONVEX_HULL_BUTTON:
								workWin->performConvexHull( );
								break;
						
							case DELAUNAY_TRIANGULATION_BUTTON:
								workWin->performDelaunay( );
								break;

							case VORONOI_BUTTON:
								workWin->performVoronoi( );
								break;
						
						}
						
						//redisplay the scene
						//postRedisplay( );
						return;
					}
				}
				
				//check if a state button has been selected
				for( uint i = 0; i < stateButtons.size(); i++ )
				{
					if( stateButtons[i].clicked( plotX, plotY) )
					{
						stateButtons[i].select( !stateButtons[i].isSelected( ) );
						switch( stateButtons[i].getCode( ) )
						{
							case SHOW_MOUSE_COORDS_BUTTON:
								workWin->showCoords( stateButtons[i].isSelected( ) );
								break;
							case QUIT_BUTTON:
								workWin->clean( );
								exit( 0 );
								break;
						}
					}
				}
				
				// check if a file button has been selected
				//first, if a file button is already active, jsut return
				for( uint i = 0; i < fileButtons.size( ); i++ )
				{
					if( fileButtons[i].clicked( plotX, plotY ) )
					{
						if( fileButtons[i].isSelected( ) )
						{ return; }
					}
				}
				// now handle the button action
				for( uint i = 0; i < fileButtons.size( ); i++ )
				{
					if( fileButtons[i].clicked( plotX, plotY ) )
					{
						fileButtons[i].select( );
						switch( fileButtons[i].getCode( ) )
						{
							case LOAD_POINT_BUTTON:
								//cout<<"Enter primary key for point -+> ";
								//cin>> objectPrimaryKey;
								workWin->loadNewPoint( /*objectPrimaryKey*/ );
								break;
							case LOAD_LINE_BUTTON:
								//cout<<"Enter primary key for line -+> ";
								//cin>> objectPrimaryKey;
								workWin->loadNewLine( /*objectPrimaryKey*/ );
								break;
							case LOAD_REGION_BUTTON:
								//cout<<"Enter primary key for region -+> ";
								//cin>> objectPrimaryKey;
								workWin->loadNewRegion( /*objectPrimaryKey*/ );
								break;
						}
						fileButtons[i].select( false );
					}
				}
				
				//redisplay the motha'
				postRedisplay( );
			}
		}
		
		int convertXMouseCoordToScreenCoord( const int x )
		{
			int plotX = x;
			// remember the screen may be moved and or zoomed
			//translate from point coord (0,0 at top left)
			// to coordinate coords (0,0 at bottom left)
			// now scale to the resolution we are actually displaying
			plotX = static_cast<int>( ( plotX*(myGlXMax-myGlXMin) )/windowWidth );
			
			// now unzoom 
			plotX = static_cast<int>( plotX/zoomFactor );
			
			// now find shift by the amount shifted up and down
			plotX = static_cast<int>( myGlXMin + plotX ); 
			return plotX;
		}
		
		int convertYMouseCoordToScreenCoord( const int y )
		{
			int plotY = windowHeight - y;
			// remember the screen may be moved and or zoomed
			//translate from point coord (0,0 at top left)
			// to coordinate coords (0,0 at bottom left)
			// now scale to the resolution we are actually displaying
			plotY = static_cast<int>( ( plotY*(myGlYMax-myGlYMin) )/windowHeight );
			
			// now unzoom 
			plotY = static_cast<int>( plotY/zoomFactor );
			
			// now find shift by the amount shifted up and down
			plotY = static_cast<int>( myGlYMin + plotY );
			return plotY; 
		}
		
		
		
		void reshapeWindow( int w, int h )
		{
			glutSetWindow( winID );
			if( h == static_cast<int>(windowHeight) ) return;
			
			if( h != static_cast<int>(windowHeight) )
			{
				myGlYMin = myGlYMax - ( (initOrthoHeight * h) / initWindowHeight );
				windowHeight = h;
			}
			glutReshapeWindow (w, h);
			glViewport(0, 0, w, h);
			glutPostRedisplay( );
		}
		
		void postRedisplay( )
		{
			glutSetWindow( winID );
			glutPostRedisplay( );
		}
		
		int getID( )
		{ return winID; }
		int getWindowWidth( )
		{ return windowWidth; }
};


#endif
