#ifndef MYGLWINDOW_H
#define MYGLWINDOW_H

#include <gl.h>
#include <glut.h>
#include "myGlMenuSubWindow.h"
#include "myGlWorkspaceSubWindow.h"

class myGlWindow
{
	protected:
		int winID; //glut window id
		bool cleaned; //used to nicely break oracle connections before quitting
		
		const uint initWindowWidth; //initial actual window width
		const uint initWindowHeight; //initial actual window height
		
		uint windowWidth; //current actual window width
		uint windowHeight; //current actual window height
		
		const uint initWindowPositionX; //initial window position
		const uint initWindowPositionY;
		
		myGlMenuSubWindow *menuWindow;
		myGlWorkspaceSubWindow *workWindow;
		
	public:
		myGlWindow( uint initPosX, uint initPosY, uint winWidth, uint winHeight, int argc, char** argv, uint pid1, uint pid2): cleaned( false ), initWindowWidth( winWidth ), initWindowHeight( winHeight ), windowWidth(initWindowWidth ), windowHeight( initWindowHeight), initWindowPositionX( initPosX  ), initWindowPositionY( initPosY )
		{
			// initialize the glutness
			glutInit( &argc, argv );
			glutInitDisplayMode( GLUT_DOUBLE | GLUT_RGB );
			glMatrixMode( GL_PROJECTION );	
			glutInitWindowSize( windowWidth, windowHeight );
			glutInitWindowPosition( initWindowPositionX, initWindowPositionY );
			winID = glutCreateWindow( "Point, Points, pretty little points..." );
			setupWindow( );
			workWindow = new myGlWorkspaceSubWindow( winID, MENUWIDTH, 0, INIT_WORKSPACE_WIN_WIDTH, INIT_WIN_HEIGHT, INIT_ORTHO_X_MIN, INIT_ORTHO_X_MAX, INIT_ORTHO_Y_MIN, INIT_ORTHO_Y_MAX, 1, pid1, pid2 );
			menuWindow = new myGlMenuSubWindow( winID,  0, 0, MENUWIDTH, INIT_WIN_HEIGHT, INIT_ORTHO_X_MIN, INIT_ORTHO_X_MAX, INIT_ORTHO_Y_MIN, INIT_ORTHO_Y_MAX, INIT_ZOOM_FACTOR, workWindow );
		}
		
		~myGlWindow( )
		{
			clean( );	
		}
		
		void setupWindow( )
		{
			glutSetWindow( winID );
			glClearColor( 1.0, 1.0, 1.0, 1.0 );
			//glColor3f(0.0, 0.0, 1.0);
			glClear( GL_COLOR_BUFFER_BIT );
		}
		
		void display( )
		{
			setupWindow( );	
			workWindow->display();
			menuWindow->display();
			glutSwapBuffers();
		}
		
		void reshapeWindow( int w, int h )
		{
			int newWorkspaceWidth = w - menuWindow->getWindowWidth( );
			if( newWorkspaceWidth < 0 ) { newWorkspaceWidth *= -1; }
			int newWindowHeight = h;
			
			if( w == static_cast<int>(windowWidth) &&  h == static_cast<int>(windowHeight) ) return;
			
			if( w != static_cast<int>(windowWidth) )
			{
				workWindow->reshapeWindow( newWorkspaceWidth, newWindowHeight );
				workWindow->postRedisplay( );
				menuWindow->reshapeWindow( MENUWIDTH,  newWindowHeight );
				menuWindow->postRedisplay( );
			}
			if( h !=static_cast<int>(windowHeight) )
			{
				workWindow->reshapeWindow( newWorkspaceWidth, newWindowHeight );
				menuWindow->reshapeWindow( MENUWIDTH,  newWindowHeight );
				workWindow->postRedisplay( );
				menuWindow->postRedisplay( );
			}
			//glutReshapeWindow (w, h);
		}
		
		void clean( )
		{
			if( ! cleaned )
			{
				cout << "Cleaning Up Main Window!" << endl;
				cleaned = true;
				workWindow->clean( );
				delete workWindow;
				delete menuWindow;
			}
		}
		
		
		int getID( )
		{ return winID; }
		int getMenuWinID( )
		{ return menuWindow->getID( ); }
		int getWorkspaceWinID( )
		{ return workWindow->getID( ); }
		int getWindowWidth( )
		{ return windowWidth; }
		int getWindowHeight( )
		{ return windowHeight; }
		
		void workspaceDisplay(void)
		{
			workWindow->display();
		}
		
		void workspaceMouse( int button, int state, int x, int y )
		{
			workWindow->mouse( button, state, x, y );
		}
		
		void workspaceMouseEnterExit( int state )
		{
			workWindow->mouseEnterExit( state );
		}
		
		void workspacePassiveMouseMovement( int x, int y )
		{
			workWindow->passiveMouseMovement( x, y );
		}
		
		void workspaceActiveMouseMovement( int x, int y )
		{
			workWindow->activeMouseMovement( x, y );
		}
		
		void menuDisplay(void)
		{
			menuWindow->display();
		}
		
		void menuMouse( int button, int state, int x, int y )
		{
			menuWindow->mouse( button, state, x, y );
		}
		
		void menuMouseEnterExit( int state )
		{
			menuWindow->mouseEnterExit( state );
		}
};


#endif
