#ifndef MYGLWORKSPACESUBWINDOW_H
#define MYGLWORKSPACESUBWINDOW_H

#include <gl.h>
#include <glut.h>

#include <string>
#include <sstream>
//slob includes
#include <occi.h>
#include "constants.h"

#include "spatialObjectWrapper.h"
#include "pointWrapper.h"
#include "lineWrapper.h"
#include "regionWrapper.h"

using namespace stu;

class myGlWorkspaceSubWindow
{
	protected:
		int winID; //glut window id
		int parentWinID; // glut id of parent window
		//default viewable area of the plane
		int myGlXMin;
		int myGlXMax;
		int myGlYMin;
		int myGlYMax ;
		int myGlZMin;
		int myGlZMax;
		const uint initOrthoWidth; //initial ortho window width
		const uint initOrthoHeight; //initial ortho window height
		
		const uint initWindowWidth; //initial actual window width
		const uint initWindowHeight; //initial actual window height
		
		uint windowWidth; //current actual window width
		uint windowHeight; //current actual window height
		
		const uint initWindowPositionX; //initial window position
		const uint initWindowPositionY;
		
		float zoomFactor; //default zoom (no zooming)
		
		bool movingView; //indicate if we are moving the window view
		int moveViewX;
		int moveViewY;
		
		int mousePosX; //last recorded mouse poisition in the window
		int mousePosY; //last recorded mouse poisition in the window
		bool mouseInScreen; // true if mouse is over this window
		
		//flag to see if the points are cleaned up
		bool cleaned;
		
		// stuff needed for mslobiness
		// database connection info
		OCIEnv *envhp;
		OCIError *errhp;
		OCISession *authp;
		OCIServer *srvhp;
		OCISvcCtx *svchp;
		text *username;
		text *password;
		 // spatial objects
		spatialObjectWrapper* sObj1;
		spatialObjectWrapper* sObj2;
		spatialObjectWrapper* sObj3;
		
		// action to be performed on mouse click
		uint actionCode;
		// boolean tells which point to perform action on
		uint actionTarget;
		
		bool showMouseCoords;
		bool showMinDistance;
		rat minDistance;
		
		bool showLength;
		rat len;
		
		bool showArea;
		rat area;

	public:
		myGlWorkspaceSubWindow( int parentID, uint initPosX, uint initPosY, uint winWidth, uint winHeight,  int orthoMinX, int orthoMaxX, int orthoMinY, int orthoMaxY, float zoomFac, uint pid1 = 0, uint pid2 = 0 ): parentWinID( parentID ),  myGlXMin(orthoMinX), myGlXMax(orthoMaxX), myGlYMin(orthoMinY), myGlYMax(orthoMaxY), myGlZMin(-100), myGlZMax(100), initOrthoWidth( myGlXMax - myGlXMin ), initOrthoHeight( myGlYMax - myGlYMin ),  initWindowWidth( winWidth ), initWindowHeight( winHeight ), windowWidth(initWindowWidth ), windowHeight( initWindowHeight), initWindowPositionX( initPosX  ), initWindowPositionY( initPosY ), zoomFactor(zoomFac),  movingView( false ), moveViewX( 0 ), moveViewY( 0 ), mousePosX(0), mousePosY(0), mouseInScreen( false ),  cleaned( false ),  envhp( NULL ), errhp( NULL ), authp( (OCISession *) 0 ), srvhp( NULL ), svchp( NULL ), username( (text *) "apauly" ), password( (text *) "testis"),  sObj1( NULL ), sObj2( NULL ), sObj3( NULL ), actionCode( 0 ), actionTarget( 0 ), showMouseCoords( false ), showMinDistance( false ), minDistance( 0.0f )
		{
			winID = glutCreateSubWindow(parentWinID, initPosX, initPosY, winWidth, winHeight );
			glutSetWindow( winID );
			glMatrixMode(GL_PROJECTION);	
			
			// create the point mslobs
			initializeDbEnvironment( );
			if( pid1 != 0 )
				sObj1 = new regionWrapper( envhp, errhp, authp, srvhp, svchp, pid1 );
			if( pid2 != 0 )
				sObj2 = new regionWrapper( envhp, errhp, authp, srvhp, svchp, pid2 ); 
			setupWindow( );
		}
		
		void setSelectedTarget( uint target )
		{
			actionTarget = target;
			cout<<"target selected: "<<actionTarget<<endl;
		}
		
		void setActionCode( uint actCode )
		{
			actionCode = actCode;
			cout<<"action selected: "<<actionCode<<endl;
		}
		
		void setupWindow( )
		{
			glutSetWindow( winID );
			glLoadIdentity();
			glScalef( zoomFactor, zoomFactor, zoomFactor );
			glOrtho( myGlXMin, myGlXMax, myGlYMin, myGlYMax, myGlZMin, myGlZMax );
			glClearColor(1.0, 1.0, 1.0, 1.0);
			glClear(GL_COLOR_BUFFER_BIT);
		}
		
		int getID( )
		{ return winID; }
		
		void display(void)
		{
			//reset the scene, set the view size and zoom
			glLoadIdentity();
			glScalef( zoomFactor, zoomFactor, zoomFactor );
			glOrtho( myGlXMin, myGlXMax, myGlYMin, myGlYMax, myGlZMin, myGlZMax );
			glClear(GL_COLOR_BUFFER_BIT);
			glPointSize(8.0);
			
			if( sObj1 != NULL )
			{
				//draw the green object
				sObj1->display( showMouseCoords, COLOR_GREEN );
			}
			
			if( sObj2 != NULL )
			{
				//draw the blue points
				sObj2->display( showMouseCoords, COLOR_BLUE );
			}
			
			if( sObj3 != NULL )
			{
				//draw the red points
				sObj3->display( showMouseCoords, COLOR_RED );
			}
			
			//if need to display min distance
			if( showMinDistance )
			{
				glPushMatrix();                             // New frame of reference
				glLoadIdentity();
				glOrtho( myGlXMin, myGlXMax, myGlYMin, myGlYMax, myGlZMin, myGlZMax );
				glTranslatef(myGlXMax-20, myGlYMin+2.0, 0); // Move to new location
				glColor3f(0.0, 0.0, 0.0);
				string tmpOut;	
				ostringstream oss1;
				oss1<<"Min Dist: "<< minDistance.toDouble( );
				tmpOut.assign( oss1.str() );
				bitmap_output(0,0, const_cast<char*>(tmpOut.c_str()), GLUT_BITMAP_HELVETICA_10);
				glPopMatrix();                              // Return to old reference
			}
				
			if( showArea )
			{
				glPushMatrix();                             // New frame of reference
				glLoadIdentity();
				glOrtho( myGlXMin, myGlXMax, myGlYMin, myGlYMax, myGlZMin, myGlZMax );
				glTranslatef(myGlXMax-20, myGlYMin+2.0, 0); // Move to new location
				glColor3f(0.0, 0.0, 0.0);
				string tmpOut;	
				ostringstream oss1;
				oss1<<"Area: "<< area.toDouble( );
				tmpOut.assign( oss1.str() );
				bitmap_output(0,0, const_cast<char*>(tmpOut.c_str()), GLUT_BITMAP_HELVETICA_10);
				glPopMatrix();                              // Return to old reference
			}
			
			if( showLength )
			{
				glPushMatrix();                             // New frame of reference
				glLoadIdentity();
				glOrtho( myGlXMin, myGlXMax, myGlYMin, myGlYMax, myGlZMin, myGlZMax );
				glTranslatef(myGlXMax-20, myGlYMin+2.0, 0); // Move to new location
				glColor3f(0.0, 0.0, 0.0);
				string tmpOut;	
				ostringstream oss1;
				oss1<<"Length: "<< len.toDouble( );
				tmpOut.assign( oss1.str() );
				bitmap_output(0,0, const_cast<char*>(tmpOut.c_str()), GLUT_BITMAP_HELVETICA_10);
				glPopMatrix();                              // Return to old reference
			}
			
			glPushMatrix();                             // New frame of reference
			glLoadIdentity();
			glOrtho( myGlXMin, myGlXMax, myGlYMin, myGlYMax, myGlZMin, myGlZMax );
			glTranslatef(myGlXMax-10, myGlYMax-2.0, 0); // Move to new location
			glColor3f(0.0, 0.0, 0.0);
			string tmpOut;	
			ostringstream oss1;
			oss1<<'('<< mousePosX<<','<<mousePosY<<')';
			tmpOut.assign( oss1.str() );
			bitmap_output(0,0, const_cast<char*>(tmpOut.c_str()), GLUT_BITMAP_HELVETICA_10);
			glPopMatrix();                              // Return to old reference
		
			// put it on the screen
			glutSwapBuffers();
		}
		
		//draws text on the screen
		void bitmap_output(int x, int y, char *string, void *font)
		{  
			int len, i;
			glRasterPos3f(x, y, 0);            // Locate Raster Position in 3-space
			len = (int) strlen(string);        // Find length of string
			for (i = 0; i < len; i++)
			{
				// Loop through plotting all characters in font style
				glutBitmapCharacter(font, string[i]);
			}
		}
		
		void performValidation( )
		{
			spatialObjectWrapper* sObjtemp=NULL;
			
			if( actionTarget == SELECT_GREEN_BUTTON )
			{ sObjtemp = sObj1;}
			else
			if( actionTarget == SELECT_BLUE_BUTTON )
			{ sObjtemp = sObj2;}
			
			if( sObjtemp != NULL )
			{
				bool valid=false;
				if( sObjtemp->isPoint( ) )
				{
					pointWrapper *pw1 = static_cast<pointWrapper*>( sObjtemp );
					valid = pw1->getPtr()->validate( );
					//postRedisplay( );
				}
				else if( sObjtemp->isLine( ) )
				{
					lineWrapper *lw1 = static_cast<lineWrapper *>( sObjtemp );
					valid = lw1->getPtr()->validate( );
					//postRedisplay( );
				}
				else if( sObjtemp->isRegion( ) )
				{
					regionWrapper *rw1 = static_cast<regionWrapper *>( sObjtemp );
					valid = rw1->getPtr()->validate( );
					//postRedisplay( );
				}
				if (valid)
					cerr << "Validated Succesfully!" << endl;
				else
					cerr << "Validation Failed! " << endl;
			}
		}
		
		void performIntersection( )
		{
			if( sObj1 != NULL && sObj2 != NULL )
			{
				if( sObj1->isPoint( ) && sObj2->isPoint( ) )
				{
					//set up the point wrappers
					pointWrapper *pw1 = static_cast<pointWrapper*>( sObj1 );
					pointWrapper *pw2 = static_cast<pointWrapper*>( sObj2 );
					point2D *newPoint = new point2D( );
					*newPoint = pw1->getPtr()->intersection( *(pw2->getPtr()) );
					// calculate the point intersection
					sObj3 = new pointWrapper( newPoint );
					postRedisplay( );
				}
				else if( sObj1->isLine( ) && sObj2->isLine( ) )
				{
					//set up the point wrappers
					lineWrapper *lw1 = static_cast<lineWrapper *>( sObj1 );
					lineWrapper *lw2 = static_cast<lineWrapper *>( sObj2 );
					line2D *newLine = new line2D( );
					*newLine = lw1->getPtr()->intersection( *(lw2->getPtr()) );
					// calculate the point intersection
					sObj3 = new lineWrapper( newLine );
					postRedisplay( );
				}
				else if( sObj1->isRegion( ) && sObj2->isRegion( ) )
				{
					//set up the point wrappers
					regionWrapper *rw1 = static_cast<regionWrapper *>( sObj1 );
					regionWrapper *rw2 = static_cast<regionWrapper *>( sObj2 );
					region2D *newRegion = new region2D( );
					*newRegion = rw1->getPtr()->intersection( *(rw2->getPtr()) );
					cout << "New Region: " << *newRegion << endl;
					// calculate the point intersection
					sObj3 = new regionWrapper( newRegion );
					postRedisplay( );
				}
			}
		}
		
		void performUnion( )
		{
			if( sObj1 != NULL && sObj2 != NULL )
			{
				if( sObj1->isPoint( ) && sObj2->isPoint( ) )
				{
					//set up the point wrappers
					pointWrapper *pw1 = static_cast<pointWrapper*>( sObj1 );
					pointWrapper *pw2 = static_cast<pointWrapper*>( sObj2 );
					point2D *newPoint = new point2D( );
					*newPoint = pw1->getPtr()->sunion( *(pw2->getPtr()) );
					// calculate the point intersection
					sObj3 = new pointWrapper( newPoint );
					postRedisplay( );
				}
				else if( sObj1->isLine( ) && sObj2->isLine( ) )
				{
					//set up the point wrappers
					lineWrapper *lw1 = static_cast<lineWrapper *>( sObj1 );
					lineWrapper *lw2 = static_cast<lineWrapper *>( sObj2 );
					line2D *newLine = new line2D( );
					*newLine = lw1->getPtr()->sunion( *(lw2->getPtr()) );
					// calculate the point intersection
					sObj3 = new lineWrapper( newLine );
					postRedisplay( );
				}
				else if( sObj1->isRegion( ) && sObj2->isRegion( ) )
				{
					//set up the point wrappers
					regionWrapper *rw1 = static_cast<regionWrapper *>( sObj1 );
					regionWrapper *rw2 = static_cast<regionWrapper *>( sObj2 );
					region2D *newRegion = new region2D( );
					*newRegion = rw1->getPtr()->sunion( *(rw2->getPtr()) );
					// calculate the point intersection
					sObj3 = new regionWrapper( newRegion );
					postRedisplay( );
				}
			}
		}
		void performDifference( )
		{
			if( sObj1 != NULL && sObj2 != NULL )
			{
				if( sObj1->isPoint( ) && sObj2->isPoint( ) )
				{
					//set up the point wrappers
					pointWrapper *pw1 = static_cast<pointWrapper*>( sObj1 );
					pointWrapper *pw2 = static_cast<pointWrapper*>( sObj2 );
					point2D *newPoint = new point2D( );
					*newPoint = pw1->getPtr()->difference( *(pw2->getPtr()) );
					// calculate the point intersection
					sObj3 = new pointWrapper( newPoint );
					postRedisplay( );
				}
				else if( sObj1->isLine( ) && sObj2->isLine( ) )
				{
					//set up the point wrappers
					lineWrapper *lw1 = static_cast<lineWrapper *>( sObj1 );
					lineWrapper *lw2 = static_cast<lineWrapper *>( sObj2 );
					line2D *newLine = new line2D( );
					*newLine = lw1->getPtr()->difference( *(lw2->getPtr()) );
					// calculate the point intersection
					sObj3 = new lineWrapper( newLine );
					postRedisplay( );
				}
				else if( sObj1->isRegion( ) && sObj2->isRegion( ) )
				{
					//set up the point wrappers
					regionWrapper *rw1 = static_cast<regionWrapper *>( sObj1 );
					regionWrapper *rw2 = static_cast<regionWrapper *>( sObj2 );
					region2D *newRegion = new region2D( );
					*newRegion = rw1->getPtr()->difference( *(rw2->getPtr()) );
					// calculate the point intersection
					sObj3 = new regionWrapper( newRegion );
					postRedisplay( );
				}
			}
		}
		
		void performMinDistance( )
		{
			showArea=false;
			showLength=false;
			if( sObj1 != NULL && sObj2 != NULL )
			{
				if( sObj1->isPoint( ) && sObj2->isPoint( ) )
				{
					//set up the point wrappers
					pointWrapper *pw1 = static_cast<pointWrapper*>( sObj1 );
					pointWrapper *pw2 = static_cast<pointWrapper*>( sObj2 );
					minDistance = pw1->getPtr()->distance( *(pw2->getPtr()) );
					showMinDistance = true;	
					postRedisplay( );
				}
				else if( sObj1->isLine( ) && sObj2->isLine( ) )
				{
					//set up the point wrappers
					lineWrapper *lw1 = static_cast<lineWrapper *>( sObj1 );
					lineWrapper *lw2 = static_cast<lineWrapper *>( sObj2 );
					minDistance = lw1->getPtr()->distance( *(lw2->getPtr()) );
					showMinDistance = true;	
					postRedisplay( );
				}
				else if( sObj1->isRegion( ) && sObj2->isRegion( ) )
				{
					//set up the point wrappers
					regionWrapper *rw1 = static_cast<regionWrapper *>( sObj1 );
					regionWrapper *rw2 = static_cast<regionWrapper *>( sObj2 );
					minDistance = rw1->getPtr()->distance( *(rw2->getPtr()) );
					// calculate the point intersection
					showMinDistance = true;	
					postRedisplay( );
				}
			}
		}
		
		void performArea( )
		{
			showMinDistance=false;
			showLength=false;
			
			spatialObjectWrapper* sObjtemp=NULL;
			
			if( actionTarget == SELECT_GREEN_BUTTON )
			{ sObjtemp = sObj1;}
			else
				if( actionTarget == SELECT_BLUE_BUTTON )
			{ sObjtemp = sObj2;}
			
			if( sObjtemp != NULL )
			{
				if( sObjtemp->isPoint( ) )
				{
				}
				else if( sObjtemp->isLine( ) )
				{
				}
				else if( sObjtemp->isRegion( ) )
				{
					regionWrapper *rw1 = static_cast<regionWrapper *>( sObjtemp );
					area = rw1->getPtr()->getArea( );
					showArea = true;	
					postRedisplay( );
				}
			}
		}


		void performConvexHull( )
		{
			
			spatialObjectWrapper* sObjtemp=NULL;
			
			if( actionTarget == SELECT_GREEN_BUTTON )
			{ sObjtemp = sObj1;}
			else
				if( actionTarget == SELECT_BLUE_BUTTON )
			{ sObjtemp = sObj2;}
			
			if( sObjtemp != NULL )
			{
				if( sObjtemp->isPoint( ) )
				{

					pointWrapper *rw1 = static_cast<pointWrapper *>( sObjtemp );
					region2D * newRegion = new region2D();
					*newRegion = rw1->getPtr()->convexHull();
					sObj3 = new regionWrapper(newRegion);
					postRedisplay( );

				}
				else if( sObjtemp->isLine( ) )
				{
					lineWrapper *rw1 = static_cast<lineWrapper *>( sObjtemp );
					region2D * newRegion = new region2D();
					*newRegion = rw1->getPtr()->convexHull();
					sObj3 = new regionWrapper(newRegion);
					postRedisplay( );
				}
				else if( sObjtemp->isRegion( ) )
				{
					regionWrapper *rw1 = static_cast<regionWrapper *>( sObjtemp );
					region2D * newRegion = new region2D();
					*newRegion = rw1->getPtr()->convexHull();
					sObj3 = new regionWrapper(newRegion);
					postRedisplay( );
				}
			}
		}


		void performDelaunay( )
		{
			spatialObjectWrapper* sObjtemp=NULL;
			
			if( actionTarget == SELECT_GREEN_BUTTON )
			{ sObjtemp = sObj1;}
			else
				if( actionTarget == SELECT_BLUE_BUTTON )
			{ sObjtemp = sObj2;}
			
			if( sObjtemp != NULL )
			{
				if( sObjtemp->isPoint( ) )
				{


					pointWrapper *rw1 = static_cast<pointWrapper *>( sObjtemp );
					line2D * newlines = new line2D();
					*newlines = rw1->getPtr()->delaunayTriangulation();
					sObj3 = new lineWrapper(newlines);
					postRedisplay( );

				}
				else if( sObjtemp->isLine( ) )
				{
					lineWrapper *rw1 = static_cast<lineWrapper *>( sObjtemp );
					line2D * newlines = new line2D();
					*newlines = rw1->getPtr()->delaunayTriangulation();
					sObj3 = new lineWrapper(newlines);
					postRedisplay( );

				}
				else if( sObjtemp->isRegion( ) )
				{
					regionWrapper *rw1 = static_cast<regionWrapper *>( sObjtemp );
					line2D * newlines = new line2D();
					*newlines = rw1->getPtr()->delaunayTriangulation();
					sObj3 = new lineWrapper(newlines);
					postRedisplay( );
				}
			}
		}



		void performVoronoi( )
		{
			spatialObjectWrapper* sObjtemp=NULL;
			
			if( actionTarget == SELECT_GREEN_BUTTON )
			{ sObjtemp = sObj1;}
			else
				if( actionTarget == SELECT_BLUE_BUTTON )
			{ sObjtemp = sObj2;}
			
			if( sObjtemp != NULL )
			{
				if( sObjtemp->isPoint( ) )
				{


					pointWrapper *rw1 = static_cast<pointWrapper *>( sObjtemp );
					line2D * newlines = new line2D();
					*newlines = rw1->getPtr()->voronoi();
					sObj3 = new lineWrapper(newlines);
					postRedisplay( );

				}
				else if( sObjtemp->isLine( ) )
				{
					lineWrapper *rw1 = static_cast<lineWrapper *>( sObjtemp );
					line2D * newlines = new line2D();
					*newlines = rw1->getPtr()->voronoi();
					sObj3 = new lineWrapper(newlines);
					postRedisplay( );

				}
				else if( sObjtemp->isRegion( ) )
				{
					regionWrapper *rw1 = static_cast<regionWrapper *>( sObjtemp );
					line2D * newlines = new line2D();
					*newlines = rw1->getPtr()->voronoi();
					sObj3 = new lineWrapper(newlines);
					postRedisplay( );
				}
			}
		}



		void performLength( )
		{
			showArea=false;
			showMinDistance=false;
			
			spatialObjectWrapper* sObjtemp=NULL;
			
			if( actionTarget == SELECT_GREEN_BUTTON )
			{ sObjtemp = sObj1;}
			else
				if( actionTarget == SELECT_BLUE_BUTTON )
			{ sObjtemp = sObj2;}
			
			if( sObjtemp != NULL )
			{
				if( sObjtemp->isPoint( ) )
				{
				}
				else if( sObjtemp->isLine( ) )
				{
					lineWrapper *lw1 = static_cast<lineWrapper *>( sObjtemp );
					len = lw1->getPtr()->getLength( );
					showLength = true;	
					postRedisplay( );
				}
				else if( sObjtemp->isRegion( ) )
				{
					regionWrapper *rw1 = static_cast<regionWrapper *>( sObjtemp );
					len = rw1->getPtr()->getLength( );
					showLength = true;	
					postRedisplay( );
				}
			}
		}
		
		void performClearResult( )
		{
			if( sObj3 != NULL )
			{
				delete sObj3;
				sObj3 = NULL;
			}
			showMinDistance = false;
			showArea = false;
			showLength = false;
			postRedisplay( );
		}
		
		void loadNewPoint(  )
		{
			if( actionTarget == SELECT_GREEN_BUTTON )
			{
				if( sObj1 != NULL ){ delete sObj1; }
				sObj1 = NULL;
				sObj1 = new pointWrapper( envhp, errhp, authp, srvhp, svchp, false );
			}
			else if(  actionTarget == SELECT_BLUE_BUTTON )
			{
				if( sObj2 != NULL ){ delete sObj2; }
				sObj2 = NULL;
				sObj2 = new pointWrapper( envhp, errhp, authp, srvhp, svchp, false );
			}
			postRedisplay( );
		}
		
		void loadNewLine(  )
		{
			if( actionTarget == SELECT_GREEN_BUTTON )
			{
				if( sObj1 != NULL ){ delete sObj1; }
				sObj1 = NULL;
				sObj1 = new lineWrapper( envhp, errhp, authp, srvhp, svchp, false );
			}
			else if(  actionTarget == SELECT_BLUE_BUTTON )
			{
				if( sObj2 != NULL ){ delete sObj2; }
				sObj2 = NULL;
				sObj2 = new lineWrapper( envhp, errhp, authp, srvhp, svchp, false );
			}
			postRedisplay( );
		}
		
		void loadNewRegion(  )
		{
			if( actionTarget == SELECT_GREEN_BUTTON )
			{
				if( sObj1 != NULL ){ delete sObj1; }
				sObj1 = NULL;
				sObj1 = new regionWrapper( envhp, errhp, authp, srvhp, svchp, false );
			}
			else if(  actionTarget == SELECT_BLUE_BUTTON )
			{
				if( sObj2 != NULL ){ delete sObj2; }
				sObj2 = NULL;
				sObj2 = new regionWrapper( envhp, errhp, authp, srvhp, svchp, false );
			}
			postRedisplay( );
		}
		
		void commitChanges( )
		{
			if( actionTarget == SELECT_GREEN_BUTTON )
				sObj1->commitChanges( );
			else if(  actionTarget == SELECT_BLUE_BUTTON )
				sObj2->commitChanges( );
			// call the appropriate commit changes function in the wrapper
			postRedisplay( );
		}
		
		void postRedisplay( )
		{
			glutSetWindow( winID );
			glutPostRedisplay( );
		}
		
		void reshapeWindow( int w, int h )
		{
			glutSetWindow( winID );
			if( static_cast<uint>(w) == windowWidth &&  static_cast<uint>(h) == windowHeight ) return;
			
			if( static_cast<uint>(w) != windowWidth )
			{
				myGlXMax = myGlXMin + ( (initOrthoWidth * w) / initWindowWidth );
				windowWidth = w;
			}
			if( static_cast<uint>(h) != windowHeight )
			{
				myGlYMax = myGlYMin + ( (initOrthoHeight * h) / initWindowHeight );
				windowHeight = h;
			}
			glutReshapeWindow (w, h);
			glViewport(0, 0, w, h);
			postRedisplay( );
		}
		
		
		void mouseEnterExit( int state )
		{
			if (state == GLUT_LEFT)
				mouseInScreen = false;
			else
				mouseInScreen = true;
		}
		
		
		void passiveMouseMovement( int x, int y )
		{
			mousePosX = convertXMouseCoordToScreenCoord( x );
			mousePosY = convertYMouseCoordToScreenCoord( y );
			
			if( sObj1 != NULL )
				sObj1->updateNewEndpoint( mousePosX, mousePosY );
			if( sObj2 != NULL )
				sObj2->updateNewEndpoint( mousePosX, mousePosY );
			postRedisplay( );
		}
		
		void activeMouseMovement( int x, int y )
		{
			if( movingView )
			{
				int plotX = convertXMouseCoordToScreenCoord( x );
				int plotY = convertYMouseCoordToScreenCoord( y );
				myGlXMax += moveViewX - plotX;
				myGlXMin += moveViewX - plotX;
				myGlYMax += moveViewY - plotY;
				myGlYMin += moveViewY - plotY;
				postRedisplay( );
			}
		}
		void mouse( int button, int state, int x, int y )
		{
			// if mouse is not in the screen, then ignore the clicks
			if( ! mouseInScreen )
				return;
			
			// if the object we are trying to edit does not exist,
			// then quit these shennanigans
			if( actionTarget == SELECT_GREEN_BUTTON && sObj1 == NULL )
			{
				return;
			}
			else if( actionTarget == SELECT_BLUE_BUTTON && sObj2 == NULL )
			{
				return;
			}
			
			// otherwise, update like its 1999
			
			//first, translate the mouse coordinates to plotting coordinates 
			int plotX = convertXMouseCoordToScreenCoord( x );
			int plotY = convertYMouseCoordToScreenCoord( y );

			//we have now found the coords of the point... like its nobody's business
			// now perform the desired action
			
			if( actionCode == MOVE_VIEW_BUTTON )
			{
				if( button == GLUT_LEFT_BUTTON && state == GLUT_DOWN )
				{
					movingView = true;
					moveViewX = plotX;
					moveViewY = plotY;
					return;
				}
			} else
			{
				movingView = false;
			}
			
			if( button == GLUT_RIGHT_BUTTON && state == GLUT_DOWN )
			{
				if( actionTarget == SELECT_GREEN_BUTTON )
					sObj1->commitChanges( );
				else if( actionTarget == SELECT_BLUE_BUTTON )
					sObj2->commitChanges( );
			}
			else if( button == GLUT_LEFT_BUTTON && state == GLUT_DOWN && ( actionTarget == SELECT_GREEN_BUTTON || actionTarget == SELECT_BLUE_BUTTON ))
			{
				switch( actionCode )
				{
					case INSERT_POINT_BUTTON:
						
						if( actionTarget == SELECT_GREEN_BUTTON && sObj1->isPoint( ) )
						{
							pointWrapper *tmpWrapperPtr = static_cast<pointWrapper*>(sObj1);
							tmpWrapperPtr->insertPoint( plotX, plotY );
						}
						else if( actionTarget == SELECT_BLUE_BUTTON && sObj2->isPoint( ) )
						{
							pointWrapper *tmpWrapperPtr = static_cast<pointWrapper*>(sObj2);
							tmpWrapperPtr->insertPoint( plotX, plotY );
						}
						break;
					case REMOVE_POINT_BUTTON:
						if( actionTarget == SELECT_GREEN_BUTTON && sObj1->isPoint( ) )
						{
							pointWrapper *tmpWrapperPtr = static_cast<pointWrapper*>(sObj1);
							tmpWrapperPtr->removePoint( plotX, plotY );
						}
						else if( actionTarget == SELECT_BLUE_BUTTON && sObj2->isPoint( ) )
						{
							pointWrapper *tmpWrapperPtr = static_cast<pointWrapper*>(sObj2);
							tmpWrapperPtr->removePoint( plotX, plotY );
						}
						break;
					case INSERT_BLOCK_BUTTON:
						if( actionTarget == SELECT_GREEN_BUTTON && sObj1->isLine( ) )
						{
							lineWrapper *tmpWrapperPtr = static_cast<lineWrapper*>(sObj1);
							tmpWrapperPtr->insertBlockHSeg( plotX, plotY );
						}
						else if( actionTarget == SELECT_BLUE_BUTTON && sObj2->isLine( ) )
						{
							lineWrapper *tmpWrapperPtr = static_cast<lineWrapper*>(sObj2);
							tmpWrapperPtr->insertBlockHSeg( plotX, plotY );
						}
						break;
					case REMOVE_BLOCK_BUTTON:
						if( actionTarget == SELECT_GREEN_BUTTON && sObj1->isLine( ) )
						{
							lineWrapper *tmpWrapperPtr = static_cast<lineWrapper*>(sObj1);
							tmpWrapperPtr->removeBlock( plotX, plotY );
						}
						else if( actionTarget == SELECT_BLUE_BUTTON && sObj2->isLine( ) )
						{
							lineWrapper *tmpWrapperPtr = static_cast<lineWrapper*>(sObj2);
							tmpWrapperPtr->removeBlock( plotX, plotY );
						}
						break;
					case INSERT_FACE_BUTTON:
						if( actionTarget == SELECT_GREEN_BUTTON && sObj1->isRegion( ) )
						{
							regionWrapper *tmpWrapperPtr = static_cast<regionWrapper*>(sObj1);
							tmpWrapperPtr->insertFaceSeg( plotX, plotY );
						}
						else if( actionTarget == SELECT_BLUE_BUTTON && sObj2->isRegion( ) )
						{
							regionWrapper *tmpWrapperPtr = static_cast<regionWrapper*>(sObj2);
							tmpWrapperPtr->insertFaceSeg( plotX, plotY );
						}
						break;
					case REMOVE_FACE_BUTTON:
						if( actionTarget == SELECT_GREEN_BUTTON && sObj1->isRegion( ) )
						{
							regionWrapper *tmpWrapperPtr = static_cast<regionWrapper*>(sObj1);
							tmpWrapperPtr->removeFace( plotX, plotY );
						}
						else if( actionTarget == SELECT_BLUE_BUTTON && sObj2->isRegion( ) )
						{
							regionWrapper *tmpWrapperPtr = static_cast<regionWrapper*>(sObj2);
							tmpWrapperPtr->removeFace( plotX, plotY );
						}
						break;
					case INSERT_HOLE_BUTTON:
						if( actionTarget == SELECT_GREEN_BUTTON && sObj1->isRegion( ) )
						{
							regionWrapper *tmpWrapperPtr = static_cast<regionWrapper*>(sObj1);
							tmpWrapperPtr->insertHoleSeg( plotX, plotY );
						}
						else if( actionTarget == SELECT_BLUE_BUTTON && sObj2->isRegion( ) )
						{
							regionWrapper *tmpWrapperPtr = static_cast<regionWrapper*>(sObj2);
							tmpWrapperPtr->insertHoleSeg( plotX, plotY );
						}
						break;
					case REMOVE_HOLE_BUTTON:
						if( actionTarget == SELECT_GREEN_BUTTON && sObj1->isRegion( ) )
						{
							regionWrapper *tmpWrapperPtr = static_cast<regionWrapper*>(sObj1);
							tmpWrapperPtr->removeHole( plotX, plotY );
						}
						else if( actionTarget == SELECT_BLUE_BUTTON && sObj2->isRegion( ) )
						{
							regionWrapper *tmpWrapperPtr = static_cast<regionWrapper*>(sObj2);
							tmpWrapperPtr->removeHole( plotX, plotY );
						}
						break;
				}
			}
			postRedisplay();
		}
		
		void increaseZoom( )
		{
			zoomFactor += 0.1;
			if( zoomFactor <= 0 ) zoomFactor = 0.1;
			postRedisplay( );
		}
		
		void decreaseZoom( )
		{
			zoomFactor -= 0.1;
			if( zoomFactor <= 0 ) zoomFactor = 0.1;
			postRedisplay( );
		}
		
		void showCoords( bool state )
		{
			showMouseCoords = state;
			postRedisplay( );
		}
		
		int convertXMouseCoordToScreenCoord( const int x )
		{
			double plotX = x;
			
			double visX =0;
			int zoomShift = 0;
			// remember the screen may be moved and or zoomed
			//translate from point coord (0,0 at top left)
			// to coordinate coords (0,0 at bottom left)
			
			//find the actual visible are after zooming:
			visX = (myGlXMax-myGlXMin)/zoomFactor;
			// now shift the min x over because we zoomed in on a specific area
			visX = ((myGlXMax-myGlXMin) - visX)/2;
			//now round visX to nearest integer
			zoomShift = static_cast<int>( visX +0.5f );
			
			
			// now scale to the resolution we are actually displaying
			plotX = ( plotX*(myGlXMax-myGlXMin) )/windowWidth ;
			
			// now zoom 
			plotX = plotX/zoomFactor;
			
			// now find shift by the amount shifted up and down
			plotX =  myGlXMin + plotX ; 
			//now round plotx, shift and return
			return( static_cast<int>(plotX+0.5f) + zoomShift );
		}
		
		int convertYMouseCoordToScreenCoord( const int y )
		{
			double plotY = windowHeight - y;
			double visY =0;
			int zoomShift = 0;
			// remember the screen may be moved and or zoomed
			//translate from point coord (0,0 at top left)
			// to coordinate coords (0,0 at bottom left)
			
			//find the actual visible are after zooming:
			visY =  (myGlYMax-myGlYMin)/zoomFactor ;
			// now shift the min x over because we zoomed in on a specific area
			visY = ((myGlYMax-myGlYMin) - visY)/2;
			//now round visY to nearest integer
			zoomShift = static_cast<int>( visY +0.5f );
			// now scale to the resolution we are actually displaying
			plotY =( plotY*(myGlYMax-myGlYMin) )/windowHeight;
			
			// now unzoom 
			plotY =plotY/zoomFactor ;
			
			// now find shift by the amount shifted up and down
			plotY = myGlYMin + plotY;
			//now round plotx, shift and return
			return( static_cast<int>(plotY+0.5f) + zoomShift );
		}
		
		~myGlWorkspaceSubWindow( )
		{
			clean( );
		}
		
		void clean( )
		{
			if( ! cleaned )
			{
				cleaned = true;
				cout<<"Cleaning Workspace!"<<endl;
				
				if( sObj1 != NULL )
				{	delete sObj1; sObj1 = NULL; }
				if( sObj2 != NULL ) 
				{	delete sObj2; sObj2 = NULL; }
				if( sObj3 != NULL ) 
				{	delete sObj3; sObj3 = NULL; }
				
				checkerr(errhp, OCITransCommit(svchp, errhp, (ub4)0 ));
				if (srvhp)
					(void) OCIHandleFree((dvoid *) errhp, OCI_HTYPE_SERVER);
				if (svchp)
					(void) OCIHandleFree((dvoid *) errhp, OCI_HTYPE_SVCCTX);
				if (errhp)
					(void) OCIHandleFree((dvoid *) errhp, OCI_HTYPE_ERROR);
				if (envhp)
					(void) OCIHandleFree((dvoid *) envhp, OCI_HTYPE_ENV);
			}
		}
		
		void initializeDbEnvironment(  )
		{
			try {
				
				(void) OCIInitialize((ub4) OCI_DEFAULT, (dvoid *)0,
				(dvoid * (*)(dvoid *, size_t)) 0,
				(dvoid * (*)(dvoid *, dvoid *, size_t))0,
				(void (*)(dvoid *, dvoid *)) 0 );
	
				(void) OCIEnvInit( (OCIEnv **) &envhp, OCI_DEFAULT, (size_t) 0,
				(dvoid **) 0 );
	
				(void) OCIHandleAlloc( (dvoid *) envhp, (dvoid **) &errhp, OCI_HTYPE_ERROR,
				(size_t) 0, (dvoid **) 0);
	
				/* server contexts */
				(void) OCIHandleAlloc( (dvoid *) envhp, (dvoid **) &srvhp, OCI_HTYPE_SERVER,
				(size_t) 0, (dvoid **) 0);
	
				(void) OCIHandleAlloc( (dvoid *) envhp, (dvoid **) &svchp, OCI_HTYPE_SVCCTX,
				(size_t) 0, (dvoid **) 0);
	
				(void) OCIServerAttach( srvhp, errhp, (text *)"berlin.dbcenter.cise.ufl.edu:1521/orastu1", strlen("berlin.dbcenter.cise.ufl.edu:1521/orastu1"), 0);
	
				/* set attribute server context in the service context */
				(void) OCIAttrSet( (dvoid *) svchp, OCI_HTYPE_SVCCTX, (dvoid *)srvhp,
				(ub4) 0, OCI_ATTR_SERVER, (OCIError *) errhp);
	
				(void) OCIHandleAlloc((dvoid *) envhp, (dvoid **)&authp,
				(ub4) OCI_HTYPE_SESSION, (size_t) 0, (dvoid **) 0);
	
				(void) OCIAttrSet((dvoid *) authp, (ub4) OCI_HTYPE_SESSION,
				(dvoid *) username, (ub4) strlen((char *)username),
				(ub4) OCI_ATTR_USERNAME, errhp);
	
				(void) OCIAttrSet((dvoid *) authp, (ub4) OCI_HTYPE_SESSION,
				(dvoid *) password, (ub4) strlen((char *)password),
				(ub4) OCI_ATTR_PASSWORD, errhp);
	
		//connect to server and being session
				checkerr(errhp, OCISessionBegin ( svchp,  errhp, authp, OCI_CRED_RDBMS,
						(ub4) OCI_DEFAULT));
	
				(void) OCIAttrSet((dvoid *) svchp, (ub4) OCI_HTYPE_SVCCTX,
				(dvoid *) authp, (ub4) 0,
				(ub4) OCI_ATTR_SESSION, errhp);
			
				cout << "\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\bConnection created succesfully!" << endl;
				
			}catch (exception &excp)
			{
				cerr << excp.what() << endl;
				cout << "Thanks...sorry we ended this on such a bad note." << endl;
				cout << "Closing connection" << endl;
	
				if (srvhp)
					(void) OCIHandleFree((dvoid *) errhp, OCI_HTYPE_SERVER);
				if (svchp)
					(void) OCIHandleFree((dvoid *) errhp, OCI_HTYPE_SVCCTX);
				if (errhp)
					(void) OCIHandleFree((dvoid *) errhp, OCI_HTYPE_ERROR);
				if (envhp)
					(void) OCIHandleFree((dvoid *) envhp, OCI_HTYPE_ENV);
			}
			/*****  Old DB connect code
			try{
					if( env == NULL )
					{
						env = Environment::createEnvironment (
								Environment::DEFAULT);
						cout << "creating connection...";
					}
					if( conn == NULL )
					{
						conn = env->createConnection ("apauly","testis", "berlin.dbcenter.cise.ufl.edu:1521/orastu1");
			
						cout << "\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\bConnection created succesfully!" << endl;
					}
			}
			catch (exception &excp)
			{
				cout << "Thanks...sorry we ended this on such a bad note." << endl;
				cout << "Closing connection" << endl;
				env->terminateConnection (conn);
				cerr << "hola "<<excp.what() << endl;
			}*/
			
		}
};


#endif
