#ifndef POINTEDITOR_H
#define POINTEDITOR_H

#include <iostream>
#include <gl.h>
#include <glut.h>

#include "myGlWindow.h"
#include "myGlWorkspaceSubWindow.h"
#include "myGlMenuSubWindow.h"

using namespace stu;

void mainDisplayWrapper(void);
void mainReshapeWindowWrapper( int, int );
void workspaceDisplayWrapper(void);
void workspaceMouseWrapper( int button, int state, int x, int y );
void workspaceMouseEnterExitWrapper( int state );
void workspacePassiveMouseMovementWrapper( int x, int y );
void workspaceActiveMouseMovementWrapper( int x, int y );
void menuDisplayWrapper(void);
void menuMouseWrapper( int button, int state, int x, int y );
void menuMouseEnterExitWrapper( int state );
void cleanup( void );
/*
void keyboardWrapper(unsigned char key, int x, int y);
void mouseWrapper( int button, int state, int x, int y );
void mouseEnterExitWrapper( int state );
void passiveMouseMovementWrapper( int x, int y );*/


#include "constants.h"

class pointEditor;
pointEditor *objectToCall;


class pointEditor
{
	private:
		
		myGlWindow mainWin;
		bool cleaned;
	
	public:
		pointEditor( int argc, char ** argv, uint pid1 = 0, uint pid2 = 0 ): 
			mainWin( INIT_WIN_X_POS, INIT_WIN_Y_POS, INIT_WORKSPACE_WIN_WIDTH + MENUWIDTH, INIT_WIN_HEIGHT, argc, argv, pid1, pid2 ), cleaned( false )
		{
			
			// pointer for callback functions to use
			objectToCall = this;
			
			//setup all the callback functions
			glutSetWindow( mainWin.getWorkspaceWinID( ) );
			glutDisplayFunc(workspaceDisplayWrapper);
			glutMouseFunc(workspaceMouseWrapper);
			glutEntryFunc(workspaceMouseEnterExitWrapper);
			glutPassiveMotionFunc(workspacePassiveMouseMovementWrapper);
			glutMotionFunc(workspaceActiveMouseMovementWrapper);
			
			glutSetWindow( mainWin.getMenuWinID( ) );
			glutDisplayFunc( menuDisplayWrapper);
			glutMouseFunc( menuMouseWrapper );
			glutEntryFunc( menuMouseEnterExitWrapper );
			
			glutSetWindow( mainWin.getID( ) );
			glutDisplayFunc(mainDisplayWrapper);
			glutReshapeFunc (mainReshapeWindowWrapper);
			
			//callback for the exit function
			atexit( cleanup );
		}
		
		~pointEditor( )
		{
			clean( );
		}
		
		void startEditor( )
		{
			glutMainLoop();	
		}
		
		void display(void)
		{
			mainWin.display();
		}
		
		void clean( )
		{
			mainWin.clean( );
		}
		
		void reshapeWindow( int w, int h )
		{
			mainWin.reshapeWindow( w, h );
		}
		
		void workspaceDisplay(void)
		{
			mainWin.workspaceDisplay();
		}

		void workspaceMouse( int button, int state, int x, int y )
		{
			mainWin.workspaceMouse( button, state, x, y );
		}

		void workspaceMouseEnterExit( int state )
		{
			mainWin.workspaceMouseEnterExit( state );
		}

		void workspacePassiveMouseMovement( int x, int y )
		{
			mainWin.workspacePassiveMouseMovement( x, y );
		}
		
		void workspaceActiveMouseMovement( int x, int y )
		{
			mainWin.workspaceActiveMouseMovement( x, y );
		}

		void menuDisplay(void)
		{
			mainWin.menuDisplay();
		}

		void menuMouse( int button, int state, int x, int y )
		{
			mainWin.menuMouse( button, state, x, y );
		}

		void menuMouseEnterExit( int state )
		{
			mainWin.menuMouseEnterExit( state );
		}
		
};

void mainDisplayWrapper(void)
{
	objectToCall->display();
}

void mainReshapeWindowWrapper( int w, int h )
{
	objectToCall->reshapeWindow( w, h );
}

void workspaceDisplayWrapper(void)
{
	objectToCall->workspaceDisplay();
}

void workspaceMouseWrapper( int button, int state, int x, int y )
{
	objectToCall->workspaceMouse( button, state, x, y );
}

void workspaceMouseEnterExitWrapper( int state )
{
	objectToCall->workspaceMouseEnterExit( state );
}

void workspacePassiveMouseMovementWrapper( int x, int y )
{
	objectToCall->workspacePassiveMouseMovement( x, y );
}
void workspaceActiveMouseMovementWrapper( int x, int y )
{
	objectToCall->workspaceActiveMouseMovement( x, y );
}

void menuDisplayWrapper(void)
{
	objectToCall->menuDisplay();
}

void menuMouseWrapper( int button, int state, int x, int y )
{
	objectToCall->menuMouse( button, state, x, y );
}

void menuMouseEnterExitWrapper( int state )
{
	objectToCall->menuMouseEnterExit( state );
}

void cleanup(void)
{
	std::cout<<"in cleanup"<<std::endl;
	objectToCall->clean( );
	
}

/*
void keyboardWrapper(unsigned char key, int x, int y)
{
	objectToCall->keyboard( key, x, y );
}

void mouseWrapper( int button, int state, int x, int y )
{
	objectToCall->mouse( button, state, x, y );
}

void mouseEnterExitWrapper( int state )
{
	objectToCall->mouseEnterExit( state );
}

void passiveMouseMovementWrapper( int x, int y )
{
	objectToCall->passiveMouseMovement( x, y );
}
*/



#endif

