#ifndef POINTWRAPPER_H
#define POINTWRAPPER_H

#include <gl.h>
#include <glut.h>

#include <vector>
#include <string>
#include <sstream>
//slob includes
#include "point2D.h"
#include <occi.h>
#include "constants.h"

#include "spatialObjectWrapper.h"

class pointWrapper : public spatialObjectWrapper
{
	protected:
		point2D *pPtr;
		vector<poi2D> vec;
		bool reReadDb;
		
		// if constructed from a pointer, don't free the memory
		bool needToDelete;
		
	public:
		pointWrapper(  OCIEnv *envHP, OCIError *errHP, OCISession *authP, OCIServer *srvHP, OCISvcCtx *svcHP, uint pk, bool useDefaults = true ): spatialObjectWrapper( envHP, errHP, authP, srvHP, svcHP, pk ), pPtr( NULL ), reReadDb( true ), needToDelete( true )
		{
			if( !useDefaults )
				changeDatabaseSelectStatement( pk );
			pPtr = new point2D( readFromDB( pk ) );
			pPtr->validate();
			cout<<*pPtr<<endl;
		}
		
		pointWrapper(  OCIEnv *envHP, OCIError *errHP, OCISession *authP, OCIServer *srvHP, OCISvcCtx *svcHP,  bool useDefaults = true ): spatialObjectWrapper( envHP, errHP, authP, srvHP, svcHP, 0 ), pPtr( NULL ), reReadDb( true ), needToDelete( true )
		{
			if( !useDefaults )
				changeDatabaseSelectStatement(  );
			pPtr = new point2D( readFromDB(  ) );
			pPtr->validate();
			cout<<*pPtr<<endl;
		}
		
		pointWrapper( point2D *pointer ):  pPtr( pointer ), reReadDb( true ), needToDelete( false )
		{
			pPtr->validate();
			cout<<*pPtr<<endl;
		}
		
		~pointWrapper( )
		{
			clean( );
		}
		
		void clean( )
		{
			cout<<"cleaning point wrapper"<<endl;
			if( pm!= NULL ){ pm->commitBatch(); pm = NULL; }
			if( pPtr != NULL && needToDelete )
			{	
				delete pPtr;
				pPtr = NULL;
			}
			spatialObjectWrapper::clean( );
		}
		
		bool isPoint( ){ return true; }
		
		virtual void display( bool showCoords, uint color )
		{

			if( reReadDb )
			{
				//re read  points from the DB
				copyPointData(  );
				reReadDb = false;
			}
			
			if( color == COLOR_GREEN )
				glColor3f(0.0, 1.0, 0.0);
			else if( color == COLOR_BLUE )
				glColor3f(0.0, 0.0, 1.0);
			else if( color == COLOR_RED )
				glColor3f(1.0, 0.0, 0.0);
			
			glPointSize(8.0);
			
			glBegin(GL_POINTS);
			for( uint i = 0; i < vec.size(); i++ )
			{
				glVertex3f( vec[i].getX().toDouble(), vec[i].getY().toDouble(), 0.0 );
			}
			glEnd();
				
			if( showCoords )
			{
				
				float fx, fy;
				int ix, iy;
				string outputStr;
				
				for( uint i = 0; i < vec.size(); i++ )
				{
					fx = vec[i].getX().toDouble();
					fy = vec[i].getY().toDouble();
					ix = static_cast<int>(fx);
					iy = static_cast<int>(fy);
						
					glColor3f(0.0, 0.0, 0.0);
					ostringstream oss;
					oss<<'('<<fx<<','<<fy<<')';
					outputStr.assign( oss.str() );
					glColor3f(0.0, 0.0, 0.0);
					bitmap_output(ix,iy,const_cast<char*>( outputStr.c_str() ), GLUT_BITMAP_HELVETICA_10);
					glColor3f(0.0, 1.0, 0.0);
				}
			}
		}
		
		point2D* getPtr( )
		{
			return pPtr;
		}
		
		void copyPointData(  )
		{
			vec.clear();
			point2D::Iterator pIter= pPtr->getIterator();
			while (pIter.next())
			{
				vec.push_back( *pIter.get() );
			}
		}
		
		//draws text on the screen
		void bitmap_output(int x, int y, char *string, void *font)
		{  
			int len, i;
			glRasterPos3f(x, y, 0);            // Locate Raster Position in 3-space
			len = (int) strlen(string);        // Find length of string
			for (i = 0; i < len; i++)
			{
				// Loop through plotting all characters in font style
				glutBitmapCharacter(font, string[i]);
			}
		}
		
		point2D::Iterator getIterator( )
		{
			return pPtr->getIterator( );
		}
		
		void insertPoint( int plotX, int plotY )
		{
			rat x( static_cast<int>( plotX ) );
			rat y( static_cast<int>( plotY ) );
			poi2D p( x, y );
			pPtr->addPoint( p );
			reReadDb=true;
		}
		
		void removePoint( int plotX, int plotY )
		{
			rat x( plotX );
			rat y( plotY );
			poi2D poiToRemove( x, y );
			
			poi2D poi;
			rat tmpX, tmpY;
			rat zero( 0 );
			rat negOne( -1 );
			rat buffer( 2 );
			point2D::Iterator pIter= pPtr->getIterator();
			while (pIter.next())
			{
				poi=*pIter.get();
				tmpX = poi.getX( ) - x;
				if( tmpX < zero ) tmpX *= negOne;
				tmpY = poi.getY( ) - y;
				if( tmpY < zero ) tmpY *= negOne;
				if( tmpX < buffer && tmpY < buffer )
				{
					pPtr->removePoint( poi );
					reReadDb=true;
					return;
				}
			}
		}
		
		
		
};
#endif
