#ifndef REGIONWRAPPER_H
#define REGIONWRAPPER_H

#include <gl.h>
#include <glut.h>

#include <vector>
#include <string>
#include <sstream>
//slob includes
#include "poly2D.h"
#include "face2D.h"
#include "region2D.h"
#include <occi.h>
#include "constants.h"

#include "spatialObjectWrapper.h"

class regionWrapper: public spatialObjectWrapper
{
	private:
		struct cycleInfo
		{
			bool isOuterCycle;
			uint faceIndexInRegion;
			uint cycleIndexInFace;
			cycleInfo( ): isOuterCycle( true ), faceIndexInRegion( 0 ), cycleIndexInFace( 0 )
			{	}
			cycleInfo( bool isouter, uint faceidx, uint cycleidx ): 
					isOuterCycle( isouter ), faceIndexInRegion( faceidx ),
				cycleIndexInFace( cycleidx )
			{	}
			cycleInfo& operator=( cycleInfo& RHS )
			{
				isOuterCycle = RHS.isOuterCycle;
				faceIndexInRegion = RHS.faceIndexInRegion;
				cycleIndexInFace = RHS.cycleIndexInFace;
				return *this;
			}
			bool operator==( cycleInfo& RHS )
			{
				return( isOuterCycle == RHS.isOuterCycle && faceIndexInRegion == RHS.faceIndexInRegion && cycleIndexInFace == RHS.cycleIndexInFace );
			}
			void debug()
			{
				cerr<<"isouter, faceIdx, cycleIdx: "<<isOuterCycle<<" "<<faceIndexInRegion<<" "<<cycleIndexInFace<<endl;
			}
		};
		
		region2D *rPtr;
		vector<vector<hseg2D> > vec;
		vector<cycleInfo> infoVec;
		vector<seg2D> insertedSegVec;
		
		// if constructed from a pointer, don't free the memory
		bool needToDelete;
		bool reReadDb;
		
		//needed to insert Face
		poi2D prevFaceEndPoint;
		poi2D newFaceEndPoint;
		bool addingNewFace;
		vector<seg2D> insertedFaceSegVec;
		
		//neede to insert holes
		cycleInfo selectedFace;
		poi2D prevHoleEndPoint;
		poi2D newHoleEndPoint;
		bool addingNewHole;
		bool selectedHoleOwner;
		vector<seg2D> insertedHoleSegVec;
		
	public:
		regionWrapper(  OCIEnv *envHP, OCIError *errHP, OCISession *authP, OCIServer *srvHP, OCISvcCtx *svcHP, uint pk, bool useDefaults = true ): spatialObjectWrapper( envHP, errHP, authP, srvHP, svcHP, pk ), rPtr( NULL ), needToDelete( true ), reReadDb( true ),  addingNewFace( false ), addingNewHole( false ), selectedHoleOwner( false )
		{
			if( !useDefaults )
				changeDatabaseSelectStatement( pk );
			rPtr = new region2D( readFromDB( pk ) );
			rPtr->validate();
			cout<<*rPtr<<endl;
		}
		
		regionWrapper(  OCIEnv *envHP, OCIError *errHP, OCISession *authP, OCIServer *srvHP, OCISvcCtx *svcHP,  bool useDefaults = true ): spatialObjectWrapper( envHP, errHP, authP, srvHP, svcHP, 0 ), rPtr( NULL ),  needToDelete( true ), reReadDb( true ), addingNewFace( false ), addingNewHole( false ), selectedHoleOwner( false )
		{
			if( !useDefaults )
				changeDatabaseSelectStatement(  );
			rPtr = new region2D( readFromDB( ) );
			rPtr->validate();
			cout<<*rPtr<<endl;
		}
		
		regionWrapper( region2D *pointer ):  rPtr( pointer ), needToDelete( false ), reReadDb( true ), addingNewFace( false ), addingNewHole( false ), selectedHoleOwner( false )
		{
			rPtr->validate();
			cout<<*rPtr<<endl;
		}
		
		~regionWrapper( )
		{
			clean( );
		}
		
		void clean( )
		{
			cout<<"cleaning region wrapper"<<endl;
			if( pm!= NULL ){ pm->commitBatch(); pm = NULL; }
			if( rPtr != NULL && needToDelete )
			{	
				delete rPtr;
				rPtr = NULL;
			}
			spatialObjectWrapper::clean( );
		}
		
		bool isRegion( ){ return true; }
		
		virtual void display( bool showCoords, uint color )
		{

			if( reReadDb )
			{
				//re read lines from the DB
				copyRegionData(  );
				reReadDb = false;
			}
			
			if( color == COLOR_GREEN )
				glColor4f(0.0, 1.0, 0.0, 0.5);
			else if( color == COLOR_BLUE )
				glColor4f(0.0, 0.0, 1.0, 0.5);
			else if( color == COLOR_RED )
				glColor4f(1.0, 0.0, 0.0, 0.5);
				
			//glClear (GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
// 			glEnable(GL_DEPTH_TEST);
// 			glClearDepth(1.0);
// 			glDepthFunc(GL_LEQUAL);
// 			glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
// 			glEnable(GL_BLEND);
// 			glAlphaFunc(GL_GREATER,0.1);
// 			glEnable(GL_ALPHA_TEST);
// 			glEnable(GL_TEXTURE_2D);
// 			glEnable(GL_CULL_FACE);
			
			for( uint i = 0; i < vec.size(); i++ )
			{
				glBegin(GL_LINES);
				for( uint j = 0 ; j < vec[i].size(); j++ )
				{
					if( vec[i][j].isLeft( ) )
					{
						glVertex3f( (vec[i])[j].getDominating().getX().toDouble(), (vec[i])[j].getDominating().getY().toDouble(), 0.0 );
						glVertex3f( (vec[i])[j].getSubmissive().getX().toDouble(), 	(vec[i])[j].getSubmissive().getY().toDouble(), 0.0 );	
					}
				}
				glEnd();
			}
			
			glBegin(GL_LINES);
			for( uint i = 0; i < insertedFaceSegVec.size(); i++ )
			{
				glVertex3f( insertedFaceSegVec[i].getP().getX().toDouble(), insertedFaceSegVec[i].getP().getY().toDouble(), 0.0 );
				glVertex3f( insertedFaceSegVec[i].getQ().getX().toDouble(), insertedFaceSegVec[i].getQ().getY().toDouble(), 0.0 );
			}
			glEnd();
				
			// show segment from the last inserted segment to the current mouse pos
			if( addingNewFace )
			{
				glBegin(GL_LINES);
				glVertex3f( prevFaceEndPoint.getX().toDouble(),prevFaceEndPoint.getY().toDouble(), 0.0 );
				glVertex3f( newFaceEndPoint.getX().toDouble(), newFaceEndPoint.getY().toDouble(), 0.0 );
				glEnd();
			}
			
			glBegin(GL_LINES);
			for( uint i = 0; i < insertedHoleSegVec.size(); i++ )
			{
				glVertex3f( insertedHoleSegVec[i].getP().getX().toDouble(), insertedHoleSegVec[i].getP().getY().toDouble(), 0.0 );
				glVertex3f( insertedHoleSegVec[i].getQ().getX().toDouble(), insertedHoleSegVec[i].getQ().getY().toDouble(), 0.0 );
			}
			glEnd();
				
			// show segment from the last inserted segment to the current mouse pos
			if( addingNewHole )
			{
				glBegin(GL_LINES);
				glVertex3f( prevHoleEndPoint.getX().toDouble(),prevHoleEndPoint.getY().toDouble(), 0.0 );
				glVertex3f( newHoleEndPoint.getX().toDouble(), newHoleEndPoint.getY().toDouble(), 0.0 );
				glEnd();
			}
			
			if( selectedHoleOwner )
			{
				glLineWidth(3.0);
				uint i = 0;
				for( uint k = 0; k < infoVec.size(); k++ )
					if( infoVec[k] == selectedFace )
						i = k;
				
				glBegin(GL_LINES);
					for( uint j=0; j < vec[i].size( ); j++ )
					{
						glVertex3f( (vec[i])[j].getDominating().getX().toDouble(), (vec[i])[j].getDominating().getY().toDouble(), 0.0 );
						glVertex3f( (vec[i])[j].getSubmissive().getX().toDouble(), (vec[i])[j].getSubmissive().getY().toDouble(), 0.0 );
					}
				glEnd();
				glLineWidth(1.0);
			}
			
			// draw the vertices
			glColor3f(0.0, 0.0, 0.0);
			glPointSize(3.0);
			for( uint i = 0; i < vec.size(); i++ )
			{
				glBegin(GL_POINTS);
				for( uint j = 0 ; j < vec[i].size(); j++ )
				{
					if( vec[i][j].isLeft( ) )
					{
						glVertex3f( (vec[i])[j].getDominating().getX().toDouble(), (vec[i])[j].getDominating().getY().toDouble(), 0.0 );
						glVertex3f( (vec[i])[j].getSubmissive().getX().toDouble(), 	(vec[i])[j].getSubmissive().getY().toDouble(), 0.0 );	
					}
				}
				glEnd();
			}
			
			if( showCoords )
			{
				
				float fx, fy;
				int ix, iy;
				string outputStr;
				
				for( uint i = 0; i < vec.size(); i++ )
				{
					for( uint j = 0; j < vec[i].size( ); j++ )
					{
						fx = (vec[i])[j].getDominating().getX().toDouble();
						fy = (vec[i])[j].getDominating().getY().toDouble();
						ix = static_cast<int>(fx);
						iy = static_cast<int>(fy);
							
						glColor3f(0.0, 0.0, 0.0);
						ostringstream oss;
						oss<<'('<<fx<<','<<fy<<')';
						outputStr.assign( oss.str() );
						glColor3f(0.0, 0.0, 0.0);
						bitmap_output(ix,iy,const_cast<char*>( outputStr.c_str() ), GLUT_BITMAP_HELVETICA_10);
						glColor3f(0.0, 1.0, 0.0);
					}
				}
			}
		}	
		
		region2D* getPtr( )
		{
			return rPtr;
		}
		
		void copyRegionData(  )
		{
			vec.clear();
			infoVec.clear();
					
			poly2D::Iterator pIter;
			face2D::PolyIterator fIter;
			face2D currFace;
			uint faceCount = rPtr->getFaceCount( );
			
			for( uint i = 0; i < faceCount; i++ )
			{
				currFace = rPtr->getFace( i );
				fIter = currFace.getPolyIterator( );
				uint counter = 0;
				while( fIter.next( ) )
				{
					if( (fIter.get())->isOuterCycle() )
					{
						infoVec.push_back( cycleInfo( true, i, counter ) );
						infoVec[ infoVec.size() - 1 ].debug(); 
					}
					else
					{
						infoVec.push_back( cycleInfo( false, i, counter ) ); 
						infoVec[ infoVec.size() - 1 ].debug();
					}
					
					pIter = (fIter.get())->getIterator( );
					vector<hseg2D> tmpVec;
					while( pIter.next( ) )
					{
						tmpVec.push_back( *(pIter.get( )) );
					}
					vec.push_back( tmpVec );
					counter++;
				}
			}
		}
		

		void updateNewEndpoint( int plotX, int plotY )
		{	
			updateNewFaceEndpoint( plotX, plotY );
			updateNewHoleEndpoint( plotX, plotY );
		}
		
		void updateNewFaceEndpoint( int plotX, int plotY )
		{
			poi2D temp( plotX, plotY );
			newFaceEndPoint = temp;
		}
		
		void updateNewHoleEndpoint( int plotX, int plotY )
		{
			poi2D temp( plotX, plotY );
			newHoleEndPoint = temp;
		}
		
		
		void insertFaceSeg( int plotX, int plotY )
		{
			// needs to keep a buffer of the new block to add
			// then add the block when finished

			updateNewFaceEndpoint( plotX, plotY );
			
			if( insertedFaceSegVec.size( ) == 0 && !addingNewFace )
			{
				addingNewFace = true;
				//first new point inserted
				prevFaceEndPoint = newFaceEndPoint;
				return;
			}
			else 
			{
				insertedFaceSegVec.push_back( seg2D( prevFaceEndPoint, newFaceEndPoint ) );
				prevFaceEndPoint = newFaceEndPoint;
				return;
			}
			
		}
		
		void insertHoleSeg( int plotX, int plotY )
		{
			// needs to keep a buffer of the new block to add
			// then add the block when finished

			updateNewHoleEndpoint( plotX, plotY );
			
			if( insertedHoleSegVec.size( ) == 0 && !selectedHoleOwner )
			{
				//find the face the user has selected
				selectedHoleOwner = getSelectedCycleID( newHoleEndPoint, false, selectedFace );
				return;
			}
			else if( insertedHoleSegVec.size( ) == 0 && !addingNewHole )
			{
				addingNewHole = true;
				//first new point inserted
				prevHoleEndPoint = newHoleEndPoint;
				return;
			}
			else 
			{
				insertedHoleSegVec.push_back( seg2D( prevHoleEndPoint, newHoleEndPoint ) );
				prevHoleEndPoint = newHoleEndPoint;
				return;
			}
			
		}
		
		void commitChanges( )
		{
			if( addingNewFace )
			{
				addingNewFace = false;
				uint arraySize = insertedFaceSegVec.size( );
				
				
				seg2D segArray[ arraySize ];
				for( uint i = 0; i < arraySize; i++ )
					segArray[i] = insertedFaceSegVec[i];
				face2D newFace = rPtr->addFace( );
				poly2D newPoly = newFace.addOuterCycle();
				newPoly.addSegments( segArray, arraySize );
				newPoly.validate();
				newFace.validate();
				insertedFaceSegVec.clear( );
			}
			else 
			{
				if(  selectedHoleOwner )
					selectedHoleOwner = false;
				if( addingNewHole )
				{
					addingNewHole = false;
					uint arraySize = insertedHoleSegVec.size( );
					seg2D segArray[ arraySize ];
					for( uint i = 0; i < arraySize; i++ )
						segArray[i] = insertedHoleSegVec[i];
					face2D editFace = rPtr->getFace( selectedFace.faceIndexInRegion );
					poly2D newPoly = editFace.addHole( );
					newPoly.addSegments( segArray, arraySize );
					newPoly.validate();
					editFace.validate();
					insertedHoleSegVec.clear( );
				}
			}
			reReadDb=true;
		}
		
		void removeHole( int plotX, int plotY )
		{
			rat x( plotX );
			rat y( plotY );
			poi2D poiToRemove( x, y );
			
			cycleInfo holeToRemove;
			if( getSelectedCycleID( poiToRemove, true, holeToRemove ) )
			{
				cerr<<"removing :";  holeToRemove.debug(); cerr<<endl;
				// added +1 on line below because need to get past the flags
				rPtr->getFace(holeToRemove.faceIndexInRegion).removeHole( holeToRemove.cycleIndexInFace );
				reReadDb = true;
			}
		}
		
		void removeFace( int plotX, int plotY )
		{
			rat x( plotX );
			rat y( plotY );
			poi2D poiToRemove( x, y );
			
			cycleInfo faceToRemove;
			if( getSelectedCycleID( poiToRemove, false, faceToRemove ) )
			{
				rPtr->removeFace(faceToRemove.faceIndexInRegion);
				reReadDb = true;
			}
		}


		
		bool getSelectedCycleID( poi2D selectPoint, bool lookForHole, cycleInfo & selectedCycleInfo )
		{
			rat tmpDist;
			rat zero( 0 );
			rat negOne( -1 );
			rat buffer( 2 );
			seg2D currSeg;
			poi2D P, Q;
			for( uint i = 0; i < vec.size( ); i++ )
			{
				if( lookForHole == !(infoVec[i].isOuterCycle) )
				{
					for( uint j = 0; j < vec[i].size( ); j++ )
					{
						P.setP( vec[i][j].getDominating( ) );
						Q.setP( vec[i][j].getSubmissive( ) );
						currSeg.setPQ( P, Q );
						tmpDist = currSeg.distance( selectPoint );
						//cerr<<"seg: "<<currSeg<<" dist: "<<tmpDist<<" selectPOint: "<<selectPoint<<endl;	
						if( tmpDist < buffer )
						{
							selectedCycleInfo = infoVec[i];
							return true;
						}
					}
				}
			}
			return false;
		}
		
		//draws text on the screen
		void bitmap_output(int x, int y, char *string, void *font)
		{  
			int len, i;
			glRasterPos3f(x, y, 0);            // Locate Raster Position in 3-space
			len = (int) strlen(string);        // Find length of string
			for (i = 0; i < len; i++)
			{
				// Loop through plotting all characters in font style
				glutBitmapCharacter(font, string[i]);
			}
		}
};
#endif
