#ifndef SPATIALOBJECTWRAPPER_H
#define SPATIALOBJECTWRAPPER_H

//slob includes
#include "point2D.h"
#include <occi.h>
#include "constants.h"
#include <ctype.h>

using namespace stu;


typedef struct OCILobWrapper{
	OCILobLocator* lob;
	OCISvcCtx*	cntxt;
	OCIError* errhp;
} OCILobWrapper;

void checkerr(OCIError *errhp, sword status)
{
	text errbuf[512];
	//ub4 buflen;
	sb4 errcode;
	string er;
	
	if (status == OCI_SUCCESS) return;

	switch (status)
	{
		case OCI_SUCCESS_WITH_INFO:
			cout << "Error - OCI_SUCCESS_WITH_INFO" << endl;
			OCIErrorGet ((dvoid *) errhp, (ub4) 1, (text *) NULL, &errcode,
						  errbuf, (ub4) sizeof(errbuf), (ub4) OCI_HTYPE_ERROR);
			er=(char*)errbuf;
			cout << "Error -" << er << endl;
			break;
		case OCI_NEED_DATA:
			cout << "Error - OCI_NEED_DATA" << endl;
			break;
		case OCI_NO_DATA:
			cout << "Error - OCI_NO_DATA" << endl;
			break;
		case OCI_ERROR:
			OCIErrorGet ((dvoid *) errhp, (ub4) 1, (text *) NULL, &errcode,
						  errbuf, (ub4) sizeof(errbuf), (ub4) OCI_HTYPE_ERROR);
			er=(char*)errbuf;
			cout << "Error -" << er << endl;
			break;
		case OCI_INVALID_HANDLE:
			cout << "Error - OCI_INVALID_HANDLE" << endl;
			break;
		case OCI_STILL_EXECUTING:
			cout << "Error - OCI_STILL_EXECUTING" << endl;
			break;
		case OCI_CONTINUE:
			cout << "Error - OCI_CONTINUE" << endl;
			break;
		default:
			cout << "Error - " << status << endl;
			break;
	}
}

class spatialObjectWrapper
{
	protected:
		uint primaryKey;
		
		//oracle connection shizzle
		OCIEnv *envhp;
		OCIError *errhp;
		OCISession *authp;
		OCIServer *srvhp;
		OCISvcCtx *svchp;
		OCIStmt	*stmthp;
		OCILobLocator *mylob;
		OCIDefine *defnp;
		sword status;
		
		mSlob *pm;
		string sqlQuery;
		string forUpdate;
		string defaultQuery;
		string defaultColumn;
		string defaultTable;
		string defaultKey;
		string column;
		string table;
		string key;
		bool cleaned;
		
	public:
		spatialObjectWrapper( OCIEnv *envHP, OCIError *errHP, OCISession *authP, OCIServer *srvHP, OCISvcCtx *svcHP, uint pk ): primaryKey( pk ), envhp( envHP ), errhp( errHP ), authp( authP ), srvhp( srvHP ), svchp( svcHP ), stmthp( NULL ), mylob( NULL ), defnp( (OCIDefine *) 0 ), pm( NULL ), forUpdate( " FOR UPDATE" ), defaultQuery( "SELECT data FROM slobber where id=" ),  defaultColumn( "data" ), defaultTable( "slobber" ), defaultKey( "id" ), column( defaultColumn ), table( defaultTable ), key( defaultKey ), cleaned( false )
		{	
			string lobID;
			ostringstream oss;
			oss<< primaryKey;
			lobID.assign( oss.str() );
			sqlQuery.assign( defaultQuery );
		}
		
		spatialObjectWrapper( ): primaryKey( 0 ), envhp( NULL ), errhp( NULL ), authp( NULL ), srvhp( NULL ), svchp( NULL ), stmthp( NULL ), mylob( NULL ), defnp( (OCIDefine *) 0 ), pm( NULL ), forUpdate( " FOR UPDATE" ), defaultQuery( "SELECT data FROM slobber where id=" ),  defaultColumn( "data" ), defaultTable( "slobber" ), defaultKey( "id" ), column( defaultColumn ), table( defaultTable ), key( defaultKey ), cleaned( false )
		{	
			string lobID;
			ostringstream oss;
			oss<< 0;
			lobID.assign( oss.str() );
			sqlQuery.assign( defaultQuery );
		}
		
		//, DBMS_LOB.getlength(data)
		
		virtual ~spatialObjectWrapper( )
		{
			clean( );
		}
		
		virtual void updateNewEndpoint( int plotX, int plotY )
		{	}
		virtual void commitChanges( )
		{	}
		
		virtual void changeDatabaseSelectStatement( uint pk =0 )
		{
			sqlQuery.clear( );
			string PK;
			
			// clear the cin buffer
			//cin.seekg (0, ios::beg);
			//std::cin.clear();
			//cin.ignore( std::numeric_limits<std::streamsize>::max(), '\0' );
			
			cout<<"Enter table name holding the mSlob -+> ";
			getline( cin, table );
			cout<<"Enter the mSlob column name -+> ";
			getline( cin, column );
			cout<<"Enter the name of the primary key column -+> ";
			getline( cin, key );
			
			if( table == "" ) table.assign( defaultTable );
			if( column == "" ) column.assign( defaultColumn );
			if( key == "" ) key.assign( defaultKey );
			
			if( pk == 0 )
			{
				bool validPK = false;
				while( !validPK )
				{
					validPK = true;
					cout<<"Enter the primary key of the desired mSlob -+> ";
					getline(cin,PK);
					if( PK == "" ) 
					{
						validPK = false;
						continue;
					}
					for( uint i = 0; i < PK.size(); i++ )
					{
						if( !isdigit( PK[i] ) )
						{
							cerr<<"Invalid primary key input...must be an integer"<<endl;
							validPK = false;
							break;
						}
					}
					
					if( validPK )
						primaryKey = atoi( PK.c_str( ) );
				}
			}
		
			string lobID;
			ostringstream oss;
			oss<< primaryKey;
			lobID.assign( oss.str() );
			
			sqlQuery.assign( "SELECT " + column + " FROM " + table + " WHERE " + key + "=" );
			
		}
		
		virtual Locator readFromDB( uint pk = 0)
		{
			try
			{
				//clean up any old query
				clean( );
				cleaned = false;
					// assign the primary key
				if( pk != 0 ){ primaryKey = pk; }
					
				//make the query
				string lobID;
				ostringstream oss;
				oss<< primaryKey;
				lobID.assign( oss.str() );
				
				while( true )
				{
					checkerr(errhp, OCIHandleAlloc( (dvoid *) envhp, (dvoid **) &stmthp,
							OCI_HTYPE_STMT, (size_t) 0, (dvoid **) 0));
						
						//allocate lob descriptor
					if (OCIDescriptorAlloc((dvoid *) envhp, (dvoid **) &mylob,
						(ub4)OCI_DTYPE_LOB, (size_t) 0, (dvoid **) 0))
					{
						cout << "FAILED: OCIDescriptorAlloc()" << endl;
						return Locator( );
					}
					
					//issue SQL and process data
					string finalQuery = sqlQuery+ lobID + forUpdate;
					checkerr(errhp, OCIStmtPrepare(stmthp, errhp, (text *)(finalQuery.c_str()),
							(ub4)strlen(finalQuery.c_str()),
							(ub4) OCI_NTV_SYNTAX, (ub4) OCI_DEFAULT));	
			
					
	// 				checkerr(errhp, OCIDefineByPos(stmthp, &defnp, errhp, 1, (dvoid *) &did,
	// 						(sword) sizeof(int), SQLT_INT, (dvoid *) 0, (ub2 *)0,
	// 						(ub2 *)0, OCI_DEFAULT));
		
			
					checkerr(errhp, OCIDefineByPos(stmthp, &defnp, errhp, (ub4) 1,
							(dvoid *) &mylob, (sb4) -1, (ub2) SQLT_BLOB,
							(dvoid *) 0, (ub2 *) 0, (ub2 *) 0, (ub4) OCI_DEFAULT));
						
					/* execute and fetch */
					if (status = OCIStmtExecute(svchp, stmthp, errhp, (ub4) 1, (ub4) 0,
						(CONST OCISnapshot *) NULL, (OCISnapshot *) NULL, OCI_DEFAULT))
					{
						if (status == OCI_NO_DATA)
						{
							// No data returned, so lets create the tuple;
							string sqlInsertQuery;
							sqlInsertQuery.assign( "INSERT INTO "+table+" VALUES( "+lobID+" , EMPTY_BLOB())");
							
							//= "insert into slobber values( " + id + ", EMPTY_BLOB())";
								
							checkerr(errhp, OCIStmtPrepare(stmthp, errhp, (text *)(sqlInsertQuery.c_str()),
									(ub4)strlen(sqlInsertQuery.c_str()),
									(ub4) OCI_NTV_SYNTAX, (ub4) OCI_DEFAULT));	
	
							checkerr(errhp, OCIStmtExecute(svchp, stmthp, errhp, (ub4) 1, (ub4) 0,
									(CONST OCISnapshot *) NULL, (OCISnapshot *) NULL, OCI_DEFAULT));
							cout << "Insert query: " << sqlInsertQuery << endl;
								
							if (stmthp)
							{
								checkerr(errhp, OCIHandleFree(stmthp,OCI_HTYPE_STMT));
								stmthp = NULL;
							}
						}
						else
						{
							checkerr(errhp, status);
							if (envhp)
								(void) OCIHandleFree((dvoid *) envhp, OCI_HTYPE_ENV);
							return Locator( );
						}
					}//else success then processs
					else{ break; }
				}
				if (status==OCI_SUCCESS || status==OCI_SUCCESS_WITH_INFO)
				{
						
						//cout << "The COUNT: " << points_getCount(blob1) << endl;
						
					OCILobWrapper *blob1=new OCILobWrapper;
					blob1->lob=mylob;
					blob1->cntxt=svchp;
					blob1->errhp=errhp;
						
					pm = new mSlob(blob1);
						
					//cout << "mSLOB Built from point with id " << did << endl;
						
					Locator daddy(pm->locateGlobal());
					if (daddy.getElements()==0) //then build an object node
						daddy.insert(0,OBJECT_LEVEL);
					return daddy.locate(0);
					
	// 				Locator points(daddy.locate(0));
	// 				points.debug();
	// 				point2D ps(points);
	// 					
	// 				cout << "The points: " << ps << endl;
	// 					
	// 					//ps.elements.myslob->sinx->debug();
	// 					
				}else
					cout << "No point with id=" << primaryKey << " found." << endl;
						
						//free the statement handle	
				checkerr(errhp, OCIHandleFree(stmthp,OCI_HTYPE_STMT));
				stmthp = NULL;
				
			}
			catch (exception &excp)
			{
				cerr << excp.what() << endl;
				cout << "Thanks...sorry we ended this on such a bad note." << endl;
				cout << "Closing connection" << endl;
			
				if ( stmthp != NULL )
					checkerr(errhp, OCIHandleFree(stmthp,OCI_HTYPE_STMT));
			}
			
			/**** old DB read code ******
			try {
				//clean up any old query
				clean( );
				cleaned = false;
				// assign the primary key
				if( pk != 0 ){ primaryKey = pk; }
				
				//make the query
				string lobID;
				ostringstream oss;
				oss<< primaryKey;
				lobID.assign( oss.str() );
				
				stmt = conn->createStatement ( sqlQuery+ lobID + forUpdate );
				rset = stmt->executeQuery( );
				blob = new Blob();
				
				if (rset->next ())
				{
					//uint did= rset->getUInt(1);
					*blob = rset->getBlob (1);
				}else{
					cerr << "Point not found!:"<<primaryKey << endl;
					spatialObjectWrapper::clean( );
				}
				
				pm = new mSlob( blob );
				pm->beginBatch();
				Locator points(pm->locateGlobalmSlob(0));
				points.debug();
				return points;
			}
			catch (exception &excp)
			{
				cout << "Thanks...sorry we ended this on such a bad note." << endl;
				cout << "Closing connection" << endl;
				env->terminateConnection (conn);
				cerr << "hola "<<excp.what() << endl;
			}*/
			
			//return a null locator
			return Locator();
		}
		
		virtual bool isPoint( ){ return false; }
		virtual bool isLine( ){ return false; }
		virtual bool isRegion( ){ return false; }
		virtual void display( bool showCoords, uint color )
		{	cerr<<"shouldnotbehere"<<endl;} 
		
		virtual void clean( )
		{
			if( ! cleaned )
			{
				cleaned = true;
				cout<<"cleaning wrapper"<<endl;
				if( pm!= NULL ){ pm->commitBatch(); pm = NULL; }
				
				if( stmthp)
				{
					checkerr(errhp, OCIHandleFree(stmthp,OCI_HTYPE_STMT));
				}	
				if (mylob != NULL )
				{
					checkerr(errhp, OCILobClose(svchp, errhp, mylob));
					//delete mylob;
					mylob = NULL;
				}
				
				
			}
			
		}
};
#endif
